@@ -55,8 +55,8 @@ class LircdParser:
                 break
             a = line.split(maxsplit=2)
             if a[0] != 'begin' or a[1] != 'remote':
-                self.error("expected 'begin remote', got '{} {}'".format(a[0], a[1]))
-                return None
+                self.warning("expected 'begin remote', got '{} {}'".format(a[0], a[1]))
+                continue
             if len(a) > 2 and a[2][0] != '#':
                 self.error("unexpected {}".format(a[2]))
                 return None
@@ -145,20 +145,29 @@ class LircdParser:
 
             a = line.split()
             if a[0] == 'name':
-                if len(codes) > 0:
-                    raw_codes.append({ 'name': name, 'codes': codes })
+                codes = raw_codes_sanitise(codes)
+                if codes:
+                    raw_codes.append({ 'keycode': name, 'raw': codes })
                 name = line.split(maxsplit=2)[1]
                 if not name.startswith('KEY_'):
                     name = 'KEY_' + name.upper()
                 codes = []
             elif a[0] == 'end':
-                if len(codes) > 0:
-                    raw_codes.append({ 'name': name, 'codes': codes })
+                codes = raw_codes_sanitise(codes)
+                if codes:
+                    raw_codes.append({ 'keycode': name, 'raw': codes })
                 return raw_codes
             else:
                 for v in a:
                     codes.append(int(v))
 
+def raw_codes_sanitise(codes):
+    if len(codes) == 0:
+        return None
+    if len(codes) % 2 == 0:
+        return codes[:-1]
+
+    return codes
 
 def eq_margin(duration, expected, margin):
     if duration >= (expected - margin) and duration <= (expected + margin):
@@ -218,10 +227,14 @@ class Converter:
 
         if 'rc5' in flags or 'shift_enc' in flags:
             return self.convert_rc5()
+        elif 'rc6' in flags:
+            return self.convert_rc6()
         elif 'rcmm' in flags:
             return self.convert_rcmm()
         elif 'space_enc' in flags:
             return self.convert_space_enc()
+        elif 'raw_codes' in flags:
+            return self.convert_raw_codes()
         else:
             self.error('Cannot convert remote with flags: {}'.format('|'.join(flags)))
 
@@ -235,7 +248,7 @@ class Converter:
         res  = {
             'protocol': 'pulse_distance',
             'params': {},
-            'map': {}
+            'scancodes': {}
         }
 
         res['name'] = self.remote['name']
@@ -336,22 +349,58 @@ class Converter:
                 elif v != variant:
                     variant = ""
 
-                res['map'][n] = self.remote['codes'][s]
+                res['scancodes'][n] = self.remote['codes'][s]
+
+            if variant:
+                res['params']['variant'] = "'" + variant + "'"
+        elif ('header_pulse' in res['params'] and
+            'header_space' in res['params'] and
+            'reverse' not in res['params'] and
+            'trailer_pulse' in res['params'] and
+            'header_optional' not in res['params'] and
+            'pulse_distance' == res['protocol'] and
+            eq_margin(res['params']['header_pulse'], 9000, 1000) and
+            eq_margin(res['params']['header_space'], 4500, 1000) and
+            eq_margin(res['params']['bit_pulse'], 560, 300) and
+            eq_margin(res['params']['bit_1_space'], 560, 300) and
+            eq_margin(res['params']['bit_0_space'], 1680, 300) and
+            eq_margin(res['params']['trailer_pulse'], 560, 300) and
+            res['params']['bits'] == 32 and
+            ('repeat_pulse' not in res['params'] or
+             (eq_margin(res['params']['repeat_pulse'], 9000, 1000) and
+              eq_margin(res['params']['repeat_space'], 2250, 1000)))):
+            self.warning('remote looks exactly like NEC, converting')
+            res['protocol'] = 'nec'
+            res['params'] = {}
+            # bit_0_space and bit_1_space have been swapped, scancode
+            # will need to be inverted
+
+            variant = None
+
+            for s in self.remote['codes']:
+                p = (s<<post_data_bits)|pre_data
+                v, n = decode_nec_scancode(~p)
+                if variant == None:
+                    variant = v
+                elif v != variant:
+                    variant = ""
+
+                res['scancodes'][n] = self.remote['codes'][s]
 
             if variant:
                 res['params']['variant'] = "'" + variant + "'"
         else:
             for s in self.remote['codes']:
                 p = (s<<post_data_bits)|pre_data
-                res['map'][p] = self.remote['codes'][s]
+                res['scancodes'][p] = self.remote['codes'][s]
 
         return res
 
     def convert_rcmm(self):
         res  = {
-            'protocol': 'rc_mm',
+            'protocol': 'rc-mm',
             'params': {},
-            'map': {}
+            'scancodes': {}
         }
 
         res['name'] = self.remote['name']
@@ -368,16 +417,83 @@ class Converter:
         if 'toggle_bit' in self.remote:
             toggle_bit = bits - int(self.remote['toggle_bit'][0])
 
+        if 'codes' not in self.remote or len(self.remote['codes']) == 0:
+            self.error("missing codes section")
+            return None
+
+        if 'pre_data_bits' in self.remote:
+            pre_data_bits = int(self.remote['pre_data_bits'][0])
+            pre_data = int(self.remote['pre_data'][0]) << bits
+            bits += pre_data_bits
+            for s in self.remote['codes']:
+                res['scancodes'][s|pre_data] = self.remote['codes'][s]
+        else:
+            res['scancodes'] = self.remote['codes']
+
+        res['params']['bits'] = bits
+        res['params']['variant'] = "'rc-mm-" + str(bits) + "'"
+
         if toggle_bit > 0 and toggle_bit < bits:
             res['params']['toggle_bit'] = toggle_bit
 
-        res['params']['bits'] = bits
+        return res
+
+    def convert_rc6(self):
+        res = {
+            'protocol': 'rc-6',
+            'params': { },
+            'scancodes': { }
+        }
+
+        res['name'] = self.remote['name']
 
         if 'codes' not in self.remote or len(self.remote['codes']) == 0:
             self.error("missing codes section")
             return None
 
-        res['map'] = self.remote['codes']
+        bits = int(self.remote['bits'][0])
+
+        pre_data = 0
+        if 'pre_data_bits' in self.remote:
+            pre_data_bits = int(self.remote['pre_data_bits'][0])
+            pre_data = int(self.remote['pre_data'][0]) << bits
+            bits += pre_data_bits
+
+        toggle_bit = 0
+        if 'toggle_bit_mask' in self.remote:
+            toggle_bit = ffs(int(self.remote['toggle_bit_mask'][0]))
+        if 'toggle_bit' in self.remote:
+            toggle_bit = bits - int(self.remote['toggle_bit'][0])
+
+        mask = (1<<(bits-5))-1
+        if toggle_bit >= 0 and toggle_bit < bits:
+            res['params']['toggle_bit'] = toggle_bit
+            mask &= ~(1<<toggle_bit)
+
+        # lircd explicitly encoded the five leading bits (start, 3 mode
+        # bits, toggle). rc-core does not, so we need to strip the first
+        # five bits.
+        bits -= 5
+        vendor = 0
+        res['params']['bits'] = bits
+        for s in self.remote['codes']:
+            # lircd inverts all the bits (not sure why), rc-core encoding
+            # matches https://www.sbprojects.net/knowledge/ir/rc6.php
+            d = ~(s|pre_data)&mask
+            if bits == 32:
+                vendor = d >> 16
+            res['scancodes'][d] = self.remote['codes'][s]
+
+        if bits == 16:
+            res['params']['variant'] = "'rc-6-0'"
+        elif bits == 20:
+            res['params']['variant'] = "'rc-6-6a-20'"
+        elif bits == 24:
+            res['params']['variant'] = "'rc-6-6a-24'"
+        elif bits == 32 and vendor != 0x800f:
+            res['params']['variant'] = "'rc-6-6a-32'"
+        elif bits == 32 and vendor == 0x800f:
+            res['params']['variant'] = "'rc-6-mce'"
 
         return res
 
@@ -389,7 +505,7 @@ class Converter:
         res  = {
             'protocol': 'manchester',
             'params': { },
-            'map': { }
+            'scancodes': { }
         }
 
         res['name'] = self.remote['name']
@@ -404,18 +520,18 @@ class Converter:
 
         bits = int(self.remote['bits'][0])
 
-        toggle_bit = 0
-        if 'toggle_bit_mask' in self.remote:
-            toggle_bit = ffs(int(self.remote['toggle_bit_mask'][0]))
-        if 'toggle_bit' in self.remote:
-            toggle_bit = bits - int(self.remote['toggle_bit'][0])
-
         pre_data = 0
         if 'pre_data_bits' in self.remote:
             pre_data_bits = int(self.remote['pre_data_bits'][0])
             pre_data = int(self.remote['pre_data'][0]) << bits
             bits += pre_data_bits
 
+        toggle_bit = 0
+        if 'toggle_bit_mask' in self.remote:
+            toggle_bit = ffs(int(self.remote['toggle_bit_mask'][0]))
+        if 'toggle_bit' in self.remote:
+            toggle_bit = bits - int(self.remote['toggle_bit'][0])
+
         if 'plead' in self.remote:
             plead = self.remote['plead'][0]
             one_pulse = self.remote['one'][0]
@@ -455,10 +571,20 @@ class Converter:
                 n = s|pre_data
                 n = (n & 0x3f) | ((n << 2) & 0x1f00)
                 newcodes[n] = self.remote['codes'][s]
-            res['map'] = newcodes
+            res['scancodes'] = newcodes
         else:
             for s in self.remote['codes']:
-                res['map'][s|pre_data] = self.remote['codes'][s]
+                res['scancodes'][s|pre_data] = self.remote['codes'][s]
+
+        return res
+
+    def convert_raw_codes(self):
+        res  = {
+            'protocol': 'raw',
+            'params': {},
+            'raw': self.remote['raw_codes'],
+            'name': self.remote['name']
+        }
 
         return res
 
@@ -471,16 +597,32 @@ def writeTOMLFile(fh, remote):
     print('protocol = {}'.format(escapeString(remote['protocol'])), file=fh)
     for p in remote['params']:
         print('{} = {}'.format(p, remote['params'][p]), file=fh)
-    print('[protocols.scancodes]', file=fh)
-    # find the largest scancode
-    length=1
-    for c in remote['map']:
-        length=max(length, c.bit_length())
 
-    # width seems to include '0x', hence the + 2
-    width = math.ceil(length/4) + 2
-    for c in remote['map']:
-        print('{:#0{width}x} = {}'.format(c, escapeString(remote['map'][c]), width=width), file=fh)
+    if 'scancodes' in remote:
+        print('[protocols.scancodes]', file=fh)
+        # find the largest scancode
+        length=1
+        for c in remote['scancodes']:
+            length=max(length, c.bit_length())
+
+        # width seems to include '0x', hence the + 2
+        width = math.ceil(length/4) + 2
+        for c in remote['scancodes']:
+            print('{:#0{width}x} = {}'.format(c, escapeString(remote['scancodes'][c]), width=width), file=fh)
+
+    elif 'raw' in remote:
+        for raw in remote['raw']:
+            print('[[protocols.raw]]', file=fh)
+            print('keycode = {}\nraw = \''.format(escapeString(raw['keycode'])), file=fh, end='')
+            for i, v in enumerate(raw['raw']):
+                if i == 0:
+                    print('+{}'.format(v), file=fh, end='')
+                elif i % 2 == 1:
+                    print(' -{}'.format(v), file=fh, end='')
+                else:
+                    print(' +{}'.format(v), file=fh, end='')
+
+            print('\'', file=fh)
 
     return True
 
@@ -489,7 +631,7 @@ parser = argparse.ArgumentParser(description="""Convert lircd.conf to rc-core to
 This program atempts to convert a lircd.conf remote definition to a
 ir-keytable toml format. This process is not perfect, and the result
 might need some tweaks for it to work. Please report any issues to
-linux-media@vger,kernel,org. If you have successfully generated and
+linux-media@vger.kernel.org. If you have successfully generated and
 tested a toml keymap, please send it to the same mailinglist so it
 can be include with the package.""")
 
