@@ -34,8 +34,11 @@
    In general the lower case is used to set something and the upper
    case is used to retrieve a setting. */
 enum Option {
+	OptSetAdapter = 'a',
 	OptTestAdapter = 'A',
+	OptColor = 'C',
 	OptSetDevice = 'd',
+	OptSetDriver = 'D',
 	OptExitOnFail = 'E',
 	OptHelp = 'h',
 	OptInteractive = 'i',
@@ -99,6 +102,7 @@ static int app_result;
 static int tests_total, tests_ok;
 
 bool show_info;
+bool show_colors;
 bool show_warnings = true;
 bool exit_on_fail;
 bool exit_on_warn;
@@ -108,6 +112,8 @@ time_t long_timeout = 60;
 
 static struct option long_options[] = {
 	{"device", required_argument, 0, OptSetDevice},
+	{"adapter", required_argument, 0, OptSetAdapter},
+	{"driver", required_argument, 0, OptSetDriver},
 	{"help", no_argument, 0, OptHelp},
 	{"no-warnings", no_argument, 0, OptNoWarnings},
 	{"exit-on-fail", no_argument, 0, OptExitOnFail},
@@ -116,6 +122,7 @@ static struct option long_options[] = {
 	{"timeout", required_argument, 0, OptTimeout},
 	{"trace", no_argument, 0, OptTrace},
 	{"verbose", no_argument, 0, OptVerbose},
+	{"color", required_argument, 0, OptColor},
 	{"skip-info", no_argument, 0, OptSkipInfo},
 	{"wall-clock", no_argument, 0, OptWallClock},
 	{"interactive", no_argument, 0, OptInteractive},
@@ -170,6 +177,8 @@ static void usage(void)
 	printf("Usage:\n"
 	       "  -d, --device <dev>   Use device <dev> instead of /dev/cec0\n"
 	       "                       If <dev> starts with a digit, then /dev/cec<dev> is used.\n"
+	       "  -D, --driver <driver>    Use a cec device with this driver name\n"
+	       "  -a, --adapter <adapter>  Use a cec device with this adapter name\n"
 	       "  -r, --remote [<la>]  As initiator test the remote logical address or all LAs if no LA was given\n"
 	       "  -R, --reply-threshold <timeout>\n"
 	       "                       Warn if replies take longer than this threshold (default 1000ms)\n"
@@ -205,6 +214,8 @@ static void usage(void)
 	       "\n"
 	       "  -E, --exit-on-fail Exit on the first fail.\n"
 	       "  -h, --help         Display this help message\n"
+	       "  -C, --color <when> Highlight OK/warn/fail/FAIL strings with colors\n"
+	       "                     <when> can be set to always, never, or auto (the default)\n"
 	       "  -n, --no-warnings  Turn off warning messages\n"
 	       "  -s, --skip-info    Skip Driver Info output\n"
 	       "  -T, --trace        Trace all called ioctls\n"
@@ -651,24 +662,28 @@ int cec_named_ioctl(struct node *node, const char *name,
 		warn("Both OK and MAX_RETRIES were set in tx_status! Applied workaround.\n");
 	}
 
-	if (!retval && request == CEC_TRANSMIT && show_info) {
-		printf("\t\t%s: Sequence: %u Tx Timestamp: %s Length: %u",
-		       opname.c_str(), msg->sequence, ts2s(msg->tx_ts).c_str(), msg->len);
-		if (msg->rx_ts)
-			printf("\n\t\t\tRx Timestamp: %s Approximate response time: %u ms",
-			       ts2s(msg->rx_ts).c_str(),
-			       response_time_ms(msg));
-		if (msg->tx_status & ~CEC_TX_STATUS_OK)
-			printf("\n\t\t\tStatus: %s", status2s(*msg).c_str());
-		printf("\n");
+	if (!retval && show_info &&
+	    (request == CEC_TRANSMIT || request == CEC_RECEIVE)) {
+		printf("\t\t%s: Sequence: %u Length: %u\n",
+		       opname.c_str(), msg->sequence, msg->len);
+		if (msg->tx_ts || msg->rx_ts) {
+			printf("\t\t\t");
+			if (msg->tx_ts)
+				printf("Tx Timestamp: %s ", ts2s(msg->tx_ts).c_str());
+			if (msg->rx_ts)
+				printf("Rx Timestamp: %s", ts2s(msg->rx_ts).c_str());
+			printf("\n");
+			if (msg->tx_ts && msg->rx_ts)
+				printf("\t\t\tApproximate response time: %u ms\n",
+				       response_time_ms(msg));
+		}
+		if ((msg->tx_status & ~CEC_TX_STATUS_OK) ||
+		    (msg->rx_status & ~CEC_RX_STATUS_OK))
+			printf("\t\t\tStatus: %s\n", status2s(*msg).c_str());
 		if (msg->tx_status & CEC_TX_STATUS_TIMEOUT)
 			warn("CEC_TX_STATUS_TIMEOUT was set, should not happen.\n");
 	}
 
-	if (!retval && request == CEC_RECEIVE && show_info)
-		printf("\t\t%s: Sequence: %u Rx Timestamp: %s Length: %u\n",
-		       opname.c_str(), msg->sequence, ts2s(msg->rx_ts).c_str(), msg->len);
-
 	if (!retval) {
 		__u8 la = cec_msg_initiator(msg);
 
@@ -716,20 +731,23 @@ const char *ok(int res)
 	static char buf[100];
 
 	if (res == NOTSUPPORTED) {
-		strcpy(buf, "OK (Not Supported)");
+		strcpy(buf, show_colors ? COLOR_GREEN("OK") " (Not Supported)" :
+		       "OK (Not Supported)");
 		res = 0;
 	} else if (res == PRESUMED_OK) {
-		strcpy(buf, "OK (Presumed)");
+		strcpy(buf, show_colors ? COLOR_GREEN("OK") " (Presumed)" :
+		       "OK (Presumed)");
 		res = 0;
 	} else if (res == REFUSED) {
-		strcpy(buf, "OK (Refused)");
+		strcpy(buf, show_colors ? COLOR_GREEN("OK") " (Refused)" :
+		       "OK (Refused)");
 		res = 0;
 	} else
-		strcpy(buf, "OK");
+		strcpy(buf, show_colors ? COLOR_GREEN("OK") : "OK");
 	tests_total++;
 	if (res) {
 		app_result = res;
-		sprintf(buf, "FAIL");
+		sprintf(buf, show_colors ? COLOR_RED("FAIL") : "FAIL");
 	} else {
 		tests_ok++;
 	}
@@ -874,6 +892,8 @@ retry:
 
 static int poll_remote_devs(struct node *node)
 {
+	unsigned retries = 0;
+
 	node->remote_la_mask = 0;
 	if (!(node->caps & CEC_CAP_TRANSMIT))
 		return 0;
@@ -882,10 +902,21 @@ static int poll_remote_devs(struct node *node)
 		struct cec_msg msg;
 
 		cec_msg_init(&msg, node->log_addr[0], i);
-
 		fail_on_test(doioctl(node, CEC_TRANSMIT, &msg));
-		if (msg.tx_status & CEC_TX_STATUS_OK)
+
+		if (msg.tx_status & CEC_TX_STATUS_OK) {
 			node->remote_la_mask |= 1 << i;
+			retries = 0;
+		} else if (msg.tx_status & CEC_TX_STATUS_NACK) {
+			retries = 0;
+		} else {
+			if (!(msg.tx_status & CEC_TX_STATUS_ARB_LOST))
+				warn("retry poll due to unexpected status: %s\n",
+				     status2s(msg).c_str());
+			retries++;
+			fail_on_test(retries > 10);
+			i--;
+		}
 	}
 	return 0;
 }
@@ -942,11 +973,15 @@ static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 	cec_msg_give_device_vendor_id(&msg, true);
 	ok = !transmit_timeout(node, &msg) || timed_out_or_abort(&msg);
 	printf("\t\tVendor ID                  : ");
-	if (ok)
+	if (ok) {
 		printf("%s\n", status2s(msg).c_str());
-	else
-		printf("0x%02x%02x%02x\n",
-		       msg.msg[2], msg.msg[3], msg.msg[4]);
+		node->remote[i].vendor_id = CEC_VENDOR_ID_NONE;
+	} else {
+		node->remote[i].vendor_id = (msg.msg[2] << 16) |
+			(msg.msg[3] << 8) | msg.msg[4];
+		printf("0x%06x %s\n", node->remote[i].vendor_id,
+		       vendor2s(node->remote[i].vendor_id));
+	}
 
 	cec_msg_init(&msg, la, i);
 	cec_msg_give_osd_name(&msg, true);
@@ -955,20 +990,16 @@ static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 	if (ok) {
 		printf("%s\n", status2s(msg).c_str());
 	} else {
-		char osd_name[15];
-
-		cec_ops_set_osd_name(&msg, osd_name);
-		printf("'%s'\n", osd_name);
+		cec_ops_set_osd_name(&msg, node->remote[i].osd_name);
+		printf("'%s'\n", node->remote[i].osd_name);
 	}
 
 	cec_msg_init(&msg, la, i);
 	cec_msg_get_menu_language(&msg, true);
 	if (transmit_timeout(node, &msg) && !timed_out_or_abort(&msg)) {
-		char language[4];
-
-		cec_ops_set_menu_language(&msg, language);
-		language[3] = 0;
-		printf("\t\tMenu Language              : %s\n", language);
+		cec_ops_set_menu_language(&msg, node->remote[i].language);
+		printf("\t\tMenu Language              : %s\n",
+		       node->remote[i].language);
 	}
 
 	cec_msg_init(&msg, la, i);
@@ -1006,9 +1037,10 @@ static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 			return;
 		node->remote[i].rc_profile = *rc_profile;
 		node->remote[i].dev_features = *dev_features;
-		node->remote[i].has_arc_rx =
+		node->remote[i].all_device_types = all_device_types;
+		node->remote[i].source_has_arc_rx =
 			(*dev_features & CEC_OP_FEAT_DEV_SOURCE_HAS_ARC_RX) != 0;
-		node->remote[i].has_arc_tx =
+		node->remote[i].sink_has_arc_tx =
 			(*dev_features & CEC_OP_FEAT_DEV_SINK_HAS_ARC_TX) != 0;
 		node->remote[i].has_aud_rate =
 			(*dev_features & CEC_OP_FEAT_DEV_HAS_SET_AUDIO_RATE) != 0;
@@ -1021,7 +1053,9 @@ static void topology_probe_device(struct node *node, unsigned i, unsigned la)
 
 int main(int argc, char **argv)
 {
-	const char *device = "/dev/cec0";	/* -d device */
+	std::string device;
+	const char *driver = NULL;
+	const char *adapter = NULL;
 	char short_options[26 * 2 * 2 + 1];
 	int remote_la = -1;
 	bool test_remote = false;
@@ -1030,6 +1064,18 @@ int main(int argc, char **argv)
 	int fd = -1;
 	int ch;
 	int i;
+	const char *env_media_apps_color = getenv("MEDIA_APPS_COLOR");
+
+	if (!env_media_apps_color || !strcmp(env_media_apps_color, "auto"))
+		show_colors = isatty(STDOUT_FILENO);
+	else if (!strcmp(env_media_apps_color, "always"))
+		show_colors = true;
+	else if (!strcmp(env_media_apps_color, "never"))
+		show_colors = false;
+	else {
+		fprintf(stderr,
+			"cec-compliance: invalid value for MEDIA_APPS_COLOR environment variable\n");
+	}
 
 	for (i = 0; long_options[i].name; i++) {
 		if (!isalpha(long_options[i].val))
@@ -1070,19 +1116,37 @@ int main(int argc, char **argv)
 			return 0;
 		case OptSetDevice:
 			device = optarg;
-			if (device[0] >= '0' && device[0] <= '9' && strlen(device) <= 3) {
+			if (device[0] >= '0' && device[0] <= '9' && device.length() <= 3) {
 				static char newdev[20];
 
-				sprintf(newdev, "/dev/cec%s", device);
+				sprintf(newdev, "/dev/cec%s", optarg);
 				device = newdev;
 			}
 			break;
+		case OptSetDriver:
+			driver = optarg;
+			break;
+		case OptSetAdapter:
+			adapter = optarg;
+			break;
 		case OptReplyThreshold:
 			reply_threshold = strtoul(optarg, NULL, 0);
 			break;
 		case OptTimeout:
 			long_timeout = strtoul(optarg, NULL, 0);
 			break;
+		case OptColor:
+			if (!strcmp(optarg, "always"))
+				show_colors = true;
+			else if (!strcmp(optarg, "never"))
+				show_colors = false;
+			else if (!strcmp(optarg, "auto"))
+				show_colors = isatty(STDOUT_FILENO);
+			else {
+				usage();
+				exit(1);
+			}
+			break;
 		case OptNoWarnings:
 			show_warnings = false;
 			break;
@@ -1128,8 +1192,19 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
-	if ((fd = open(device, O_RDWR)) < 0) {
-		fprintf(stderr, "Failed to open %s: %s\n", device,
+	if (device.empty() && (driver || adapter)) {
+		device = cec_device_find(driver, adapter);
+		if (device.empty()) {
+			fprintf(stderr,
+				"Could not find a CEC device for the given driver/adapter combination\n");
+			exit(1);
+		}
+	}
+	if (device.empty())
+		device = "/dev/cec0";
+
+	if ((fd = open(device.c_str(), O_RDWR)) < 0) {
+		fprintf(stderr, "Failed to open %s: %s\n", device.c_str(),
 			strerror(errno));
 		exit(1);
 	}
@@ -1138,7 +1213,7 @@ int main(int argc, char **argv)
 	struct cec_caps caps = { };
 
 	node.fd = fd;
-	node.device = device;
+	node.device = device.c_str();
 	doioctl(&node, CEC_ADAP_G_CAPS, &caps);
 	node.caps = caps.capabilities;
 	node.available_log_addrs = caps.available_log_addrs;
@@ -1290,7 +1365,8 @@ int main(int argc, char **argv)
 		exit(-1);
 
 	if (!options[OptSkipInfo]) {
-		printf("\nCompliance test for device %s:\n\n", device);
+		printf("\nCompliance test for %s device %s:\n\n",
+		       caps.driver, device.c_str());
 		printf("    The test results mean the following:\n"
 		       "        OK                  Supported correctly by the device.\n"
 		       "        OK (Not Supported)  Not supported and not mandatory for the device.\n"
@@ -1308,8 +1384,13 @@ int main(int argc, char **argv)
 	printf("Find remote devices:\n");
 	printf("\tPolling: %s\n", ok(poll_remote_devs(&node)));
 
+	if (!node.remote_la_mask) {
+		printf("\nFAIL: No remote devices found, exiting.\n");
+		exit(1);
+	}
+
 	if (options[OptTestAdapter])
-		testAdapter(node, laddrs, device);
+		testAdapter(node, laddrs, device.c_str());
 	printf("\n");
 
 	printf("Network topology:\n");
@@ -1339,7 +1420,8 @@ int main(int argc, char **argv)
 
 	close(fd);
 
-	printf("Total: %d, Succeeded: %d, Failed: %d, Warnings: %d\n",
+	printf("Total for %s device %s: %d, Succeeded: %d, Failed: %d, Warnings: %d\n",
+	       caps.driver, device.c_str(),
 	       tests_total, tests_ok, tests_total - tests_ok, warnings);
 	exit(app_result);
 }
