@@ -111,6 +111,7 @@ struct short_audio_desc {
 #define SAD_EXT_TYPE_LPCM_3D_AUDIO		13
 
 extern bool show_info;
+extern bool show_colors;
 extern bool show_warnings;
 extern bool exit_on_fail;
 extern bool exit_on_warn;
@@ -123,17 +124,21 @@ struct remote {
 	bool unrecognized_op[256];
 	unsigned prim_type;
 	__u16 phys_addr;
+	char osd_name[15];
+	char language[4];
 	__u8 cec_version;
 	__u8 rc_profile;
 	__u8 dev_features;
+	__u8 all_device_types;
+	__u32 vendor_id;
 	bool has_osd;
 	bool has_power_status;
 	bool has_image_view_on;
 	bool has_text_view_on;
 	bool in_standby;
 	bool has_remote_control_passthrough;
-	bool has_arc_rx;
-	bool has_arc_tx;
+	bool source_has_arc_rx;
+	bool sink_has_arc_tx;
 	bool arc_initiated;
 	bool has_sys_audio_mode_req;
 	bool has_set_sys_audio_mode;
@@ -163,12 +168,14 @@ struct node {
 	unsigned remote_la_mask;
 	struct remote remote[16];
 	__u16 phys_addr;
+	bool in_standby;
 };
 
 struct remote_subtest {
 	const char *name;
 	const __u16 la_mask;
 	int (*const test_fn)(struct node *node, unsigned me, unsigned la, bool interactive);
+	bool in_standby;
 };
 
 #define PRESUMED_OK 1
@@ -186,6 +193,10 @@ struct remote_subtest {
 #define cec_phys_addr_exp(pa) \
 	((pa) >> 12), ((pa) >> 8) & 0xf, ((pa) >> 4) & 0xf, (pa) & 0xf
 
+#define COLOR_GREEN(s) "\033[32m" s "\033[0m"
+#define COLOR_RED(s) "\033[1;31m" s "\033[0m"
+#define COLOR_BOLD(s) "\033[1m" s "\033[0m"
+
 #define info(fmt, args...) 					\
 	do {							\
 		if (show_info)					\
@@ -208,14 +219,17 @@ struct remote_subtest {
 		}							\
 	} while(0)
 
-#define warn(fmt, args...) 					\
-	do {							\
-		warnings++;					\
-		if (show_warnings)				\
-			printf("\t\twarn: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
-		if (exit_on_warn)				\
-			exit(1);				\
-	} while (0)
+#define warn(fmt, args...) 						\
+({									\
+	warnings++;							\
+	if (show_warnings)						\
+		printf("\t\%s: %s(%d): " fmt,				\
+		       show_colors ? COLOR_BOLD("warn") : "warn",	\
+		       __FILE__, __LINE__, ##args);			\
+	if (exit_on_warn)						\
+		exit(1);						\
+	0;								\
+})
 
 #define warn_once(fmt, args...)						\
 	do {								\
@@ -223,23 +237,28 @@ struct remote_subtest {
 									\
 		if (!show) {						\
 			show = true;					\
-			warnings++;					\
-			if (show_warnings)				\
-				printf("\t\twarn: %s(%d): " fmt,	\
-					__FILE__, __LINE__, ##args); 	\
-			if (exit_on_warn)				\
-				exit(1);				\
+			warn(fmt, ##args);				\
 		}							\
 	} while (0)
 
 #define fail(fmt, args...) 						\
 ({ 									\
-	printf("\t\tfail: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
+	printf("\t\t%s: %s(%d): " fmt, show_colors ?			\
+	       COLOR_RED("fail") : "fail", __FILE__, __LINE__, ##args);	\
 	if (exit_on_fail)						\
 		exit(1);						\
 	FAIL;								\
 })
 
+#define fail_or_warn(node, fmt, args...)		\
+({							\
+ 	if ((node)->in_standby)				\
+		warn(fmt, ##args);			\
+	else						\
+		fail(fmt, ##args);			\
+	(node)->in_standby ? 0 : FAIL;			\
+})
+
 #define fail_on_test(test) 				\
 	do {						\
 		if (test)				\
@@ -383,6 +402,15 @@ void testAdapter(struct node &node, struct cec_log_addrs &laddrs,
 
 // CEC core tests
 int testCore(struct node *node);
+int core_unknown(struct node *node, unsigned me, unsigned la, bool interactive);
+int core_abort(struct node *node, unsigned me, unsigned la, bool interactive);
+int system_info_polling(struct node *node, unsigned me, unsigned la, bool interactive);
+int system_info_phys_addr(struct node *node, unsigned me, unsigned la, bool interactive);
+int system_info_version(struct node *node, unsigned me, unsigned la, bool interactive);
+int system_info_get_menu_lang(struct node *node, unsigned me, unsigned la, bool interactive);
+int system_info_give_features(struct node *node, unsigned me, unsigned la, bool interactive);
+int vendor_specific_commands_id(struct node *node, unsigned me, unsigned la, bool interactive);
+int device_osd_transfer_give(struct node *node, unsigned me, unsigned la, bool interactive);
 
 // CEC processing
 int testProcessing(struct node *node, unsigned me);
