@@ -48,10 +48,14 @@ static int dal_request_current_latency(struct node *node, unsigned me, unsigned
 	cec_ops_report_current_latency(&msg, &phys_addr, &video_latency, &low_latency_mode,
 				       &audio_out_compensated, &audio_out_delay);
 	fail_on_test(phys_addr != node->remote[la].phys_addr);
-	info("Video latency: %d\n", video_latency);
+	info("Video latency: %d (%dms)\n", video_latency, (video_latency - 1) * 2);
 	info("Low latency mode: %d\n", low_latency_mode);
 	info("Audio output compensation: %d\n", audio_out_compensated);
-	info("Audio out delay: %d\n", audio_out_delay);
+	if (audio_out_compensated == CEC_OP_AUD_OUT_COMPENSATED_PARTIAL_DELAY) {
+		info("Audio out delay: %d (%dms)\n", audio_out_delay, (audio_out_delay - 1) * 2);
+		fail_on_test(audio_out_delay == 0 || audio_out_delay > 251);
+	}
+	fail_on_test(video_latency == 0 || video_latency > 251);
 
 	return 0;
 }
@@ -161,17 +165,17 @@ static int arc_initiate_tx(struct node *node, unsigned me, unsigned la, bool int
 	cec_msg_initiate_arc(&msg, true);
 	fail_on_test(!transmit_timeout(node, &msg));
 	if (timed_out(&msg)) {
-		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].has_arc_tx);
+		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].sink_has_arc_tx);
 		warn("Timed out waiting for Report ARC Initiated/Terminated.\n");
 		return PRESUMED_OK;
 	}
 	if (unrecognized_op(&msg)) {
-		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].has_arc_tx);
+		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].sink_has_arc_tx);
 		return NOTSUPPORTED;
 	}
 	if (cec_msg_opcode(&msg) == CEC_MSG_REPORT_ARC_INITIATED) {
 		fail_on_test(!pa_are_adjacent(node->phys_addr, node->remote[la].phys_addr));
-		fail_on_test_v2(node->remote[la].cec_version, !node->remote[la].has_arc_tx);
+		fail_on_test_v2(node->remote[la].cec_version, !node->remote[la].sink_has_arc_tx);
 		node->remote[la].arc_initiated = true;
 	}
 	else if (cec_msg_opcode(&msg) == CEC_MSG_REPORT_ARC_TERMINATED)
@@ -244,11 +248,11 @@ static int arc_initiate_rx(struct node *node, unsigned me, unsigned la, bool int
 		}
 	}
 	if (unsupported) {
-		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].has_arc_rx);
+		fail_on_test_v2(node->remote[la].cec_version, node->remote[la].source_has_arc_rx);
 		return NOTSUPPORTED;
 	}
 	fail_on_test(!pa_are_adjacent(node->phys_addr, node->remote[la].phys_addr));
-	fail_on_test_v2(node->remote[la].cec_version, !node->remote[la].has_arc_rx);
+	fail_on_test_v2(node->remote[la].cec_version, !node->remote[la].source_has_arc_rx);
 
 	cec_msg_init(&msg, me, la);
 	cec_msg_report_arc_initiated(&msg);
@@ -305,6 +309,44 @@ const unsigned arc_subtests_size = ARRAY_SIZE(arc_subtests);
 
 /* System Audio Control */
 
+/*
+ * The following scenarios are defined in section 13.15 of the CEC 1.4
+ * specification.
+ *
+ * These are not tested as they need three CEC devices.  An amplifier
+ * provides the audio for a source that is being displayed on a TV.
+ *
+ * 1.  Amplifier initiated <System Audio Mode Request> and active source
+ *     discovery with a <Request Active Source> broadcast plus the
+ *     <Active Source> response.
+ * 2.  Post discovery, subsequent amplifier <Set System Audio Mode> [On]
+ *     and System Audio Control feature confirmation with TV.
+ * 3.  Amplifier broadcasts <Set System Audio Mode> [On] to mute the TV and
+ *     unmute amplifier.
+ * 4.  Amplifier broadcasts <Set System Audio Mode> [Off] to unmute the TV
+ *     and mute the amplifier.
+ * 5.  When System Audio Mode is On, muting and unmuting an amplifier sends
+ *     a <Report Audio Status> message to the TV.
+ * 6.  When System Audio Mode is On, the amplifier sends a <Set System Audio
+ *     Mode> [Off] to unmute the TV before going into standby.
+ * 7.  When System Audio Mode is On, only the amplifier can control system
+ *     volume.
+ *
+ * These are not tested as they are hard-to-test corner cases.
+ *
+ * 1.  Optional features in subsection 13.15.4 of version 1.4.
+ *
+ * These are not tested as they deal with 1.3a or older versions and is not
+ * worth spending time on.
+ *
+ * 1.  <Request Audio Descriptor> message is from version 1.4 so older versions
+ *     report <Feature Abort>.
+ * 2.  <Report Audio Descriptor> message is from version 1.4 so older versions
+ *     report <Feature Abort>.
+ * 3.  System Audio Control is from version 1.3a so older versions report
+ *     <Feature Abort>.
+ */
+
 static int sac_request_sad_probe(struct node *node, unsigned me, unsigned la, bool interactive)
 {
 	struct cec_msg msg = {};
@@ -669,6 +711,32 @@ static int sac_user_control_press_mute(struct node *node, unsigned me, unsigned
 	return 0;
 }
 
+static int sac_user_control_press_mute_function(struct node *node, unsigned me, unsigned la, bool interactive)
+{
+	__u8 ret;
+
+	if ((ret = sac_util_send_user_control_press(node, me, la, 0x65)))
+		return ret;
+	fail_on_test_v2(node->remote[la].cec_version,
+			la == CEC_LOG_ADDR_AUDIOSYSTEM &&
+			node->remote[la].mute == CEC_OP_AUD_MUTE_STATUS_ON);
+
+	return 0;
+}
+
+static int sac_user_control_press_restore_volume_function(struct node *node, unsigned me, unsigned la, bool interactive)
+{
+	__u8 ret;
+
+	if ((ret = sac_util_send_user_control_press(node, me, la, 0x66)))
+		return ret;
+	fail_on_test_v2(node->remote[la].cec_version,
+			la == CEC_LOG_ADDR_AUDIOSYSTEM &&
+			node->remote[la].mute == CEC_OP_AUD_MUTE_STATUS_OFF);
+
+	return 0;
+}
+
 static int sac_user_control_release(struct node *node, unsigned me, unsigned la, bool interactive)
 {
 	struct cec_msg msg = {};
@@ -759,6 +827,12 @@ struct remote_subtest sac_subtests[] = {
 	{ "User Control Pressed (Mute)",
 	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
 	  sac_user_control_press_mute },
+	{ "User Control Pressed (Restore Volume Function)",
+	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+	  sac_user_control_press_restore_volume_function },
+	{ "User Control Pressed (Mute Function)",
+	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+	  sac_user_control_press_mute_function },
 	{ "User Control Released",
 	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
 	  sac_user_control_release },
