@@ -18,6 +18,7 @@
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <stdarg.h>
+#include <ctime>
 #include <cerrno>
 #include <string>
 #include <vector>
@@ -357,6 +358,20 @@ static unsigned parse_phys_addr(const char *value)
 	return (p1 << 12) | (p2 << 8) | (p3 << 4) | p4;
 }
 
+static unsigned parse_latency(const char *value)
+{
+	char *end;
+	unsigned delay = strtoul(value, &end, 0);
+
+	if (!memcmp(end, "ms", 2))
+		delay = (delay / 2) + 1;
+	if (delay < 1)
+		delay = 1;
+	else if (delay > 251)
+		delay = 251;
+	return delay;
+}
+
 static char options[512];
 
 static void log_arg(const struct arg *arg, const char *arg_name, __u32 val)
@@ -374,7 +389,10 @@ static void log_arg(const struct arg *arg, const char *arg_name, __u32 val)
 		}
 		/* fall through */
 	case CEC_TYPE_U8:
-		printf("\t%s: %u (0x%02x)\n", arg_name, val, val);
+		if (strstr(arg_name, "audio-out-delay") || strstr(arg_name, "video-latency"))
+			printf("\t%s: %u (0x%02x, %ums)\n", arg_name, val, val, (val - 1) * 2);
+		else
+			printf("\t%s: %u (0x%02x)\n", arg_name, val, val);
 		return;
 	case CEC_TYPE_U16:
 		if (strstr(arg_name, "phys-addr"))
@@ -565,8 +583,10 @@ static void log_u8_array(const char *arg_name, unsigned num, const __u8 *vals)
    In general the lower case is used to set something and the upper
    case is used to retrieve a setting. */
 enum Option {
+	OptSetAdapter = 'a',
 	OptClear = 'C',
 	OptSetDevice = 'd',
+	OptSetDriver = 'D',
 	OptFrom = 'f',
 	OptHelp = 'h',
 	OptLogicalAddress = 'l',
@@ -601,6 +621,7 @@ enum Option {
 	OptAllowUnregFallback,
 	OptNoRC,
 	OptReplyToFollowers,
+	OptRawMsg,
 	OptListDevices,
 	OptTimeout,
 	OptMonitorTime,
@@ -624,6 +645,8 @@ enum Option {
 	OptFeatSetAudioRate,
 	OptFeatSinkHasARCTx,
 	OptFeatSourceHasARCRx,
+	OptStressTestPowerCycle,
+	OptTestPowerCycle,
 	OptVendorCommand = 508,
 	OptVendorCommandWithID,
 	OptVendorRemoteButtonDown,
@@ -657,6 +680,8 @@ static void init_messages()
 
 static struct option long_options[] = {
 	{ "device", required_argument, 0, OptSetDevice },
+	{ "adapter", required_argument, 0, OptSetAdapter },
+	{ "driver", required_argument, 0, OptSetDriver },
 	{ "help", no_argument, 0, OptHelp },
 	{ "trace", no_argument, 0, OptTrace },
 	{ "verbose", no_argument, 0, OptVerbose },
@@ -668,6 +693,7 @@ static struct option long_options[] = {
 	{ "allow-unreg-fallback", no_argument, 0, OptAllowUnregFallback },
 	{ "no-rc-passthrough", no_argument, 0, OptNoRC },
 	{ "reply-to-followers", no_argument, 0, OptReplyToFollowers },
+	{ "raw-msg", no_argument, 0, OptRawMsg },
 	{ "timeout", required_argument, 0, OptTimeout },
 	{ "clear", no_argument, 0, OptClear },
 	{ "wait-for-msgs", no_argument, 0, OptWaitForMsgs },
@@ -723,6 +749,9 @@ static struct option long_options[] = {
 	{ "vendor-command", required_argument, 0, OptVendorCommand }, \
 	{ "custom-command", required_argument, 0, OptCustomCommand }, \
 
+	{ "test-power-cycle", no_argument, 0, OptTestPowerCycle }, \
+	{ "stress-test-power-cycle", required_argument, 0, OptStressTestPowerCycle }, \
+
 	{ 0, 0, 0, 0 }
 };
 
@@ -731,6 +760,8 @@ static void usage(void)
 	printf("Usage:\n"
 	       "  -d, --device <dev>       Use device <dev> instead of /dev/cec0\n"
 	       "                           If <dev> starts with a digit, then /dev/cec<dev> is used.\n"
+	       "  -D, --driver <driver>    Use a cec device with this driver name\n"
+	       "  -a, --adapter <adapter>  Use a cec device with this adapter name\n"
 	       "  -p, --phys-addr <addr>   Use this physical address\n"
 	       "  -o, --osd-name <name>    Use this OSD name\n"
 	       "  -V, --vendor-id <id>     Use this vendor ID\n"
@@ -756,6 +787,7 @@ static void usage(void)
 	       "  --allow-unreg-fallback   Allow fallback to Unregistered\n"
 	       "  --no-rc-passthrough      Disable the RC passthrough\n"
 	       "  --reply-to-followers     The reply will be sent to followers as well\n"
+	       "  --raw-msg                Transmit the message without validating it (must be root)\n"
 	       "  --timeout <ms>           Set the reply timeout in milliseconds (default is 1000 ms)\n"
 	       "  --list-ui-commands       List all UI commands that can be used with --user-control-pressed\n"
 	       "  --list-devices           List all cec devices\n"
@@ -800,6 +832,12 @@ static void usage(void)
 	       "                           Use - for stdout.\n"
 	       "  --analyze-pin <from>     Analyze the low-level CEC pin changes from the file <from>.\n"
 	       "                           Use - for stdin.\n"
+	       "  --test-power-cycle       Test power cycle behavior of the display.\n"
+	       "  --stress-test-power-cycle cnt=<count>[,max-sleep=<secs>]\n"
+	       "                           Power cycle display <count> times. If 0, then never stop.\n"
+	       "                           If <secs> is non-zero (0 is the default), then sleep for\n"
+	       "                           a random number of seconds between 0 and <secs> before\n"
+	       "                           each Standby or Image View On message.\n"
 	       "\n"
 	       CEC_USAGE
 	       "\n"
@@ -1307,27 +1345,27 @@ static void show_msg(const cec_msg &msg)
 	log_msg(&msg);
 	if (options[OptShowRaw])
 		log_raw_msg(&msg);
+	std::string status;
+	if ((msg.tx_status & ~CEC_TX_STATUS_OK) ||
+	    (msg.rx_status & ~CEC_RX_STATUS_OK))
+		status = status2s(msg);
 	if (verbose && transmitted)
-		printf("\tSequence: %u Tx Timestamp: %s\n",
-		       msg.sequence, ts2s(msg.tx_ts).c_str());
+		printf("\tSequence: %u Tx Timestamp: %s %s\n",
+		       msg.sequence, ts2s(msg.tx_ts).c_str(),
+		       status.c_str());
 	else if (verbose && !transmitted)
-		printf("\tSequence: %u Rx Timestamp: %s\n",
-		       msg.sequence, ts2s(msg.rx_ts).c_str());
+		printf("\tSequence: %u Rx Timestamp: %s %s\n",
+		       msg.sequence, ts2s(msg.rx_ts).c_str(),
+		       status.c_str());
 }
 
 static void wait_for_msgs(struct node &node, __u32 monitor_time)
 {
-	__u32 monitor = CEC_MODE_INITIATOR | CEC_MODE_FOLLOWER;
 	fd_set rd_fds;
 	fd_set ex_fds;
 	int fd = node.fd;
 	time_t t;
 	
-	if (doioctl(&node, CEC_S_MODE, &monitor)) {
-		fprintf(stderr, "Selecting follower mode failed.\n");
-		return;
-	}
-
 	fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_NONBLOCK);
 	t = time(NULL) + monitor_time;
 
@@ -1343,6 +1381,13 @@ static void wait_for_msgs(struct node &node, __u32 monitor_time)
 		res = select(fd + 1, &rd_fds, NULL, &ex_fds, &tv);
 		if (res < 0)
 			break;
+		if (FD_ISSET(fd, &ex_fds)) {
+			struct cec_event ev;
+
+			if (doioctl(&node, CEC_DQEVENT, &ev))
+				continue;
+			log_event(ev, true);
+		}
 		if (FD_ISSET(fd, &rd_fds)) {
 			struct cec_msg msg = { };
 
@@ -1354,13 +1399,6 @@ static void wait_for_msgs(struct node &node, __u32 monitor_time)
 			if (!res)
 				show_msg(msg);
 		}
-		if (FD_ISSET(fd, &ex_fds)) {
-			struct cec_event ev;
-
-			if (doioctl(&node, CEC_DQEVENT, &ev))
-				continue;
-			log_event(ev, true);
-		}
 	}
 }
 
@@ -1588,6 +1626,460 @@ err:
 	exit(1);
 }
 
+static bool wait_for_pwr_state(struct node &node, unsigned from, bool on)
+{
+	struct cec_msg msg;
+	__u8 pwr;
+	int ret;
+
+	cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
+	cec_msg_give_device_power_status(&msg, true);
+	ret = doioctl(&node, CEC_TRANSMIT, &msg);
+	if (ret == ENONET) {
+		printf("X");
+		fflush(stdout);
+		return !on;
+	}
+	if (ret) {
+		fprintf(stderr, "Give Device Power Status Transmit failed: %s\n",
+			strerror(ret));
+		exit(1);
+	}
+	if (msg.rx_status & CEC_RX_STATUS_OK) {
+		cec_ops_report_power_status(&msg, &pwr);
+		switch (pwr) {
+		case CEC_OP_POWER_STATUS_ON:
+			printf("+");
+			break;
+		case CEC_OP_POWER_STATUS_STANDBY:
+			printf("-");
+			break;
+		case CEC_OP_POWER_STATUS_TO_ON:
+			printf("/");
+			break;
+		case CEC_OP_POWER_STATUS_TO_STANDBY:
+			printf("\\");
+			break;
+		default:
+			printf(" %d ", pwr);
+			break;
+		}
+		fflush(stdout);
+		return pwr == (on ? CEC_OP_POWER_STATUS_ON : CEC_OP_POWER_STATUS_STANDBY);
+	}
+	printf("N");
+	fflush(stdout);
+	return false;
+}
+
+static int init_power_cycle_test(struct node &node)
+{
+	struct cec_msg msg;
+	unsigned from;
+	unsigned tries;
+	const unsigned max_tries = 30;
+	__u16 pa;
+	int ret;
+
+	printf("Legend:\n\n"
+	       "X   No LA claimed (HPD is likely pulled low)\n"
+	       "N   No Report Power Status received\n"
+	       "+   Reported On\n"
+	       "-   Reported In Standby\n"
+	       "/   Reported Transitioning to On\n"
+	       "\\   Reported Transitioning to Standby\n\n");
+
+	struct cec_log_addrs laddrs = { };
+	doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+	if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID) {
+		from = laddrs.log_addr[0];
+	} else {
+		switch (laddrs.log_addr_type[0]) {
+		case CEC_LOG_ADDR_TYPE_TV:
+			fprintf(stderr, "A TV can't run the power cycle test.\n");
+			exit(1);
+		case CEC_LOG_ADDR_TYPE_RECORD:
+			from = CEC_LOG_ADDR_RECORD_1;
+			break;
+		case CEC_LOG_ADDR_TYPE_TUNER:
+			from = CEC_LOG_ADDR_TUNER_1;
+			break;
+		case CEC_LOG_ADDR_TYPE_PLAYBACK:
+			from = CEC_LOG_ADDR_PLAYBACK_1;
+			break;
+		case CEC_LOG_ADDR_TYPE_AUDIOSYSTEM:
+			from = CEC_LOG_ADDR_AUDIOSYSTEM;
+			break;
+		case CEC_LOG_ADDR_TYPE_SPECIFIC:
+			from = CEC_LOG_ADDR_SPECIFIC;
+			break;
+		case CEC_LOG_ADDR_TYPE_UNREGISTERED:
+		default:
+			from = CEC_LOG_ADDR_UNREGISTERED;
+			break;
+		}
+	}
+
+	if (laddrs.log_addr[0] == CEC_LOG_ADDR_INVALID) {
+		printf("No Logical Addresses claimed, assume TV is already in Standby\n\n");
+	} else {
+		doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+		/*
+		 * Some displays only accept Standby from the Active Source.
+		 * So make us the Active Source before sending Standby.
+		 */
+		printf("Transmit Active Source to TV: ");
+		fflush(stdout);
+		cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
+		cec_msg_active_source(&msg, pa);
+		ret = doioctl(&node, CEC_TRANSMIT, &msg);
+		if (ret) {
+			printf("FAIL: %s\n", strerror(ret));
+			exit(1);
+		}
+		printf("OK\nTransmit Standby to TV: ");
+		fflush(stdout);
+		cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
+		cec_msg_standby(&msg);
+
+		tries = 0;
+		for (;;) {
+			doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+			if (laddrs.log_addr[0] == CEC_LOG_ADDR_INVALID)
+				break;
+
+			ret = doioctl(&node, CEC_TRANSMIT, &msg);
+			if (ret) {
+				printf("FAIL: %s\n", strerror(ret));
+				exit(1);
+			}
+
+			if (wait_for_pwr_state(node, from, false))
+				break;
+			sleep(1);
+			if (++tries > max_tries) {
+				printf(" FAIL: never went into standby\n");
+				exit(1);
+			}
+		}
+		printf(" OK\nTV is in Standby\n");
+		sleep(5);
+	}
+	doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+	printf("Physical Address: %x.%x.%x.%x\n\n",
+	       cec_phys_addr_exp(pa));
+	return from;
+}
+
+static void test_power_cycle(struct node &node)
+{
+	struct cec_log_addrs laddrs = { };
+	struct cec_msg msg;
+	unsigned from;
+	unsigned tries;
+	const unsigned max_tries = 30;
+	unsigned secs;
+	__u16 pa, prev_pa;
+	__u8 wakeup_la;
+	int ret;
+
+	from = init_power_cycle_test(node);
+
+	for (unsigned iter = 0; iter <= 2 * 12; iter++) {
+		unsigned i = iter / 2;
+
+		/*
+		 * For sleep values 0-5 run the test twice,
+		 * after that run it only once.
+		 */
+		if (i > 5 && (iter & 1))
+			continue;
+
+		doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+		if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+			wakeup_la = from = laddrs.log_addr[0];
+		else
+			wakeup_la = CEC_LOG_ADDR_UNREGISTERED;
+		printf("Wake up TV using Image View On from LA %s: ", la2s(wakeup_la));
+		fflush(stdout);
+		cec_msg_init(&msg, wakeup_la, CEC_LOG_ADDR_TV);
+		cec_msg_image_view_on(&msg);
+		ret = doioctl(&node, CEC_TRANSMIT, &msg);
+		if (ret) {
+			printf("FAIL: %s\n", strerror(ret));
+			exit(1);
+		}
+		tries = 0;
+		for (;;) {
+			doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+			if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+				from = laddrs.log_addr[0];
+			if (wait_for_pwr_state(node, from, true))
+				break;
+			if (++tries > max_tries)
+				break;
+			sleep(1);
+			doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+			if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+				continue;
+
+			doioctl(&node, CEC_TRANSMIT, &msg);
+		}
+
+		if (tries > max_tries) {
+			wakeup_la = from;
+			printf("\nFAIL: never woke up, retry\n");
+			printf("Wake up TV using Image View On from LA %s: ", la2s(wakeup_la));
+			fflush(stdout);
+			cec_msg_init(&msg, wakeup_la, CEC_LOG_ADDR_TV);
+			cec_msg_image_view_on(&msg);
+			ret = doioctl(&node, CEC_TRANSMIT, &msg);
+			if (ret) {
+				printf("FAIL: %s\n", strerror(ret));
+				exit(1);
+			}
+			tries = 0;
+			for (;;) {
+				doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+				if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+					from = laddrs.log_addr[0];
+				if (wait_for_pwr_state(node, from, true))
+					break;
+				if (++tries > max_tries) {
+					printf("\nFAIL: never woke up\n");
+					exit(1);
+				}
+				sleep(1);
+				doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+				if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+					continue;
+
+				doioctl(&node, CEC_TRANSMIT, &msg);
+			}
+		}
+		printf(" %d second%s\n", tries, tries == 1 ? "" : "s");
+
+		doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+		if (laddrs.log_addr[0] == CEC_LOG_ADDR_INVALID) {
+			printf("FAIL: invalid logical address\n");
+			exit(1);
+		}
+		from = laddrs.log_addr[0];
+		doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+		printf("\tPhysical Address: %x.%x.%x.%x LA: %s\n",
+		       cec_phys_addr_exp(pa), la2s(from));
+		if (pa == CEC_PHYS_ADDR_INVALID) {
+			printf("FAIL: invalid physical address\n");
+			exit(1);
+		}
+		prev_pa = pa;
+		secs = i % 10;
+		printf("\tSleep %d seconds\n", secs);
+		sleep(secs);
+		doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+		if (pa != prev_pa) {
+			printf("\tFAIL: PA is now %x.%x.%x.%x\n\n",
+			       cec_phys_addr_exp(pa));
+			exit(1);
+		}
+
+		printf("\nPut TV in standby from LA %s: ", la2s(from));
+		fflush(stdout);
+		/*
+		 * Some displays only accept Standby from the Active Source.
+		 * So make us the Active Source before sending Standby.
+		 */
+		cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
+		cec_msg_active_source(&msg, pa);
+		ret = doioctl(&node, CEC_TRANSMIT, &msg);
+		if (ret) {
+			printf("FAIL: Active Source Transmit failed: %s\n", strerror(ret));
+			exit(1);
+		}
+		cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
+		cec_msg_standby(&msg);
+		ret = doioctl(&node, CEC_TRANSMIT, &msg);
+		if (ret) {
+			printf("FAIL: %s\n", strerror(ret));
+			exit(1);
+		}
+
+		tries = 0;
+		bool first_standby = true;
+		for (;;) {
+			doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+			if (laddrs.log_addr[0] == CEC_LOG_ADDR_INVALID)
+				break;
+			if (wait_for_pwr_state(node, from, false))
+				break;
+			if (++tries > max_tries) {
+				if (first_standby) {
+					printf("\nFAIL: never went into standby, retry\n");
+					printf("Put TV in standby from LA %s: ", la2s(from));
+					fflush(stdout);
+					first_standby = false;
+					tries = 0;
+					cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
+					cec_msg_standby(&msg);
+					ret = doioctl(&node, CEC_TRANSMIT, &msg);
+					if (!ret)
+						continue;
+					printf("FAIL: %s\n", strerror(ret));
+					exit(1);
+				}
+				printf("\nFAIL: never went into standby\n");
+				exit(1);
+			}
+			sleep(1);
+		}
+		printf(" %d second%s\n", tries, tries == 1 ? "" : "s");
+		doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+		printf("\tPhysical Address: %x.%x.%x.%x\n",
+		       cec_phys_addr_exp(pa));
+		prev_pa = pa;
+		secs = i <= 10 ? i : 10 + 10 * (i - 10);
+		printf("\tSleep %d second%s\n", secs, secs == 1 ? "" : "s");
+		sleep(secs);
+		doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+		if (pa != prev_pa)
+			printf("\tPhysical Address: %x.%x.%x.%x\n",
+			       cec_phys_addr_exp(pa));
+		printf("\n");
+	}
+}
+
+static void stress_test_power_cycle(struct node &node,
+				    unsigned cnt, unsigned max_sleep)
+{
+	struct cec_log_addrs laddrs = { };
+	struct cec_msg msg;
+	const unsigned max_tries = 50;
+	unsigned tries = 0;
+	unsigned iter = 0;
+	unsigned mod_usleep = 0;
+	unsigned wakeup_la;
+	__u16 pa, prev_pa;
+	int ret;
+
+	if (max_sleep)
+		mod_usleep = 1000000 * max_sleep + 1;
+
+	unsigned from = init_power_cycle_test(node);
+
+	for (;;) {
+		unsigned usecs1 = mod_usleep ? random() % mod_usleep : 0;
+		unsigned usecs2 = mod_usleep ? random() % mod_usleep : 0;
+
+		iter++;
+
+		doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+		if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+			wakeup_la = from = laddrs.log_addr[0];
+		else
+			wakeup_la = CEC_LOG_ADDR_UNREGISTERED;
+
+		if (mod_usleep)
+			printf("Sleep %.2fs, then transmit Image View On ", usecs1 / 1000000.0);
+		else
+			printf("Transmit Image View On ");
+		printf("from LA %s (iteration %u): ", la2s(wakeup_la), iter);
+		fflush(stdout);
+		usleep(usecs1);
+
+		tries = 0;
+		cec_msg_init(&msg, wakeup_la, CEC_LOG_ADDR_TV);
+		cec_msg_image_view_on(&msg);
+		ret = doioctl(&node, CEC_TRANSMIT, &msg);
+		if (ret) {
+			printf("FAIL: %s\n", strerror(ret));
+			exit(1);
+		}
+
+		for (;;) {
+			doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+			if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+				from = laddrs.log_addr[0];
+			if (wait_for_pwr_state(node, from, true))
+				break;
+			if (++tries > max_tries) {
+				printf("\nFAIL: never woke up\n");
+				exit(1);
+			}
+			sleep(1);
+			doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+			if (laddrs.log_addr[0] != CEC_LOG_ADDR_INVALID)
+				continue;
+
+			doioctl(&node, CEC_TRANSMIT, &msg);
+		}
+		printf(" %d second%s\n", tries, tries == 1 ? "" : "s");
+		doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+		if (laddrs.log_addr[0] == CEC_LOG_ADDR_INVALID) {
+			printf("FAIL: invalid logical address\n");
+			exit(1);
+		}
+		from = laddrs.log_addr[0];
+		doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+		prev_pa = pa;
+		printf("\tPhysical Address: %x.%x.%x.%x LA: %s\n",
+		       cec_phys_addr_exp(pa), la2s(from));
+		if (pa == CEC_PHYS_ADDR_INVALID) {
+			printf("FAIL: invalid physical address\n");
+			exit(1);
+		}
+
+		if (mod_usleep)
+			printf("Sleep %.2fs, then transmit Standby ", usecs2 / 1000000.0);
+		else
+			printf("Transmit Standby ");
+		printf("(iteration %u): ", iter);
+		fflush(stdout);
+		usleep(usecs2);
+		doioctl(&node, CEC_ADAP_G_PHYS_ADDR, &pa);
+		if (pa != prev_pa) {
+			printf("\tFAIL: PA is now %x.%x.%x.%x\n\n",
+			       cec_phys_addr_exp(pa));
+			exit(1);
+		}
+
+		cec_msg_init(&msg, from, CEC_LOG_ADDR_TV);
+		/*
+		 * Some displays only accept Standby from the Active Source.
+		 * So make us the Active Source before sending Standby.
+		 */
+		cec_msg_active_source(&msg, pa);
+		ret = doioctl(&node, CEC_TRANSMIT, &msg);
+		if (ret) {
+			printf("FAIL: Active Source Transmit failed: %s\n", strerror(ret));
+			exit(1);
+		}
+		cec_msg_standby(&msg);
+		ret = doioctl(&node, CEC_TRANSMIT, &msg);
+		if (ret) {
+			printf("FAIL: %s\n", strerror(ret));
+			exit(1);
+		}
+
+		tries = 0;
+		for (;;) {
+			doioctl(&node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
+			if (laddrs.log_addr[0] == CEC_LOG_ADDR_INVALID)
+				break;
+			if (wait_for_pwr_state(node, from, false))
+				break;
+			if (++tries > max_tries) {
+				printf("\nFAIL: never went into standby\n");
+				exit(1);
+			}
+			sleep(1);
+		}
+		printf(" %d second%s\n", tries, tries == 1 ? "" : "s");
+
+		if (cnt && iter == cnt)
+			break;
+	}
+}
+
 static int calc_node_val(const char *s)
 {
 	s = strrchr(s, '/') + 1;
@@ -1687,13 +2179,18 @@ static void list_devices()
 
 int main(int argc, char **argv)
 {
-	const char *device = "/dev/cec0";	/* -d device */
+	std::string device;
+	const char *driver = NULL;
+	const char *adapter = NULL;
 	const message *opt;
 	msg_vec msgs;
 	char short_options[26 * 2 * 2 + 1];
 	__u32 timeout = 1000;
 	__u32 monitor_time = 0;
 	__u32 vendor_id = 0x000c03; /* HDMI LLC vendor ID */
+	unsigned int pwr_cycle_cnt = 0;
+	unsigned int max_sleep = 0;
+	bool warn_if_unconfigured = false;
 	__u16 phys_addr;
 	__u8 from = 0, to = 0, first_to = 0xff;
 	__u8 dev_features = 0;
@@ -1730,7 +2227,7 @@ int main(int argc, char **argv)
 			break;
 
 		cec_msg_init(&msg, 0, 0);
-		msg.msg[0] = cec_msg_is_broadcast(&msg) ? 0xf : (options[OptTo] ? to : 0xf0);
+		msg.msg[0] = options[OptTo] ? to : 0xf0;
 		options[(int)ch] = 1;
 
 		switch (ch) {
@@ -1739,13 +2236,19 @@ int main(int argc, char **argv)
 			return 0;
 		case OptSetDevice:
 			device = optarg;
-			if (device[0] >= '0' && device[0] <= '9' && strlen(device) <= 3) {
+			if (device[0] >= '0' && device[0] <= '9' && device.length() <= 3) {
 				static char newdev[20];
 
-				sprintf(newdev, "/dev/cec%s", device);
+				sprintf(newdev, "/dev/cec%s", optarg);
 				device = newdev;
 			}
 			break;
+		case OptSetDriver:
+			driver = optarg;
+			break;
+		case OptSetAdapter:
+			adapter = optarg;
+			break;
 		case OptVerbose:
 			verbose = true;
 			break;
@@ -2058,6 +2561,34 @@ int main(int argc, char **argv)
 			list_devices();
 			break;
 
+		case OptTestPowerCycle:
+			warn_if_unconfigured = true;
+			break;
+
+		case OptStressTestPowerCycle: {
+			static const char *arg_names[] = {
+				"cnt",
+				"max-sleep",
+				NULL
+			};
+			char *value, *subs = optarg;
+
+			while (*subs != '\0') {
+				switch (parse_subopt(&subs, arg_names, &value)) {
+				case 0:
+					pwr_cycle_cnt = strtoul(value, 0L, 0);
+					break;
+				case 1:
+					max_sleep = strtoul(value, 0L, 0);
+					break;
+				default:
+					exit(1);
+				}
+			}
+			warn_if_unconfigured = true;
+			break;
+		}
+
 		default:
 			if (ch >= OptHelpAll) {
 				usage_options(ch);
@@ -2080,6 +2611,9 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
+	if (msgs.size())
+		warn_if_unconfigured = true;
+
 	if (store_pin && analyze_pin) {
 		fprintf(stderr, "--store-pin and --analyze-pin options cannot be combined.\n\n");
 		usage();
@@ -2136,11 +2670,22 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
+	if (device.empty() && (driver || adapter)) {
+		device = cec_device_find(driver, adapter);
+		if (device.empty()) {
+			fprintf(stderr,
+				"Could not find a CEC device for the given driver/adapter combination\n");
+			exit(1);
+		}
+	}
+	if (device.empty())
+		device = "/dev/cec0";
+
 	clock_gettime(CLOCK_MONOTONIC, &start_monotonic);
 	gettimeofday(&start_timeofday, NULL);
 
-	if ((fd = open(device, O_RDWR)) < 0) {
-		fprintf(stderr, "Failed to open %s: %s\n", device,
+	if ((fd = open(device.c_str(), O_RDWR)) < 0) {
+		fprintf(stderr, "Failed to open %s: %s\n", device.c_str(),
 			strerror(errno));
 		exit(1);
 	}
@@ -2149,7 +2694,7 @@ int main(int argc, char **argv)
 	struct cec_caps caps = { };
 
 	node.fd = fd;
-	node.device = device;
+	node.device = device.c_str();
 	doioctl(&node, CEC_ADAP_G_CAPS, &caps);
 	node.caps = caps.capabilities;
 	node.available_log_addrs = caps.available_log_addrs;
@@ -2312,6 +2857,8 @@ int main(int argc, char **argv)
 		if (options[OptMonitor] || options[OptMonitorAll] ||
 		    options[OptMonitorPin] || options[OptStorePin])
 			goto skip_la;
+		if (warn_if_unconfigured)
+			fprintf(stderr, "\nAdapter is unconfigured, please configure it first.\n");
 		return 0;
 	}
 	if (!options[OptSkipInfo])
@@ -2331,6 +2878,14 @@ int main(int argc, char **argv)
 		printf("\n");
 	}
 
+	if (options[OptWaitForMsgs]) {
+		__u32 monitor = CEC_MODE_INITIATOR | CEC_MODE_FOLLOWER;
+
+		if (doioctl(&node, CEC_S_MODE, &monitor)) {
+			fprintf(stderr, "Selecting follower mode failed.\n");
+			return 1;
+		}
+	}
 	if (options[OptNonBlocking])
 		fcntl(node.fd, F_SETFL, fcntl(node.fd, F_GETFL) | O_NONBLOCK);
 
@@ -2344,11 +2899,13 @@ int main(int argc, char **argv)
 		}
 		if (msg.msg[0] == 0xf0)
 			msg.msg[0] = first_to;
+		msg.msg[0] &= 0x0f;
 		msg.msg[0] |= from << 4;
 		to = msg.msg[0] & 0xf;
 		printf("\nTransmit from %s to %s (%d to %d):\n",
 		       la2s(from), to == 0xf ? "all" : la2s(to), from, to);
 		msg.flags = options[OptReplyToFollowers] ? CEC_MSG_FL_REPLY_TO_FOLLOWERS : 0;
+		msg.flags |= options[OptRawMsg] ? CEC_MSG_FL_RAW : 0;
 		msg.timeout = msg.reply ? timeout : 0;
 		log_msg(&msg);
 		if (doioctl(&node, CEC_TRANSMIT, &msg))
@@ -2375,6 +2932,10 @@ int main(int argc, char **argv)
 	if (options[OptNonBlocking])
 		fcntl(node.fd, F_SETFL, fcntl(node.fd, F_GETFL) & ~O_NONBLOCK);
 
+	if (options[OptTestPowerCycle])
+		test_power_cycle(node);
+	if (options[OptStressTestPowerCycle])
+		stress_test_power_cycle(node, pwr_cycle_cnt, max_sleep);
 skip_la:
 	if (options[OptMonitor] || options[OptMonitorAll] ||
 	    options[OptMonitorPin] || options[OptStorePin])
