@@ -742,6 +742,7 @@ public:
 	unsigned g_memory() const { return v4l_queue_g_memory(this); }
 	unsigned g_buffers() const { return v4l_queue_g_buffers(this); }
 	unsigned g_num_planes() const { return v4l_queue_g_num_planes(this); }
+	unsigned g_capabilities() const { return v4l_queue_g_capabilities(this); }
 	unsigned g_length(unsigned plane) const { return v4l_queue_g_length(this, plane); }
 	unsigned g_mem_offset(unsigned index, unsigned plane) const { return v4l_queue_g_mem_offset(this, index, plane); }
 	void *g_mmapping(unsigned index, unsigned plane) const { return v4l_queue_g_mmapping(this, index, plane); }
@@ -768,25 +769,25 @@ public:
 	{
 		return v4l_queue_mmap_bufs(fd->g_v4l_fd(), this, from);
 	}
-	int munmap_bufs(cv4l_fd *fd)
+	int munmap_bufs(cv4l_fd *fd, unsigned from = 0)
 	{
-		return v4l_queue_munmap_bufs(fd->g_v4l_fd(), this);
+		return v4l_queue_munmap_bufs(fd->g_v4l_fd(), this, from);
 	}
 	int alloc_bufs(cv4l_fd *fd, unsigned from = 0)
 	{
 		return v4l_queue_alloc_bufs(fd->g_v4l_fd(), this, from);
 	}
-	int free_bufs()
+	int free_bufs(unsigned from = 0)
 	{
-		return v4l_queue_free_bufs(this);
+		return v4l_queue_free_bufs(this, from);
 	}
 	int obtain_bufs(cv4l_fd *fd, unsigned from = 0)
 	{
 		return v4l_queue_obtain_bufs(fd->g_v4l_fd(), this, from);
 	}
-	int release_bufs(cv4l_fd *fd)
+	int release_bufs(cv4l_fd *fd, unsigned from = 0)
 	{
-		return v4l_queue_release_bufs(fd->g_v4l_fd(), this);
+		return v4l_queue_release_bufs(fd->g_v4l_fd(), this, from);
 	}
 	bool has_expbuf(cv4l_fd *fd)
 	{
@@ -808,6 +809,10 @@ public:
 	{
 		v4l_queue_buffer_init(this, &buf, index);
 	}
+	void buffer_update(v4l_buffer &buf, unsigned index) const
+	{
+		v4l_queue_buffer_update(this, &buf, index);
+	}
 	int queue_all(cv4l_fd *fd);
 };
 
@@ -841,9 +846,15 @@ public:
 		if (v4l_type_is_planar(g_type()))
 			buf.m.planes = planes;
 	}
+	void update(const cv4l_queue &q, unsigned index = 0)
+	{
+		q.buffer_update(*this, index);
+	}
 
 	__u32 g_index() const { return v4l_buffer_g_index(this); }
 	void s_index(unsigned index) { v4l_buffer_s_index(this, index); }
+	__s32 g_request_fd() const { return v4l_buffer_g_request_fd(this); }
+	void s_request_fd(__s32 request_fd) { v4l_buffer_s_request_fd(this, request_fd); }
 	unsigned g_type() const { return v4l_buffer_g_type(this); }
 	unsigned g_memory() const { return v4l_buffer_g_memory(this); }
 	__u32 g_flags() const { return v4l_buffer_g_flags(this); }
@@ -871,6 +882,7 @@ public:
 	void s_timestamp_src(__u32 src) { v4l_buffer_s_timestamp_src(this, src); }
 	bool ts_is_copy() const { return v4l_buffer_is_copy(this); }
 	const timeval &g_timestamp() const { return *v4l_buffer_g_timestamp(this); }
+	__u64 g_timestamp_ns() const { return v4l2_timeval_to_ns(v4l_buffer_g_timestamp(this)); }
 	void s_timestamp(const timeval &tv) { v4l_buffer_s_timestamp(this, &tv); }
 	void s_timestamp_ts(const timespec &ts) { v4l_buffer_s_timestamp_ts(this, &ts); }
 	void s_timestamp_clock() { v4l_buffer_s_timestamp_clock(this); }
