@@ -392,6 +392,11 @@ static inline bool v4l_has_meta_cap(const struct v4l_fd *f)
 	return v4l_g_caps(f) & V4L2_CAP_META_CAPTURE;
 }
 
+static inline bool v4l_has_meta_out(const struct v4l_fd *f)
+{
+	return v4l_g_caps(f) & V4L2_CAP_META_OUTPUT;
+}
+
 static inline bool v4l_has_touch(const struct v4l_fd *f)
 {
 	return v4l_g_caps(f) & V4L2_CAP_TOUCH;
@@ -440,6 +445,9 @@ static inline __u32 v4l_determine_type(const struct v4l_fd *f)
 		return V4L2_BUF_TYPE_SDR_OUTPUT;
 	if (v4l_has_meta_cap(f))
 		return V4L2_BUF_TYPE_META_CAPTURE;
+	if (v4l_has_meta_out(f))
+		return V4L2_BUF_TYPE_META_OUTPUT;
+
 	return 0;
 }
 
@@ -672,6 +680,7 @@ static inline void v4l_format_s_pixelformat(struct v4l2_format *fmt, __u32 pixel
 		fmt->fmt.vbi.sample_format = pixelformat;
 		break;
 	case V4L2_BUF_TYPE_META_CAPTURE:
+	case V4L2_BUF_TYPE_META_OUTPUT:
 		fmt->fmt.meta.dataformat = pixelformat;
 		break;
 	}
@@ -693,6 +702,7 @@ static inline __u32 v4l_format_g_pixelformat(const struct v4l2_format *fmt)
 	case V4L2_BUF_TYPE_VBI_OUTPUT:
 		return fmt->fmt.vbi.sample_format;
 	case V4L2_BUF_TYPE_META_CAPTURE:
+	case V4L2_BUF_TYPE_META_OUTPUT:
 		return fmt->fmt.meta.dataformat;
 	default:
 		return 0;
@@ -1031,6 +1041,7 @@ v4l_format_g_sizeimage(const struct v4l2_format *fmt, unsigned plane)
 	case V4L2_BUF_TYPE_SDR_OUTPUT:
 		return plane ? 0 : fmt->fmt.sdr.buffersize;
 	case V4L2_BUF_TYPE_META_CAPTURE:
+	case V4L2_BUF_TYPE_META_OUTPUT:
 		return plane ? 0 : fmt->fmt.meta.buffersize;
 	default:
 		return 0;
@@ -1152,7 +1163,8 @@ static inline bool v4l_type_is_sdr(unsigned type)
 
 static inline bool v4l_type_is_meta(unsigned type)
 {
-       return type == V4L2_BUF_TYPE_META_CAPTURE;
+	return type == V4L2_BUF_TYPE_META_CAPTURE ||
+	       type == V4L2_BUF_TYPE_META_OUTPUT;
 }
 
 static inline unsigned v4l_type_invert(unsigned type)
@@ -1183,6 +1195,17 @@ static inline void v4l_buffer_s_index(struct v4l_buffer *buf, __u32 index)
 	buf->buf.index = index;
 }
 
+static inline __s32 v4l_buffer_g_request_fd(const struct v4l_buffer *buf)
+{
+	return buf->buf.request_fd;
+}
+
+static inline void v4l_buffer_s_request_fd(struct v4l_buffer *buf, __s32 request_fd)
+{
+	buf->buf.request_fd = request_fd;
+}
+
+
 static inline unsigned v4l_buffer_g_type(const struct v4l_buffer *buf)
 {
 	return buf->buf.type;
@@ -1383,7 +1406,9 @@ struct v4l_queue {
 	unsigned type;
 	unsigned memory;
 	unsigned buffers;
+	unsigned mappings;
 	unsigned num_planes;
+	unsigned capabilities;
 
 	__u32 lengths[VIDEO_MAX_PLANES];
 	__u32 mem_offsets[VIDEO_MAX_FRAME][VIDEO_MAX_PLANES];
@@ -1408,7 +1433,9 @@ static inline void v4l_queue_init(struct v4l_queue *q,
 static inline unsigned v4l_queue_g_type(const struct v4l_queue *q) { return q->type; }
 static inline unsigned v4l_queue_g_memory(const struct v4l_queue *q) { return q->memory; }
 static inline unsigned v4l_queue_g_buffers(const struct v4l_queue *q) { return q->buffers; }
+static inline unsigned v4l_queue_g_mappings(const struct v4l_queue *q) { return q->mappings; }
 static inline unsigned v4l_queue_g_num_planes(const struct v4l_queue *q) { return q->num_planes; }
+static inline unsigned v4l_queue_g_capabilities(const struct v4l_queue *q) { return q->capabilities; }
 
 static inline __u32 v4l_queue_g_length(const struct v4l_queue *q, unsigned plane)
 {
@@ -1427,7 +1454,7 @@ static inline void v4l_queue_s_mmapping(struct v4l_queue *q, unsigned index, uns
 
 static inline void *v4l_queue_g_mmapping(const struct v4l_queue *q, unsigned index, unsigned plane)
 {
-	if (index >= v4l_queue_g_buffers(q) || plane >= v4l_queue_g_num_planes(q))
+	if (index >= v4l_queue_g_mappings(q) || plane >= v4l_queue_g_num_planes(q))
 		return NULL;
 	return q->mmappings[index][plane];
 }
@@ -1503,6 +1530,7 @@ static inline int v4l_queue_reqbufs(struct v4l_fd *f,
 	if (ret)
 		return ret;
 	q->buffers = reqbufs.count;
+	q->capabilities = reqbufs.capabilities;
 	return v4l_queue_querybufs(f, q, 0);
 }
 
@@ -1536,6 +1564,7 @@ static inline int v4l_queue_create_bufs(struct v4l_fd *f,
 	ret = v4l_ioctl(f, VIDIOC_CREATE_BUFS, &createbufs);
 	if (ret)
 		return ret;
+	q->capabilities = createbufs.capabilities;
 	q->buffers += createbufs.count;
 	return v4l_queue_querybufs(f, q, q->buffers - createbufs.count);
 }
@@ -1548,7 +1577,7 @@ static inline int v4l_queue_mmap_bufs(struct v4l_fd *f,
 	if (q->memory != V4L2_MEMORY_MMAP && q->memory != V4L2_MEMORY_DMABUF)
 		return 0;
 
-	for (b = 0; b < v4l_queue_g_buffers(q); b++) {
+	for (b = from; b < v4l_queue_g_buffers(q); b++) {
 		for (p = 0; p < v4l_queue_g_num_planes(q); p++) {
 			void *m = MAP_FAILED;
 
@@ -1564,9 +1593,11 @@ static inline int v4l_queue_mmap_bufs(struct v4l_fd *f,
 			v4l_queue_s_mmapping(q, b, p, m);
 		}
 	}
+	q->mappings = b;
 	return 0;
 }
-static inline int v4l_queue_munmap_bufs(struct v4l_fd *f, struct v4l_queue *q)
+static inline int v4l_queue_munmap_bufs(struct v4l_fd *f, struct v4l_queue *q,
+					unsigned from)
 {
 	unsigned b, p;
 	int ret = 0;
@@ -1574,7 +1605,7 @@ static inline int v4l_queue_munmap_bufs(struct v4l_fd *f, struct v4l_queue *q)
 	if (q->memory != V4L2_MEMORY_MMAP && q->memory != V4L2_MEMORY_DMABUF)
 		return 0;
 
-	for (b = 0; b < v4l_queue_g_buffers(q); b++) {
+	for (b = from; b < v4l_queue_g_mappings(q); b++) {
 		for (p = 0; p < v4l_queue_g_num_planes(q); p++) {
 			void *m = v4l_queue_g_mmapping(q, b, p);
 
@@ -1590,6 +1621,7 @@ static inline int v4l_queue_munmap_bufs(struct v4l_fd *f, struct v4l_queue *q)
 			v4l_queue_s_mmapping(q, b, p, NULL);
 		}
 	}
+	q->mappings = from;
 	return 0;
 }
 
@@ -1600,7 +1632,7 @@ static inline int v4l_queue_alloc_bufs(struct v4l_fd *f,
 
 	if (q->memory != V4L2_MEMORY_USERPTR)
 		return 0;
-	for (b = 0; b < v4l_queue_g_buffers(q); b++) {
+	for (b = from; b < v4l_queue_g_buffers(q); b++) {
 		for (p = 0; p < v4l_queue_g_num_planes(q); p++) {
 			void *m = malloc(v4l_queue_g_length(q, p));
 
@@ -1612,13 +1644,13 @@ static inline int v4l_queue_alloc_bufs(struct v4l_fd *f,
 	return 0;
 }
 
-static inline int v4l_queue_free_bufs(struct v4l_queue *q)
+static inline int v4l_queue_free_bufs(struct v4l_queue *q, unsigned from)
 {
 	unsigned b, p;
 
 	if (q->memory != V4L2_MEMORY_USERPTR)
 		return 0;
-	for (b = 0; b < v4l_queue_g_buffers(q); b++) {
+	for (b = from; b < v4l_queue_g_buffers(q); b++) {
 		for (p = 0; p < v4l_queue_g_num_planes(q); p++) {
 			free(v4l_queue_g_userptr(q, b, p));
 			v4l_queue_s_userptr(q, b, p, NULL);
@@ -1635,11 +1667,12 @@ static inline int v4l_queue_obtain_bufs(struct v4l_fd *f,
 	return v4l_queue_mmap_bufs(f, q, from);
 }
 
-static inline int v4l_queue_release_bufs(struct v4l_fd *f, struct v4l_queue *q)
+static inline int v4l_queue_release_bufs(struct v4l_fd *f, struct v4l_queue *q,
+					 unsigned from)
 {
 	if (q->memory == V4L2_MEMORY_USERPTR)
-		return v4l_queue_free_bufs(q);
-	return v4l_queue_munmap_bufs(f, q);
+		return v4l_queue_free_bufs(q, from);
+	return v4l_queue_munmap_bufs(f, q, from);
 }
 
 
@@ -1696,16 +1729,16 @@ static inline void v4l_queue_close_exported_fds(struct v4l_queue *q)
 static inline void v4l_queue_free(struct v4l_fd *f, struct v4l_queue *q)
 {
 	v4l_ioctl(f, VIDIOC_STREAMOFF, &q->type);
-	v4l_queue_release_bufs(f, q);
+	v4l_queue_release_bufs(f, q, 0);
 	v4l_queue_close_exported_fds(q);
 	v4l_queue_reqbufs(f, q, 0);
 }
 
-static inline void v4l_queue_buffer_init(const struct v4l_queue *q, struct v4l_buffer *buf, unsigned index)
+static inline void v4l_queue_buffer_update(const struct v4l_queue *q,
+					   struct v4l_buffer *buf, unsigned index)
 {
 	unsigned p;
 		
-	v4l_buffer_init(buf, v4l_queue_g_type(q), v4l_queue_g_memory(q), index);
 	if (v4l_type_is_planar(q->type)) {
 		buf->buf.length = v4l_queue_g_num_planes(q);
 		buf->buf.m.planes = buf->planes;
@@ -1726,6 +1759,12 @@ static inline void v4l_queue_buffer_init(const struct v4l_queue *q, struct v4l_b
 	}
 }
 
+static inline void v4l_queue_buffer_init(const struct v4l_queue *q, struct v4l_buffer *buf, unsigned index)
+{
+	v4l_buffer_init(buf, v4l_queue_g_type(q), v4l_queue_g_memory(q), index);
+	v4l_queue_buffer_update(q, buf, index);
+}
+
 static inline int v4l_query_ext_ctrl(v4l_fd *f, struct v4l2_query_ext_ctrl *qec,
 		bool next_ctrl, bool next_compound)
 {
