@@ -17,6 +17,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -30,6 +31,8 @@
 #include <linux/lirc.h>
 
 #include "ir-encode.h"
+#include "keymap.h"
+#include "bpf_encoder.h"
 
 #ifdef ENABLE_NLS
 # define _(string) gettext(string)
@@ -53,20 +56,21 @@
        __result; })
 #endif
 
-/* See drivers/media/rc/ir-lirc-codec.c line 23 */
-#define LIRCBUF_SIZE	512
+/* See drivers/media/rc/lirc_dev.c line 22 */
+#define LIRCBUF_SIZE 1024
 #define IR_DEFAULT_TIMEOUT 125000
 
-const char *argp_program_version = "IR raw version " V4L_UTILS_VERSION;
+const char *argp_program_version = "IR ctl version " V4L_UTILS_VERSION;
 const char *argp_program_bug_address = "Sean Young <sean@mess.org>";
 
 /*
  * Since this program drives the lirc interface, use the same terminology
  */
-struct file {
-	struct file *next;
+struct send {
+	struct send *next;
 	const char *fname;
 	bool is_scancode;
+	bool is_keycode;
 	union {
 		struct {
 			unsigned carrier;
@@ -77,6 +81,7 @@ struct file {
 			unsigned scancode;
 			unsigned protocol;
 		};
+		char	keycode[1];
 	};
 };
 
@@ -85,7 +90,9 @@ struct arguments {
 	bool features;
 	bool receive;
 	bool verbose;
-	struct file *send;
+	bool mode2;
+	struct keymap *keymap;
+	struct send *send;
 	bool oneshot;
 	char *savetofile;
 	int wideband;
@@ -104,19 +111,22 @@ static const struct argp_option options[] = {
 	{ "features",	'f',	0,		0,	N_("list lirc device features") },
 	{ "receive",	'r',	N_("FILE"),	OPTION_ARG_OPTIONAL,	N_("receive IR to stdout or file") },
 	{ "send",	's',	N_("FILE"),	0,	N_("send IR pulse and space file") },
-	{ "scancode", 'S',	N_("SCANCODE"),	0,	N_("send IR scancode in protocol specified") },
+	{ "scancode",	'S',	N_("SCANCODE"),	0,	N_("send IR scancode in protocol specified") },
+	{ "keycode",	'K',	N_("KEYCODE"),	0,	N_("send IR keycode from keymap") },
 	{ "verbose",	'v',	0,		0,	N_("verbose output") },
 		{ .doc = N_("Receiving options:") },
 	{ "one-shot",	'1',	0,		0,	N_("end receiving after first message") },
+	{ "mode2",	2,	0,		0,	N_("output in mode2 format") },
 	{ "wideband",	'w',	0,		0,	N_("use wideband receiver aka learning mode") },
-	{ "no-wideband",'n',	0,		0,	N_("use normal narrowband receiver, disable learning mode") },
+	{ "narrowband",	'n',	0,		0,	N_("use narrowband receiver, disable learning mode") },
 	{ "carrier-range", 'R', N_("RANGE"),	0,	N_("set receiver carrier range") },
 	{ "measure-carrier", 'm', 0,		0,	N_("report carrier frequency") },
 	{ "no-measure-carrier", 'M', 0,		0,	N_("disable reporting carrier frequency") },
 	{ "timeout",	't',	N_("TIMEOUT"),	0,	N_("set receiving timeout") },
 		{ .doc = N_("Sending options:") },
+	{ "keymap",	'k',	N_("KEYMAP"),	0,	N_("use keymap to send key from") },
 	{ "carrier",	'c',	N_("CARRIER"),	0,	N_("set send carrier") },
-	{ "duty-cycle",	'D',	N_("DUTY"),	0,	N_("set duty cycle") },
+	{ "duty-cycle",	'D',	N_("DUTY"),	0,	N_("set send duty cycle") },
 	{ "emitters",	'e',	N_("EMITTERS"),	0,	N_("set send emitters") },
 	{ "gap",	'g',	N_("GAP"),	0,	N_("set gap between files or scancodes") },
 	{ }
@@ -127,21 +137,24 @@ static const char args_doc[] = N_(
 	"--receive [save to file]\n"
 	"--send [file to send]\n"
 	"--scancode [scancode to send]\n"
+	"--keycode [keycode to send]\n"
 	"[to set lirc option]");
 
 static const char doc[] = N_(
 	"\nReceive IR, send IR and list features of lirc device\n"
 	"You will need permission on /dev/lirc for the program to work\n"
 	"\nOn the options below, the arguments are:\n"
-	"  DEV	    - the /dev/lirc* device to use\n"
+	"  DEV      - the /dev/lirc* device to use\n"
 	"  FILE     - a text file containing pulses and spaces\n"
 	"  CARRIER  - the carrier frequency to use for sending\n"
 	"  DUTY     - the duty cycle to use for sending\n"
 	"  EMITTERS - comma separated list of emitters to use for sending, e.g. 1,2\n"
-	"  GAP      - gap between pulse and files or scancodes in microseconds\n"
+	"  GAP      - gap between sending in microseconds\n"
 	"  RANGE    - set range of accepted carrier frequencies, e.g. 20000-40000\n"
 	"  TIMEOUT  - set length of space before receiving stops in microseconds\n"
-	"  SCANCODE - protocol:scancode, e.g. nec:0xa814\n\n"
+	"  KEYCODE  - key code in keymap\n"
+	"  SCANCODE - protocol:scancode, e.g. nec:0xa814\n"
+	"  KEYMAP   - a rc keymap file from which to send keys\n\n"
 	"Note that most lirc setting have global state, i.e. the device will remain\n"
 	"in this state until set otherwise.");
 
@@ -192,21 +205,14 @@ static unsigned parse_emitters(char *p)
 	return emit;
 }
 
-static struct file *read_file(struct arguments *args, const char *fname)
+static struct send *read_file_pulse_space(struct arguments *args, const char *fname, FILE *input)
 {
 	bool expect_pulse = true;
 	int lineno = 0, lastspace = 0;
 	char line[1024];
 	int len = 0;
-	static const char *whitespace = " \n\r\t";
-	struct file *f;
-
-	FILE *input = fopen(fname, "r");
-
-	if (!input) {
-		fprintf(stderr, _("%s: could not open: %m\n"), fname);
-		return NULL;
-	}
+	static const char whitespace[] = " \n\r\t";
+	struct send *f;
 
 	f = malloc(sizeof(*f));
 	if (f == NULL) {
@@ -214,6 +220,7 @@ static struct file *read_file(struct arguments *args, const char *fname)
 		return NULL;
 	}
 	f->is_scancode = false;
+	f->is_keycode = false;
 	f->carrier = 0;
 	f->fname = fname;
 
@@ -244,7 +251,9 @@ static struct file *read_file(struct arguments *args, const char *fname)
 
 			scancodestr = strchr(p, ':');
 			if (!scancodestr) {
-				fprintf(stderr, _("error: %s:%d: scancode argument '%s' should in protocol:scancode format\n"), fname, lineno, p);
+				fprintf(stderr, _("error: %s:%d: scancode argument '%s' should be in protocol:scancode format\n"), fname, lineno, p);
+				fclose(input);
+				free(f);
 				return NULL;
 			}
 
@@ -252,26 +261,31 @@ static struct file *read_file(struct arguments *args, const char *fname)
 
 			if (!protocol_match(p, &proto)) {
 				fprintf(stderr, _("error: %s:%d: protocol '%s' not found\n"), fname, lineno, p);
+				fclose(input);
+				free(f);
 				return NULL;
 			}
 
 			if (!strtoscancode(scancodestr, &scancode)) {
 				fprintf(stderr, _("error: %s:%d: invalid scancode '%s'\n"), fname, lineno, scancodestr);
+				fclose(input);
+				free(f);
 				return NULL;
 			}
 
 			if (!protocol_encoder_available(proto)) {
 				fprintf(stderr, _("error: %s:%d: no encoder available for `%s'\n"), fname, lineno, protocol_name(proto));
+				fclose(input);
+				free(f);
 				return NULL;
 			}
 
-			if (!protocol_scancode_valid(proto, scancode)) {
-				fprintf(stderr, _("error: %s:%d: invalid scancode '%s' for protocol '%s'\n"), fname, lineno, scancodestr, protocol_name(proto));
-				return NULL;
-			}
+			protocol_scancode_valid(&proto, &scancode);
 
 			if (len + protocol_max_size(proto) >= LIRCBUF_SIZE) {
 				fprintf(stderr, _("error: %s:%d: too much IR for one transmit\n"), fname, lineno);
+				fclose(input);
+				free(f);
 				return NULL;
 			}
 
@@ -353,10 +367,145 @@ static struct file *read_file(struct arguments *args, const char *fname)
 	return f;
 }
 
-static struct file *read_scancode(const char *name)
+static struct send *read_file_raw(struct arguments *args, const char *fname, FILE *input)
+{
+	int lineno = 0, lastspace = 0;
+	char line[1024];
+	int len = 0;
+	static const char whitespace[] = " \n\r\t,";
+	struct send *f;
+
+	f = malloc(sizeof(*f));
+	if (f == NULL) {
+		fprintf(stderr, _("Failed to allocate memory\n"));
+		fclose(input);
+		return NULL;
+	}
+	f->is_scancode = false;
+	f->is_keycode = false;
+	f->carrier = 0;
+	f->fname = fname;
+
+	while (fgets(line, sizeof(line), input)) {
+		long int value;
+		char *p, *saveptr;
+		lineno++;
+		char *keyword = strtok_r(line, whitespace, &saveptr);
+
+		for (;;) {
+			if (keyword == NULL || *keyword == 0 || *keyword == '#' ||
+			    (keyword[0] == '/' && keyword[1] == '/'))
+				break;
+
+			value = strtol(keyword, &p, 10);
+			if (errno || *p) {
+				fprintf(stderr, _("%s:%d: error: expected integer, got `%s'\n"),
+					fname, lineno, keyword);
+				fclose(input);
+				free(f);
+				return NULL;
+			}
+
+			if (len % 2) {
+				if (keyword[0] == '+') {
+					fprintf(stderr, _("%s:%d: error: pulse found where space expected `%s'\n"), fname, lineno, keyword);
+					free(f);
+					return NULL;
+				}
+				if (keyword[0] == '-')
+					value = -value;
+			} else {
+				if (keyword[0] == '-') {
+					fprintf(stderr, _("%s:%d: error: space found where pulse expected `%s'\n"), fname, lineno, keyword);
+					fclose(input);
+					free(f);
+					return NULL;
+				}
+				lastspace = lineno;
+			}
+
+			if (value <= 0 || value >= LIRC_VALUE_MASK) {
+				fprintf(stderr, _("%s:%d: error: value `%s' out of range\n"), fname, lineno, keyword);
+				fclose(input);
+				free(f);
+				return NULL;
+			}
+
+			f->buf[len++] = value;
+
+			if (len >= LIRCBUF_SIZE) {
+				fprintf(stderr, _("warning: %s:%d: IR cannot exceed %u edges\n"), fname, lineno, LIRCBUF_SIZE);
+				break;
+			}
+
+			keyword = strtok_r(NULL, whitespace, &saveptr);
+		}
+	}
+
+	fclose(input);
+
+	if (len == 0) {
+		fprintf(stderr, _("%s: no pulses or spaces found\n"), fname);
+		free(f);
+		return NULL;
+	}
+
+	if ((len % 2) == 0) {
+		fprintf(stderr, _("warning: %s:%d: trailing space ignored\n"),
+							fname, lastspace);
+		len--;
+	}
+
+	f->len = len;
+
+	return f;
+}
+
+static struct send *read_file(struct arguments *args, const char *fname)
+{
+	FILE *input = fopen(fname, "r");
+	char line[1024];
+
+	if (!input) {
+		fprintf(stderr, _("%s: could not open: %m\n"), fname);
+		return NULL;
+	}
+
+	while (fgets(line, sizeof(line), input)) {
+		int start = 0;
+
+		while (isspace(line[start]))
+			start++;
+
+		switch (line[start]) {
+		case '/':
+			if (line[start+1] != '/')
+				break;
+		case 0:
+		case '#':
+			continue;
+		case '+':
+		case '-':
+		case '0' ... '9':
+			rewind(input);
+			return read_file_raw(args, fname, input);
+		default:
+			rewind(input);
+			return read_file_pulse_space(args, fname, input);
+		}
+	}
+
+	fclose(input);
+
+	fprintf(stderr, _("%s: file is empty\n"), fname);
+
+	return NULL;
+}
+
+static struct send *read_scancode(const char *name)
 {
 	enum rc_proto proto;
-	struct file *f;
+	struct send *f;
 	unsigned scancode;
 	char *pstr;
 	char *p = strchr(name, ':');
@@ -380,10 +529,7 @@ static struct file *read_scancode(const char *name)
 		return NULL;
 	}
 
-	if (!protocol_scancode_valid(proto, scancode)) {
-		fprintf(stderr, _("error: invalid scancode '%s' for protocol '%s'\n"), p + 1, protocol_name(proto));
-		return NULL;
-	}
+	protocol_scancode_valid(&proto, &scancode);
 
 	f = malloc(sizeof(*f));
 	if (f == NULL) {
@@ -392,6 +538,7 @@ static struct file *read_scancode(const char *name)
 	}
 
 	f->is_scancode = true;
+	f->is_keycode = false;
 	f->scancode = scancode;
 	f->protocol = proto;
 
@@ -401,7 +548,8 @@ static struct file *read_scancode(const char *name)
 static error_t parse_opt(int k, char *arg, struct argp_state *state)
 {
 	struct arguments *arguments = state->input;
-	struct file *s;
+	struct keymap *map;
+	struct send *s;
 
 	switch (k) {
 	case 'f':
@@ -425,6 +573,9 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 	case '1':
 		arguments->oneshot = true;
 		break;
+	case 2:
+		arguments->mode2 = true;
+		break;
 	case 'v':
 		arguments->verbose = true;
 		break;
@@ -508,7 +659,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		if (arguments->send == NULL)
 			arguments->send = s;
 		else {
-			struct file *p = arguments->send;
+			struct send *p = arguments->send;
 			while (p->next) p = p->next;
 			p->next = s;
 		}
@@ -524,12 +675,45 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		if (arguments->send == NULL)
 			arguments->send = s;
 		else {
-			struct file *p = arguments->send;
+			struct send *p = arguments->send;
 			while (p->next) p = p->next;
 			p->next = s;
 		}
 		break;
 
+	case 'K':
+		if (arguments->receive || arguments->features)
+			argp_error(state, _("key send can not be combined with receive or features option"));
+		s = malloc(sizeof(*s) + strlen(arg));
+		if (s == NULL)
+			exit(EX_DATAERR);
+
+		s->next = NULL;
+		strcpy(s->keycode, arg);
+		s->is_scancode = false;
+		s->is_keycode = true;
+		if (arguments->send == NULL)
+			arguments->send = s;
+		else {
+			struct send *p = arguments->send;
+			while (p->next) p = p->next;
+			p->next = s;
+		}
+		break;
+
+	case 'k':
+		if (parse_keymap(arg, &map, arguments->verbose))
+			exit(EX_DATAERR);
+		if (arguments->keymap == NULL)
+			arguments->keymap = map;
+		else {
+			// add to end of list of keymaps
+			struct keymap *p = arguments->keymap;
+			while (p->next) p = p->next;
+			p->next = map;
+		}
+		break;
+
 	case ARGP_KEY_END:
 		if (!arguments->work_to_do)
 			argp_usage(state);
@@ -539,12 +723,87 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		return ARGP_ERR_UNKNOWN;
 	}
 
-	if (k != '1' && k != 'd' && k != 'v')
+	if (k != '1' && k != 'd' && k != 'v' && k != 'k' && k != 2)
 		arguments->work_to_do = true;
 
 	return 0;
 }
 
+static struct send* convert_keycode(struct keymap *map, const char *keycode)
+{
+	struct send *s = NULL;
+	int count = 0;
+
+	for (;map; map = map->next) {
+		struct scancode_entry *se;
+		struct raw_entry *re;
+
+		for (re = map->raw; re; re = re->next) {
+			if (strcmp(re->keycode, keycode))
+				continue;
+
+			count++;
+
+			if (!s) {
+				s = malloc(sizeof(*s) + re->raw_length * sizeof(int));
+				s->len = re->raw_length;
+				memcpy(s->buf, re->raw, s->len * sizeof(int));
+				s->is_scancode = false;
+				s->is_keycode = false;
+				s->carrier = keymap_param(map, "carrier", 0);
+				s->next = NULL;
+			}
+		}
+
+		for (se = map->scancode; se; se = se->next) {
+			int buf[LIRCBUF_SIZE], length;
+			const char *proto_str;
+			enum rc_proto proto;
+
+			if (strcmp(se->keycode, keycode))
+				continue;
+
+			count++;
+
+			if (s)
+				continue;
+
+			proto_str = map->variant ?: map->protocol;
+
+			if (protocol_match(proto_str, &proto)) {
+				s = malloc(sizeof(*s));
+				s->protocol = proto;
+				s->scancode = se->scancode;
+				s->is_scancode = true;
+				s->is_keycode = false;
+				s->next = NULL;
+			} else if (encode_bpf_protocol(map, se->scancode,
+						       buf, &length)) {
+				s = malloc(sizeof(*s) + sizeof(int) * length);
+				s->len = length;
+				memcpy(s->buf, buf, length * sizeof(int));
+				s->is_scancode = false;
+				s->is_keycode = false;
+				s->carrier = keymap_param(map, "carrier", 0);
+				s->next = NULL;
+			} else {
+				fprintf(stderr, _("error: protocol '%s' not supported\n"), proto_str);
+				return NULL;
+			}
+		}
+	}
+
+	if (!s) {
+		fprintf(stderr, _("error: keycode `%s' not found in keymap\n"), keycode);
+		return NULL;
+	}
+
+	if (count > 1)
+		fprintf(stderr, _("warning: keycode `%s' has %d definitions in keymaps, using first\n"), keycode, count);
+
+	return s;
+}
+
 static const struct argp argp = {
 	.options = options,
 	.parser = parse_opt,
@@ -708,8 +967,13 @@ static void lirc_features(struct arguments *args, int fd, unsigned features)
 		if (features & LIRC_CAN_MEASURE_CARRIER)
 			printf(_(" - Can measure carrier\n"));
 		if (features & LIRC_CAN_SET_REC_TIMEOUT) {
-			unsigned min_timeout, max_timeout;
-			int rc = ioctl(fd, LIRC_GET_MIN_TIMEOUT, &min_timeout);
+			unsigned min_timeout, max_timeout, timeout;
+
+			// This ioctl is only supported from kernel 4.18 onwards
+			int rc = ioctl(fd, LIRC_GET_REC_TIMEOUT, &timeout);
+			if (rc == 0)
+				printf(_(" - Receiving timeout %u microseconds\n"), timeout);
+			rc = ioctl(fd, LIRC_GET_MIN_TIMEOUT, &min_timeout);
 			if (rc) {
 				fprintf(stderr, _("warning: %s: device supports setting receiving timeout but LIRC_GET_MIN_TIMEOUT returns: %m\n"), dev);
 				min_timeout = 0;
@@ -724,7 +988,7 @@ static void lirc_features(struct arguments *args, int fd, unsigned features)
 			}
 
 			if (min_timeout || max_timeout)
-				printf(_(" - Can set receiving timeout min:%u microseconds max:%u microseconds\n"), min_timeout, max_timeout);
+				printf(_(" - Can set receiving timeout min %u microseconds, max %u microseconds\n"), min_timeout, max_timeout);
 		}
 	} else if (features & LIRC_CAN_REC_LIRCCODE) {
 		printf(_(" - Device can receive using device dependent LIRCCODE mode (not supported)\n"));
@@ -758,7 +1022,7 @@ static void lirc_features(struct arguments *args, int fd, unsigned features)
 	}
 }
 
-static int lirc_send(struct arguments *args, int fd, unsigned features, struct file *f)
+static int lirc_send(struct arguments *args, int fd, unsigned features, struct send *f)
 {
 	const char *dev = args->device;
 	int rc, mode;
@@ -770,6 +1034,10 @@ static int lirc_send(struct arguments *args, int fd, unsigned features, struct f
 	}
 
 	if (f->is_scancode) {
+		if (args->verbose)
+			printf("Sending to kernel encoder protocol:%s scancode:0x%x\n",
+			       protocol_name(f->protocol), f->scancode);
+
 		mode = LIRC_MODE_SCANCODE;
 		rc = ioctl(fd, LIRC_SET_SEND_MODE, &mode);
 		if (rc == 0) {
@@ -811,9 +1079,10 @@ static int lirc_send(struct arguments *args, int fd, unsigned features, struct f
 	size_t size = f->len * sizeof(unsigned);
 	if (args->verbose) {
 		int i;
-		printf("Sending:\n");
+		printf("Sending:");
 		for (i=0; i<f->len; i++)
-			printf("%s %u\n", i & 1 ? "space" : "pulse", f->buf[i]);
+			printf("%s%u", i & 1 ? " -" : " +", f->buf[i]);
+		putchar('\n');
 	}
 	ret = TEMP_FAILURE_RETRY(write(fd, f->buf, size));
 	if (ret < 0) {
@@ -863,6 +1132,7 @@ int lirc_receive(struct arguments *args, int fd, unsigned features)
 
 	bool keep_reading = true;
 	bool leading_space = true;
+	unsigned carrier = 0;
 
 	while (keep_reading) {
 		ssize_t ret = TEMP_FAILURE_RETRY(read(fd, buf, sizeof(buf)));
@@ -896,20 +1166,42 @@ int lirc_receive(struct arguments *args, int fd, unsigned features)
 				break;
 			}
 
-			switch (msg) {
-			case LIRC_MODE2_TIMEOUT:
-				fprintf(out, "timeout %u\n", val);
-				leading_space = true;
-				break;
-			case LIRC_MODE2_PULSE:
-				fprintf(out, "pulse %u\n", val);
-				break;
-			case LIRC_MODE2_SPACE:
-				fprintf(out, "space %u\n", val);
-				break;
-			case LIRC_MODE2_FREQUENCY:
-				fprintf(out, "carrier %u\n", val);
-				break;
+			if (args->mode2) {
+				switch (msg) {
+				case LIRC_MODE2_TIMEOUT:
+					fprintf(out, "timeout %u\n", val);
+					leading_space = true;
+					break;
+				case LIRC_MODE2_PULSE:
+					fprintf(out, "pulse %u\n", val);
+					break;
+				case LIRC_MODE2_SPACE:
+					fprintf(out, "space %u\n", val);
+					break;
+				case LIRC_MODE2_FREQUENCY:
+					fprintf(out, "carrier %u\n", val);
+					break;
+				}
+			} else {
+				switch (msg) {
+				case LIRC_MODE2_TIMEOUT:
+					if (carrier)
+						fprintf(out, "# carrier %uHz, timeout %u\n", carrier, val);
+					else
+						fprintf(out, "# timeout %u\n", val);
+					leading_space = true;
+					carrier = 0;
+					break;
+				case LIRC_MODE2_PULSE:
+					fprintf(out, "+%u ", val);
+					break;
+				case LIRC_MODE2_SPACE:
+					fprintf(out, "-%u ", val);
+					break;
+				case LIRC_MODE2_FREQUENCY:
+					carrier = val;
+					break;
+				}
 			}
 
 			fflush(out);
@@ -950,12 +1242,28 @@ int main(int argc, char *argv[])
 	if (rc)
 		exit(EX_IOERR);
 
-	struct file *s = args.send;
+	struct send *s = args.send;
 	while (s) {
-		struct file *next = s->next;
+		struct send *next = s->next;
 		if (s != args.send)
 			usleep(args.gap);
 
+		if (s->is_keycode) {
+			struct send *k;
+
+			if (!args.keymap) {
+				fprintf(stderr, _("error: no keymap specified\n"));
+				exit(EX_DATAERR);
+			}
+
+			k = convert_keycode(args.keymap, s->keycode);
+			if (!k)
+				exit(EX_DATAERR);
+
+			free(s);
+			s = k;
+		}
+
 		rc = lirc_send(&args, fd, features, s);
 		if (rc) {
 			close(fd);
@@ -977,6 +1285,7 @@ int main(int argc, char *argv[])
 	if (args.features)
 		lirc_features(&args, fd, features);
 
+	free_keymap(args.keymap);
 	close(fd);
 
 	return 0;
