@@ -25,6 +25,7 @@
 #include <linux/lirc.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
 #include <dirent.h>
 #include <argp.h>
@@ -33,7 +34,7 @@
 
 #include "ir-encode.h"
 #include "parse.h"
-#include "toml.h"
+#include "keymap.h"
 
 #ifdef HAVE_BPF
 #include "bpf.h"
@@ -80,7 +81,13 @@ struct keytable_entry {
 	struct keytable_entry *next;
 };
 
+// Whenever for each key which has a raw entry rather than a scancode,
+// we need to assign a globally unique scancode for dealing with reading
+// more than keymap with raw entries.
+static int raw_scancode = 0;
+
 struct keytable_entry *keytable = NULL;
+struct raw_entry *rawtable = NULL;
 
 struct uevents {
 	char		*key;
@@ -95,7 +102,7 @@ struct cfgfile {
 	struct cfgfile	*next;
 };
 
-struct sysfs_names  {
+struct sysfs_names {
 	char			*name;
 	struct sysfs_names	*next;
 };
@@ -127,6 +134,8 @@ enum sysfs_protocols {
 	SYSFS_XMP		= (1 << 12),
 	SYSFS_CEC		= (1 << 13),
 	SYSFS_IMON		= (1 << 14),
+	SYSFS_RCMM		= (1 << 15),
+	SYSFS_XBOX_DVD		= (1 << 16),
 	SYSFS_INVALID		= 0,
 };
 
@@ -161,6 +170,8 @@ const struct protocol_map_entry protocol_map[] = {
 	{ "xmp",	"/xmp_decoder",	SYSFS_XMP	},
 	{ "cec",	NULL,		SYSFS_CEC	},
 	{ "imon",	NULL,		SYSFS_IMON	},
+	{ "rc-mm",	NULL,		SYSFS_RCMM	},
+	{ "xbox-dvd",	NULL,		SYSFS_XBOX_DVD	},
 	{ NULL,		NULL,		SYSFS_INVALID	},
 };
 
@@ -225,15 +236,18 @@ const char *argp_program_version = "IR keytable control version " V4L_UTILS_VERS
 const char *argp_program_bug_address = "Mauro Carvalho Chehab <mchehab@kernel.org>";
 
 static const char doc[] = N_(
-	"\nAllows get/set IR keycode/scancode tables\n"
+	"\nLists Remote Controller devices, loads rc keymaps, tests events, and adjusts\n"
+	"other Remote Controller options. Rather than loading a rc keymap, it is also\n"
+	"possible to set protocol decoders and set rc scancode to keycode mappings\n"
+	"directly.\n"
+
 	"You need to have read permissions on /dev/input for the program to work\n"
 	"\nOn the options below, the arguments are:\n"
-	"  DEV       - the /dev/input/event* device to control\n"
-	"  SYSDEV    - the ir class as found at /sys/class/rc\n"
-	"  TABLE     - a file with a set of scancode=keycode value pairs\n"
+	"  SYSDEV    - the rc device as found at /sys/class/rc\n"
+	"  KEYMAP    - a keymap file with protocols and scancode to keycode mappings\n"
 	"  SCANKEY   - a set of scancode1=keycode1,scancode2=keycode2.. value pairs\n"
 	"  PROTOCOL  - protocol name (nec, rc-5, rc-6, jvc, sony, sanyo, rc-5-sz, lirc,\n"
-	"              sharp, mce_kbd, xmp, imon, other, all) to be enabled,\n"
+	"              sharp, mce_kbd, xmp, imon, rc-mm, other, all) to be enabled,\n"
 	"              or a bpf protocol name or file\n"
 	"  DELAY     - Delay before repeating a keystroke\n"
 	"  PERIOD    - Period to repeat a keystroke\n"
@@ -244,54 +258,45 @@ static const char doc[] = N_(
 
 static const struct argp_option options[] = {
 	{"verbose",	'v',	0,		0,	N_("enables debug messages"), 0},
-	{"clear",	'c',	0,		0,	N_("clears the old table"), 0},
-	{"sysdev",	's',	N_("SYSDEV"),	0,	N_("ir class device to control"), 0},
+	{"clear",	'c',	0,		0,	N_("Clears the scancode to keycode mappings"), 0},
+	{"sysdev",	's',	N_("SYSDEV"),	0,	N_("rc device to control, defaults to rc0 if not specified"), 0},
 	{"test",	't',	0,		0,	N_("test if IR is generating events"), 0},
-	{"device",	'd',	N_("DEV"),	0,	N_("ir device to control"), 0},
-	{"read",	'r',	0,		0,	N_("reads the current scancode/keycode table"), 0},
-	{"write",	'w',	N_("TABLE"),	0,	N_("write (adds) the scancodes to the device scancode/keycode table from an specified file"), 0},
+	{"read",	'r',	0,		0,	N_("reads the current scancode/keycode mapping"), 0},
+	{"write",	'w',	N_("KEYMAP"),	0,	N_("write (adds) the keymap from the specified file"), 0},
 	{"set-key",	'k',	N_("SCANKEY"),	0,	N_("Change scan/key pairs"), 0},
 	{"protocol",	'p',	N_("PROTOCOL"),	0,	N_("Protocol to enable (the other ones will be disabled). To enable more than one, use the option more than one time"), 0},
 	{"parameter",	'e',	N_("PARAMETER"), 0,	N_("Set a parameter for the protocol decoder")},
 	{"delay",	'D',	N_("DELAY"),	0,	N_("Sets the delay before repeating a keystroke"), 0},
 	{"period",	'P',	N_("PERIOD"),	0,	N_("Sets the period to repeat a keystroke"), 0},
-	{"auto-load",	'a',	N_("CFGFILE"),	0,	N_("Auto-load a table, based on a configuration file. Only works with sysdev."), 0},
+	{"auto-load",	'a',	N_("CFGFILE"),	0,	N_("Auto-load keymaps, based on a configuration file. Only works with --sysdev."), 0},
+	{"test-keymap",	1,	N_("KEYMAP"),	0,	N_("Test if keymap is valid"), 0},
 	{"help",        '?',	0,		0,	N_("Give this help list"), -1},
 	{"usage",	-3,	0,		0,	N_("Give a short usage message")},
 	{"version",	'V',	0,		0,	N_("Print program version"), -1},
 	{ 0, 0, 0, 0, 0, 0 }
 };
 
-static const char args_doc[] = N_(
-	"--device [/dev/input/event* device]\n"
-	"--sysdev [ir class (f. ex. rc0)]\n"
-	"[for using the rc0 sysdev]");
+static const char args_doc[] = N_("");
 
 /* Static vars to store the parameters */
 static char *devclass = NULL;
-static char *devicename = NULL;
 static int readtable = 0;
 static int clear = 0;
 int debug = 0;
 static int test = 0;
 static int delay = -1;
 static int period = -1;
+static int test_keymap = 0;
 static enum sysfs_protocols ch_proto = 0;
 
 struct bpf_protocol {
 	struct bpf_protocol *next;
-	struct toml_table_t *toml;
+	struct protocol_param *param;
 	char *name;
 };
 
-struct bpf_parameter {
-	struct bpf_parameter *next;
-	int value;
-	char name[0];
-};
-
 static struct bpf_protocol *bpf_protocol;
-static struct bpf_parameter *bpf_parameter;
+static struct protocol_param *bpf_parameter;
 
 struct cfgfile cfg = {
 	NULL, NULL, NULL, NULL
@@ -320,29 +325,21 @@ struct rc_device {
 	enum sysfs_protocols supported, current; /* Current and supported IR protocols */
 };
 
-static bool compare_parameters(struct toml_table_t *a, struct toml_table_t *b)
+static bool compare_parameters(struct protocol_param *a, struct protocol_param *b)
 {
-	int i = 0;
-	int64_t avalue, bvalue;
-	const char *name, *raw;
+	struct protocol_param *b2;
 
-	while ((name = toml_key_in(a, i++)) != NULL) {
-		raw = toml_raw_in(a, name);
-		if (!raw)
-			continue;
+	for (; a; a = a->next) {
+		for (b2 = b; b2; b2 = b2->next) {
+			if (!strcmp(a->name, b2->name) &&
+			    a->value == b2->value) {
+				break;
+			}
+		}
 
-		if (toml_rtoi(raw, &avalue))
-			continue;
-
-		raw = toml_raw_in(b, name);
-		if (!raw)
-			return false;
-
-		if (toml_rtoi(raw, &bvalue))
-			return false;
-
-		if (avalue != bvalue)
+		if (!b2) {
 			return false;
+		}
 	}
 
 	return true;
@@ -361,8 +358,8 @@ static void add_bpf_protocol(struct bpf_protocol *new)
 		if (strcmp(a->name, new->name))
 			continue;
 
-		if (compare_parameters(a->toml, new->toml) &&
-		    compare_parameters(new->toml, a->toml))
+		if (compare_parameters(a->param, new->param) &&
+		    compare_parameters(new->param, a->param))
 			return;
 	}
 
@@ -370,278 +367,96 @@ static void add_bpf_protocol(struct bpf_protocol *new)
 	bpf_protocol = new;
 }
 
-static error_t parse_plain_keyfile(char *fname, char **table)
+static int add_keymap(struct keymap *map, const char *fname)
 {
-	FILE *fin;
-	int value, line = 0;
-	char *scancode, *keycode, s[2048];
-	struct keytable_entry *ke;
-
-	*table = NULL;
-
-	if (debug)
-		fprintf(stderr, _("Parsing %s keycode file as plain text\n"), fname);
-
-	fin = fopen(fname, "r");
-	if (!fin) {
-		return errno;
-	}
-
-	while (fgets(s, sizeof(s), fin)) {
-		char *p = s;
-
-		line++;
-		while (*p == ' ' || *p == '\t')
-			p++;
-		if (line==1 && p[0] == '#') {
-			p++;
-			p = strtok(p, "\n\t =:");
-			do {
-				if (!p)
-					goto err_einval;
-				if (!strcmp(p, "table")) {
-					p = strtok(NULL,"\n, ");
-					if (!p)
-						goto err_einval;
-					*table = malloc(strlen(p) + 1);
-					strcpy(*table, p);
-				} else if (!strcmp(p, "type")) {
-					p = strtok(NULL, " ,\n");
-					if (!p)
-						goto err_einval;
-
-					while (p) {
-						enum sysfs_protocols protocol;
-
-						protocol = parse_sysfs_protocol(p, false);
-						if (protocol == SYSFS_INVALID) {
-							fprintf(stderr, _("Protocol %s invalid\n"), p);
-							goto err_einval;
-						}
-						ch_proto |= protocol;
-						p = strtok(NULL, " ,\n");
-					}
-				} else {
-					goto err_einval;
-				}
-				p = strtok(NULL, "\n\t =:");
-			} while (p);
-			continue;
-		}
-
-		if (*p == '\n' || *p == '#')
-			continue;
-
-		scancode = strtok(p, "\n\t =:");
-		if (!scancode)
-			goto err_einval;
-		if (!strcasecmp(scancode, "scancode")) {
-			scancode = strtok(NULL, "\n\t =:");
-			if (!scancode)
-				goto err_einval;
-		}
-
-		keycode = strtok(NULL, "\n\t =:(");
-		if (!keycode)
-			goto err_einval;
-
-		if (debug)
-			fprintf(stderr, _("parsing %s=%s:"), scancode, keycode);
-		value = parse_code(keycode);
-		if (debug)
-			fprintf(stderr, _("\tvalue=%d\n"), value);
-
-		if (value == -1) {
-			value = strtol(keycode, NULL, 0);
-			if (errno)
-				perror(_("value"));
-		}
-
-		ke = calloc(1, sizeof(*ke));
-		if (!ke) {
-			perror("parse_keyfile");
-			return ENOMEM;
-		}
-
-		ke->scancode	= strtoul(scancode, NULL, 0);
-		ke->keycode	= value;
-		ke->next	= keytable;
-		keytable	= ke;
-	}
-	fclose(fin);
-
-	return 0;
-
-err_einval:
-	fprintf(stderr, _("Invalid parameter on line %d of %s\n"),
-		line, fname);
-	return EINVAL;
-}
-
-static error_t parse_toml_protocol(struct toml_table_t *proot)
-{
-	struct toml_table_t *scancodes;
-	enum sysfs_protocols protocol;
-	const char *raw;
-	char *p;
-	int i = 0;
-
-	raw = toml_raw_in(proot, "protocol");
-	if (!raw) {
-		fprintf(stderr, _("protocol missing\n"));
-		return EINVAL;
-	}
-
-	if (toml_rtos(raw, &p)) {
-		fprintf(stderr, _("Bad value `%s' for protocol\n"), raw);
-		return EINVAL;
-	}
-
-	protocol = parse_sysfs_protocol(p, false);
-	if (protocol == SYSFS_INVALID) {
-		struct bpf_protocol *b;
-
-		b = malloc(sizeof(*b));
-		b->name = p;
-		b->toml = proot;
-		add_bpf_protocol(b);
-	}
-	else {
-		ch_proto |= protocol;
-		free(p);
-	}
-
-	scancodes = toml_table_in(proot, "scancodes");
-	if (!scancodes) {
-		if (debug)
-			fprintf(stderr, _("No [protocols.scancodes] section\n"));
-		return 0;
-	}
-
-	for (;;) {
+	for (; map; map = map->next) {
+		enum sysfs_protocols protocol;
 		struct keytable_entry *ke;
-		const char *scancode;
-		char *keycode;
-		int value;
+		struct scancode_entry *se;
+		struct raw_entry *re, *re_next;
 
-		scancode = toml_key_in(scancodes, i++);
-		if (!scancode)
-			break;
+		protocol = parse_sysfs_protocol(map->protocol, false);
+		if (protocol == SYSFS_INVALID) {
+			struct bpf_protocol *b;
 
-		raw = toml_raw_in(scancodes, scancode);
-		if (!raw) {
-			fprintf(stderr, _("Invalid value `%s'\n"), scancode);
-			return EINVAL;
+			b = malloc(sizeof(*b));
+			b->name = strdup(map->protocol);
+			b->param = map->param;
+			/* steal param */
+			map->param = NULL;
+			add_bpf_protocol(b);
+		} else {
+			ch_proto |= protocol;
 		}
 
-		if (toml_rtos(raw, &keycode)) {
-			fprintf(stderr, _("Bad value `%s' for keycode\n"),
-				keycode);
-			return EINVAL;
+		for (se = map->scancode; se; se = se->next) {
+			int value;
+			char *p;
+
+			value = parse_code(se->keycode);
+			if (debug)
+				fprintf(stderr, _("\tvalue=%d\n"), value);
+
+			if (value == -1) {
+				value = strtol(se->keycode, &p, 0);
+				if (errno || *p) {
+					fprintf(stderr, _("%s: keycode `%s' not recognised, no mapping for scancode %d\n"), fname, se->keycode, se->scancode);
+					continue;
+				}
+			}
+
+			ke = malloc(sizeof(*ke));
+			ke->scancode = se->scancode;
+			ke->keycode = value;
+			ke->next = keytable;
+
+			keytable = ke;
 		}
 
-		if (debug)
-			fprintf(stderr, _("parsing %s=%s:"), scancode, keycode);
+		for (re = map->raw; re; re = re_next) {
+			int value;
+			char *p;
 
-		value = parse_code(keycode);
-		if (debug)
-			fprintf(stderr, _("\tvalue=%d\n"), value);
+			re_next = re->next;
 
-		if (value == -1) {
-			value = strtol(keycode, &p, 0);
-			if (errno || *p)
-				fprintf(stderr, _("keycode `%s' not recognised, no mapping for scancode %s\n"), keycode, scancode);
-		}
-		free(keycode);
+			value = parse_code(re->keycode);
+			if (debug)
+				fprintf(stderr, _("\tvalue=%d\n"), value);
 
-		ke = calloc(1, sizeof(*ke));
-		if (!ke) {
-			perror("parse_keyfile");
-			return ENOMEM;
+			if (value == -1) {
+				value = strtol(re->keycode, &p, 0);
+				if (errno || *p) {
+					fprintf(stderr, _("%s: keycode `%s' not recognised, no mapping\n"), fname, re->keycode);
+					continue;
+				}
+			}
+
+			ke = malloc(sizeof(*ke));
+			ke->scancode = raw_scancode;
+			ke->keycode = value;
+			ke->next = keytable;
+
+			keytable = ke;
+
+			re->scancode = raw_scancode++;
+			re->next = rawtable;
+			rawtable = re;
 		}
 
-		ke->scancode	= strtoul(scancode, NULL, 0);
-		ke->keycode	= value;
-		ke->next	= keytable;
-		keytable	= ke;
+		/* Steal the raw entries */
+		map->raw = NULL;
 	}
 
 	return 0;
 }
 
-static error_t parse_toml_keyfile(char *fname, char **table)
-{
-	struct toml_table_t *root, *proot;
-	struct toml_array_t *arr;
-	int ret, i = 0;
-	char buf[200];
-	FILE *fin;
-
-	*table = NULL;
-
-	if (debug)
-		fprintf(stderr, _("Parsing %s keycode file as toml\n"), fname);
-
-	fin = fopen(fname, "r");
-	if (!fin)
-		return errno;
-
-	root = toml_parse_file(fin, buf, sizeof(buf));
-	fclose(fin);
-	if (!root) {
-		fprintf(stderr, _("Failed to parse toml: %s\n"), buf);
-		return EINVAL;
-	}
-
-	arr = toml_array_in(root, "protocols");
-	if (!arr) {
-		fprintf(stderr, _("Missing [protocols] section\n"));
-		return EINVAL;
-	}
-
-	for (;;) {
-		proot = toml_table_at(arr, i);
-		if (!proot)
-			break;
-
-		ret = parse_toml_protocol(proot);
-		if (ret)
-			goto out;
-
-		i++;
-	}
-
-	if (i == 0) {
-		fprintf(stderr, _("No protocols found\n"));
-		goto out;
-	}
-
-	// Don't free toml, this is used during bpf loading */
-	//toml_free(root);
-	return 0;
-out:
-	toml_free(root);
-	return EINVAL;
-}
-
-static error_t parse_keyfile(char *fname, char **table)
-{
-	size_t len = strlen(fname);
-
-	if (len >= 5 && strcasecmp(fname + len - 5, ".toml") == 0)
-		return parse_toml_keyfile(fname, table);
-	else
-		return parse_plain_keyfile(fname, table);
-}
-
-struct cfgfile *nextcfg = &cfg;
-
 static error_t parse_cfgfile(char *fname)
 {
 	FILE *fin;
 	int line = 0;
 	char s[2048];
 	char *driver, *table, *filename;
+	struct cfgfile *nextcfg = &cfg;
 
 	if (debug)
 		fprintf(stderr, _("Parsing %s config file\n"), fname);
@@ -731,9 +546,6 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		if (!p || *p || period < 0)
 			argp_error(state, _("Invalid period: %s"), arg);
 		break;
-	case 'd':
-		devicename = arg;
-		break;
 	case 's':
 		devclass = arg;
 		break;
@@ -741,13 +553,18 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		readtable++;
 		break;
 	case 'w': {
-		char *name = NULL;
+		struct keymap *map = NULL;
 
-		rc = parse_keyfile(arg, &name);
-		if (rc)
+		rc = parse_keymap(arg, &map, debug);
+		if (rc) {
 			argp_error(state, _("Failed to read table file %s"), arg);
-		if (name)
-			fprintf(stderr, _("Read %s table\n"), name);
+			break;
+		}
+		if (map->name)
+			fprintf(stderr, _("Read %s table\n"), map->name);
+
+		add_keymap(map, arg);
+		free_keymap(map);
 		break;
 	}
 	case 'a': {
@@ -818,7 +635,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 
 				b = malloc(sizeof(*b));
 				b->name = strdup(p);
-				b->toml = NULL;
+				b->param = NULL;
 				b->next = bpf_protocol;
 				bpf_protocol = b;
 			}
@@ -830,20 +647,20 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 	case 'e':
 		p = strtok(arg, ":=");
 		do {
-			struct bpf_parameter *param;
+			struct protocol_param *param;
 
 			if (!p) {
 				argp_error(state, _("Missing parameter name: %s"), arg);
 				break;
 			}
 
-			param = calloc(1, sizeof(*param) + strlen(p) + 1);
+			param = malloc(sizeof(*param));
 			if (!p) {
 				perror(_("No memory!\n"));
 				return ENOMEM;
 			}
 
-			strcpy(param->name, p);
+			param->name = strdup(p);
 
 			p = strtok(NULL, ",;");
 			if (!p) {
@@ -860,7 +677,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 			}
 
 			if (debug)
-				fprintf(stderr, _("parameter %s=%d\n"),
+				fprintf(stderr, _("parameter %s=%ld\n"),
 					param->name, param->value);
 
 			param->next = bpf_parameter;
@@ -869,7 +686,16 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 			p = strtok(NULL, ":=");
 		} while (p);
 		break;
+	case 1:
+		test_keymap++;
+		struct keymap *map ;
 
+		rc = parse_keymap(arg, &map, debug);
+		if (rc)
+			argp_error(state, _("Failed to read table file %s"), arg);
+		add_keymap(map, arg);
+		free_keymap(map);
+		break;
 	case '?':
 		argp_state_help(state, state->out_stream,
 				ARGP_HELP_SHORT_USAGE | ARGP_HELP_LONG
@@ -1114,6 +940,27 @@ static struct sysfs_names *find_device(char *name)
 	return names;
 }
 
+static enum sysfs_protocols load_bpf_for_unsupported(enum sysfs_protocols protocols, enum sysfs_protocols supported)
+{
+	const struct protocol_map_entry *pme;
+	struct bpf_protocol *b;
+
+	for (pme = protocol_map; pme->name; pme++) {
+		if (!(protocols & pme->sysfs_protocol) ||
+		    (supported & pme->sysfs_protocol))
+			continue;
+
+		b = malloc(sizeof(*b));
+		b->name = strdup(pme->name);
+		b->param = NULL;
+		add_bpf_protocol(b);
+
+		protocols &= ~pme->sysfs_protocol;
+	}
+
+	return protocols;
+}
+
 static enum sysfs_protocols v1_get_hw_protocols(char *name)
 {
 	FILE *fp;
@@ -1470,13 +1317,13 @@ static int set_proto(struct rc_device *rc_dev)
 {
 	int rc = 0;
 
-	rc_dev->current &= rc_dev->supported;
-
 	if (rc_dev->version == VERSION_2) {
 		rc = v2_set_protocols(rc_dev);
 		return rc;
 	}
 
+	rc_dev->current &= rc_dev->supported;
+
 	if (rc_dev->type == SOFTWARE_DECODER) {
 		const struct protocol_map_entry *pme;
 
@@ -1601,7 +1448,7 @@ static void print_scancodes(const struct lirc_scancode *scancodes, unsigned coun
 {
 	unsigned i;
 
-	for (i=0; i< count; i++)  {
+	for (i = 0; i < count; i++) {
 		const char *p = protocol_name(scancodes[i].rc_proto);
 
 		printf(_("%llu.%06llu: "),
@@ -1852,31 +1699,46 @@ static void device_info(int fd, char *prepend)
 
 #ifdef HAVE_BPF
 #define MAX_PROGS 64
-static void attach_bpf(const char *lirc_name, const char *bpf_prog, struct toml_table_t *toml)
+// This value is what systemd sets PID 1 to, see:
+// https://github.com/systemd/systemd/blob/master/src/basic/def.h#L60
+#define HIGH_RLIMIT_MEMLOCK (1024ULL*1024ULL*64ULL)
+
+static bool attach_bpf(const char *lirc_name, const char *bpf_prog, struct protocol_param *param)
 {
 	unsigned int features;
-	int fd;
+	struct rlimit rl;
+	int fd, ret;
 
 	fd = open(lirc_name, O_RDONLY);
 	if (fd == -1) {
 		perror(lirc_name);
-		return;
+		return false;
 	}
 
 	if (ioctl(fd, LIRC_GET_FEATURES, &features)) {
 		perror(lirc_name);
 		close(fd);
-		return;
+		return false;
 	}
 
 	if (!(features & LIRC_CAN_REC_MODE2)) {
 		fprintf(stderr, _("%s: not a raw IR receiver\n"), lirc_name);
 		close(fd);
-		return;
+		return false;
 	}
 
-	load_bpf_file(bpf_prog, fd, toml);
+	// BPF programs are charged against RLIMIT_MEMLOCK. We'll need pages
+	// for the state, program text, and any raw IR. None of these are
+	// particularly large. However, the kernel defaults to 64KB
+	// memlock, which is only 16 pages which are mostly used by the
+	// time we are trying to load our BPF program.
+	rl.rlim_cur = rl.rlim_max = HIGH_RLIMIT_MEMLOCK;
+	(void) setrlimit(RLIMIT_MEMLOCK, &rl);
+
+	ret = load_bpf_file(bpf_prog, fd, param, rawtable);
 	close(fd);
+
+	return ret == 0;
 }
 
 static void show_bpf(const char *lirc_name)
@@ -1983,9 +1845,10 @@ static void clear_bpf(const char *lirc_name)
 		fprintf(stderr, _("BPF protocols removed\n"));
 }
 #else
-static void attach_bpf(const char *lirc_name, const char *bpf_prog, struct toml_table_t *toml)
+static bool attach_bpf(const char *lirc_name, const char *bpf_prog, struct protocol_param *param)
 {
 	fprintf(stderr, _("error: ir-keytable was compiled without BPF support\n"));
+	return false;
 }
 static void show_bpf(const char *lirc_name) {}
 static void clear_bpf(const char *lirc_name) {}
@@ -2003,15 +1866,17 @@ static int show_sysfs_attribs(struct rc_device *rc_dev, char *name)
 		if (cur->name) {
 			if (get_attribs(rc_dev, cur->name))
 				continue;
-			fprintf(stderr, _("Found %s (%s) with:\n"),
-				rc_dev->sysfs_name,
-				rc_dev->input_name);
+			fprintf(stderr, _("Found %s with:\n"),
+				rc_dev->sysfs_name);
 			if (rc_dev->dev_name)
 				fprintf(stderr, _("\tName: %s\n"),
 					rc_dev->dev_name);
-			fprintf(stderr, _("\tDriver: %s, table: %s\n"),
-				rc_dev->drv_name,
+			fprintf(stderr, _("\tDriver: %s\n"),
+				rc_dev->drv_name);
+			fprintf(stderr, _("\tDefault keymap: %s\n"),
 				rc_dev->keytable_name);
+			fprintf(stderr, _("\tInput device: %s\n"),
+				rc_dev->input_name);
 			if (rc_dev->lirc_name) {
 				fprintf(stderr, _("\tLIRC device: %s\n"),
 					rc_dev->lirc_name);
@@ -2066,9 +1931,9 @@ static char *find_bpf_file(const char *name)
 	return fname;
 }
 
-int bpf_param(const char *name, int *val)
+int bpf_param(struct protocol_param *protocol_param, const char *name, int *val)
 {
-	struct bpf_parameter *param = bpf_parameter;
+	struct protocol_param *param = bpf_parameter;
 
 	while (param) {
 		if (strcmp(name, param->name) == 0) {
@@ -2078,6 +1943,14 @@ int bpf_param(const char *name, int *val)
 		param = param->next;
 	}
 
+	while (protocol_param) {
+		if (strcmp(name, protocol_param->name) == 0) {
+			*val = protocol_param->value;
+			return 0;
+		}
+		protocol_param = protocol_param->next;
+	}
+
 	return -ENOENT;
 }
 
@@ -2096,19 +1969,11 @@ int main(int argc, char *argv[])
 
 	argp_parse(&argp, argc, argv, ARGP_NO_HELP, 0, 0);
 
+	if (test_keymap)
+		return 0;
+
 	/* Just list all devices */
 	if (!clear && !readtable && !keytable && !ch_proto && !cfg.next && !test && delay < 0 && period < 0 && !bpf_protocol) {
-		if (devicename) {
-			fd = open(devicename, O_RDONLY);
-			if (fd < 0) {
-				perror(_("Can't open device"));
-				return -1;
-			}
-			device_name(fd, "");
-			device_info(fd, "");
-			close(fd);
-			return 0;
-		}
 		if (show_sysfs_attribs(&rc_dev, devclass))
 			return -1;
 
@@ -2118,90 +1983,98 @@ int main(int argc, char *argv[])
 	if (!devclass)
 		devclass = "rc0";
 
-	if (cfg.next && (clear || keytable || ch_proto || devicename)) {
-		fprintf (stderr, _("Auto-mode can be used only with --read, --debug and --sysdev options\n"));
+	if (cfg.next && (clear || keytable || ch_proto)) {
+		fprintf (stderr, _("Auto-mode can be used only with --read, --verbose and --sysdev options\n"));
 		return -1;
 	}
-	if (!devicename) {
-		names = find_device(devclass);
-		if (!names)
-			return -1;
-		rc_dev.sysfs_name = names->name;
-		if (get_attribs(&rc_dev, names->name)) {
-			free_names(names);
-			return -1;
-		}
-		names->name = NULL;
-		free_names(names);
 
-		devicename = rc_dev.input_name;
-		dev_from_class++;
+	names = find_device(devclass);
+	if (!names)
+		return -1;
+	rc_dev.sysfs_name = names->name;
+	if (get_attribs(&rc_dev, names->name)) {
+		free_names(names);
+		return -1;
 	}
+	names->name = NULL;
+	free_names(names);
+
+	dev_from_class++;
 
 	if (cfg.next) {
 		struct cfgfile *cur;
-		char *fname, *name;
+		char *fname;
 		int rc;
+		int matches = 0;
 
 		for (cur = &cfg; cur->next; cur = cur->next) {
 			if ((!rc_dev.drv_name || strcasecmp(cur->driver, rc_dev.drv_name)) && strcasecmp(cur->driver, "*"))
 				continue;
 			if ((!rc_dev.keytable_name || strcasecmp(cur->table, rc_dev.keytable_name)) && strcasecmp(cur->table, "*"))
 				continue;
-			break;
+
+			if (debug)
+				fprintf(stderr, _("Table for %s, %s is on %s file.\n"),
+					rc_dev.drv_name, rc_dev.keytable_name,
+					cur->fname);
+
+			if (cur->fname[0] == '/' || ((cur->fname[0] == '.') && strchr(cur->fname, '/'))) {
+				struct keymap *map;
+				fname = cur->fname;
+				rc = parse_keymap(fname, &map, debug);
+				if (rc < 0) {
+					fprintf(stderr, _("Can't load %s table\n"), fname);
+					return -1;
+				}
+				add_keymap(map, fname);
+				free_keymap(map);
+			} else {
+				struct keymap *map;
+				fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_USER_DIR) + 2);
+				strcpy(fname, IR_KEYTABLE_USER_DIR);
+				strcat(fname, "/");
+				strcat(fname, cur->fname);
+				rc = parse_keymap(fname, &map, debug);
+				if (rc != 0) {
+					fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_SYSTEM_DIR) + 2);
+					strcpy(fname, IR_KEYTABLE_SYSTEM_DIR);
+					strcat(fname, "/");
+					strcat(fname, cur->fname);
+					rc = parse_keymap(fname, &map, debug);
+				}
+				if (rc != 0) {
+					fprintf(stderr, _("Can't load %s table from %s or %s\n"), cur->fname, IR_KEYTABLE_USER_DIR, IR_KEYTABLE_SYSTEM_DIR);
+					return -1;
+				}
+
+				add_keymap(map, fname);
+				free_keymap(map);
+			}
+			if (!keytable) {
+				fprintf(stderr, _("Empty table %s\n"), fname);
+				return -1;
+			}
+			clear = 1;
+			matches++;
 		}
 
-		if (!cur->next) {
+		if (!matches) {
 			if (debug)
 				fprintf(stderr, _("Table for %s, %s not found. Keep as-is\n"),
 				       rc_dev.drv_name, rc_dev.keytable_name);
 			return 0;
 		}
-		if (debug)
-			fprintf(stderr, _("Table for %s, %s is on %s file.\n"),
-				rc_dev.drv_name, rc_dev.keytable_name,
-				cur->fname);
-		if (cur->fname[0] == '/' || ((cur->fname[0] == '.') && strchr(cur->fname, '/'))) {
-			fname = cur->fname;
-			rc = parse_keyfile(fname, &name);
-			if (rc < 0) {
-				fprintf(stderr, _("Can't load %s table\n"), fname);
-				return -1;
-			}
-		} else {
-			fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_USER_DIR) + 2);
-			strcpy(fname, IR_KEYTABLE_USER_DIR);
-			strcat(fname, "/");
-			strcat(fname, cur->fname);
-			rc = parse_keyfile(fname, &name);
-			if (rc != 0) {
-				fname = malloc(strlen(cur->fname) + strlen(IR_KEYTABLE_SYSTEM_DIR) + 2);
-				strcpy(fname, IR_KEYTABLE_SYSTEM_DIR);
-				strcat(fname, "/");
-				strcat(fname, cur->fname);
-				rc = parse_keyfile(fname, &name);
-			}
-			if (rc != 0) {
-				fprintf(stderr, _("Can't load %s table from %s or %s\n"), cur->fname, IR_KEYTABLE_USER_DIR, IR_KEYTABLE_SYSTEM_DIR);
-				return -1;
-			}
-		}
-		if (!keytable) {
-			fprintf(stderr, _("Empty table %s\n"), fname);
-			return -1;
-		}
-		clear = 1;
 	}
 
 	if (debug)
-		fprintf(stderr, _("Opening %s\n"), devicename);
-	fd = open(devicename, O_RDONLY | O_NONBLOCK);
+		fprintf(stderr, _("Opening %s\n"), rc_dev.input_name);
+	fd = open(rc_dev.input_name, O_RDONLY | O_NONBLOCK);
 	if (fd < 0) {
-		perror(devicename);
+		perror(rc_dev.input_name);
 		return -1;
 	}
 	if (dev_from_class)
-		free(devicename);
+		free(rc_dev.input_name);
 	if (get_input_protocol_version(fd))
 		return -1;
 
@@ -2227,7 +2100,8 @@ int main(int argc, char *argv[])
 		if (rc_dev.lirc_name)
 			clear_bpf(rc_dev.lirc_name);
 
-		rc_dev.current = ch_proto;
+		rc_dev.current = load_bpf_for_unsupported(ch_proto, rc_dev.supported);
+
 		if (set_proto(&rc_dev))
 			fprintf(stderr, _("Couldn't change the IR protocols\n"));
 		else {
@@ -2248,8 +2122,8 @@ int main(int argc, char *argv[])
 			char *fname = find_bpf_file(b->name);
 
 			if (fname) {
-				attach_bpf(rc_dev.lirc_name, fname, b->toml);
-				fprintf(stderr, _("Loaded BPF protocol %s\n"), b->name);
+				if (attach_bpf(rc_dev.lirc_name, fname, b->param))
+					fprintf(stderr, _("Loaded BPF protocol %s\n"), b->name);
 				free(fname);
 			}
 		}
