@@ -151,6 +151,7 @@ struct parse_event key_events[] = {
 	{"KEY_SCALE", 120},
 	{"KEY_KPCOMMA", 121},
 	{"KEY_HANGEUL", 122},
+	{"KEY_HANGUEL", 122},
 	{"KEY_HANJA", 123},
 	{"KEY_YEN", 124},
 	{"KEY_LEFTMETA", 125},
@@ -181,7 +182,9 @@ struct parse_event key_events[] = {
 	{"KEY_WWW", 150},
 	{"KEY_MSDOS", 151},
 	{"KEY_COFFEE", 152},
+	{"KEY_SCREENLOCK", 152},
 	{"KEY_ROTATE_DISPLAY", 153},
+	{"KEY_DIRECTION", 153},
 	{"KEY_CYCLEWINDOWS", 154},
 	{"KEY_MAIL", 155},
 	{"KEY_BOOKMARKS", 156},
@@ -268,8 +271,10 @@ struct parse_event key_events[] = {
 	{"KEY_VIDEO_PREV", 242},
 	{"KEY_BRIGHTNESS_CYCLE", 243},
 	{"KEY_BRIGHTNESS_AUTO", 244},
+	{"KEY_BRIGHTNESS_ZERO", 244},
 	{"KEY_DISPLAY_OFF", 245},
 	{"KEY_WWAN", 246},
+	{"KEY_WIMAX", 246},
 	{"KEY_RFKILL", 247},
 	{"KEY_MICMUTE", 248},
 	{"BTN_MISC", 0x100},
@@ -362,9 +367,11 @@ struct parse_event key_events[] = {
 	{"KEY_TITLE", 0x171},
 	{"KEY_SUBTITLE", 0x172},
 	{"KEY_ANGLE", 0x173},
+	{"KEY_FULL_SCREEN", 0x174},
 	{"KEY_ZOOM", 0x174},
 	{"KEY_MODE", 0x175},
 	{"KEY_KEYBOARD", 0x176},
+	{"KEY_ASPECT_RATIO", 0x177},
 	{"KEY_SCREEN", 0x177},
 	{"KEY_PC", 0x178},
 	{"KEY_TV", 0x179},
@@ -422,6 +429,7 @@ struct parse_event key_events[] = {
 	{"KEY_ADDRESSBOOK", 0x1ad},
 	{"KEY_MESSENGER", 0x1ae},
 	{"KEY_DISPLAYTOGGLE", 0x1af},
+	{"KEY_BRIGHTNESS_TOGGLE", 0x1af},
 	{"KEY_SPELLCHECK", 0x1b0},
 	{"KEY_LOGOFF", 0x1b1},
 	{"KEY_DOLLAR", 0x1b2},
@@ -513,6 +521,7 @@ struct parse_event key_events[] = {
 	{"KEY_SCREENSAVER", 0x245},
 	{"KEY_VOICECOMMAND", 0x246},
 	{"KEY_ASSISTANT", 0x247},
+	{"KEY_KBD_LAYOUT_NEXT", 0x248},
 	{"KEY_BRIGHTNESS_MIN", 0x250},
 	{"KEY_BRIGHTNESS_MAX", 0x251},
 	{"KEY_KBDINPUTASSIST_PREV", 0x260},
@@ -581,7 +590,6 @@ struct parse_event key_events[] = {
 	{"BTN_TRIGGER_HAPPY38", 0x2e5},
 	{"BTN_TRIGGER_HAPPY39", 0x2e6},
 	{"BTN_TRIGGER_HAPPY40", 0x2e7},
-	{"KEY_MAX", 0x2ff},
 	{ NULL, 0}
 };
 struct parse_event rel_events[] = {
@@ -595,6 +603,9 @@ struct parse_event rel_events[] = {
 	{"REL_DIAL", 0x07},
 	{"REL_WHEEL", 0x08},
 	{"REL_MISC", 0x09},
+	{"REL_RESERVED", 0x0a},
+	{"REL_WHEEL_HI_RES", 0x0b},
+	{"REL_HWHEEL_HI_RES", 0x0c},
 	{"REL_MAX", 0x0f},
 	{ NULL, 0}
 };
@@ -625,6 +636,7 @@ struct parse_event abs_events[] = {
 	{"ABS_TOOL_WIDTH", 0x1c},
 	{"ABS_VOLUME", 0x20},
 	{"ABS_MISC", 0x28},
+	{"ABS_RESERVED", 0x2e},
 	{"ABS_MT_SLOT", 0x2f},
 	{"ABS_MT_TOUCH_MAJOR", 0x30},
 	{"ABS_MT_TOUCH_MINOR", 0x31},
