@@ -35,6 +35,8 @@ enum device_type {
 	MEDIA_V4L_VIDEO = 0,
 	MEDIA_V4L_VBI,
 	MEDIA_V4L_RADIO,
+	MEDIA_V4L_SWRADIO,
+	MEDIA_V4L_TOUCH,
 	MEDIA_V4L_SUBDEV,
 
 	MEDIA_DVB_VIDEO = 100,
