@@ -54,6 +54,7 @@
 enum Option {
 	OptStreamAllIO = 'a',
 	OptStreamAllColorTest = 'c',
+	OptColor = 'C',
 	OptSetDevice = 'd',
 	OptSetExpBufDevice = 'e',
 	OptExitOnFail = 'E',
@@ -62,6 +63,7 @@ enum Option {
 	OptSetMediaDevice = 'm',
 	OptSetMediaDeviceOnly = 'M',
 	OptNoWarnings = 'n',
+	OptNoProgress = 'P',
 	OptSetRadioDevice = 'r',
 	OptStreaming = 's',
 	OptSetSWRadioDevice = 'S',
@@ -72,6 +74,9 @@ enum Option {
 	OptSetVbiDevice = 'V',
 	OptUseWrapper = 'w',
 	OptExitOnWarn = 'W',
+	OptMediaBusInfo = 'z',
+	OptStreamFrom = 128,
+	OptStreamFromHdr,
 	OptLast = 256
 };
 
@@ -79,12 +84,16 @@ static char options[OptLast];
 
 static int app_result;
 static int tests_total, tests_ok;
+static int grand_total, grand_ok, grand_warnings;
 
 // Globals
 bool show_info;
+bool no_progress;
 bool show_warnings = true;
+bool show_colors;
 bool exit_on_fail;
 bool exit_on_warn;
+bool is_vivid;
 int kernel_version;
 int media_fd = -1;
 unsigned warnings;
@@ -125,9 +134,12 @@ static struct option long_options[] = {
 	{"touch-device", required_argument, 0, OptSetTouchDevice},
 	{"media-device", required_argument, 0, OptSetMediaDevice},
 	{"media-device-only", required_argument, 0, OptSetMediaDeviceOnly},
+	{"media-bus-info", required_argument, 0, OptMediaBusInfo},
 	{"help", no_argument, 0, OptHelp},
 	{"verbose", no_argument, 0, OptVerbose},
+	{"color", required_argument, 0, OptColor},
 	{"no-warnings", no_argument, 0, OptNoWarnings},
+	{"no-progress", no_argument, 0, OptNoProgress},
 	{"exit-on-fail", no_argument, 0, OptExitOnFail},
 	{"exit-on-warn", no_argument, 0, OptExitOnWarn},
 	{"trace", no_argument, 0, OptTrace},
@@ -135,7 +147,9 @@ static struct option long_options[] = {
 	{"wrapper", no_argument, 0, OptUseWrapper},
 #endif
 	{"streaming", optional_argument, 0, OptStreaming},
-	{"stream-all-formats", no_argument, 0, OptStreamAllFormats},
+	{"stream-from", required_argument, 0, OptStreamFrom},
+	{"stream-from-hdr", required_argument, 0, OptStreamFromHdr},
+	{"stream-all-formats", optional_argument, 0, OptStreamAllFormats},
 	{"stream-all-io", no_argument, 0, OptStreamAllIO},
 	{"stream-all-color", required_argument, 0, OptStreamAllColorTest},
 	{0, 0, 0, 0}
@@ -147,43 +161,68 @@ static void usage(void)
 	printf("Common options:\n");
 	printf("  -d, --device <dev> Use device <dev> as the video device.\n");
 	printf("                     If <dev> starts with a digit, then /dev/video<dev> is used.\n");
+	printf("                     Otherwise if -z was specified earlier, then <dev> is the entity name\n");
+	printf("                     or interface ID (if prefixed with 0x) as found in the topology of the\n");
+	printf("                     media device with the bus info string as specified by the -z option.\n");
 	printf("  -V, --vbi-device <dev>\n");
 	printf("                     Use device <dev> as the vbi device.\n");
 	printf("                     If <dev> starts with a digit, then /dev/vbi<dev> is used.\n");
+	printf("                     See the -d description of how <dev> is used in combination with -z.\n");
 	printf("  -r, --radio-device <dev>\n");
 	printf("                     Use device <dev> as the radio device.\n");
 	printf("                     If <dev> starts with a digit, then /dev/radio<dev> is used.\n");
+	printf("                     See the -d description of how <dev> is used in combination with -z.\n");
 	printf("  -S, --sdr-device <dev>\n");
 	printf("                     Use device <dev> as the SDR device.\n");
 	printf("                     If <dev> starts with a digit, then /dev/swradio<dev> is used.\n");
+	printf("                     See the -d description of how <dev> is used in combination with -z.\n");
 	printf("  -t, --touch-device <dev>\n");
 	printf("                     Use device <dev> as the touch device.\n");
 	printf("                     If <dev> starts with a digit, then /dev/v4l-touch<dev> is used.\n");
+	printf("                     See the -d description of how <dev> is used in combination with -z.\n");
 	printf("  -u, --subdev-device <dev>\n");
 	printf("                     Use device <dev> as the v4l-subdev device.\n");
 	printf("                     If <dev> starts with a digit, then /dev/v4l-subdev<dev> is used.\n");
+	printf("                     See the -d description of how <dev> is used in combination with -z.\n");
+	printf("  -e, --expbuf-device <dev>\n");
+	printf("                     Use video device <dev> to obtain DMABUF handles.\n");
+	printf("                     If <dev> starts with a digit, then /dev/video<dev> is used.\n");
+	printf("                     See the -d description of how <dev> is used in combination with -z.\n");
+	printf("  -z, --media-bus-info <bus-info>\n");
+	printf("                     Find the media device with the given bus info string. If set, then\n");
+	printf("                     the options above can use the entity name or interface ID to refer\n");
+	printf("                     to the device nodes.\n");
 	printf("  -m, --media-device <dev>\n");
 	printf("                     Use device <dev> as the media controller device. Besides this\n");
 	printf("                     device it also tests all interfaces it finds.\n");
 	printf("                     If <dev> starts with a digit, then /dev/media<dev> is used.\n");
+	printf("                     If <dev> doesn't exist, then attempt to find a media device with a\n");
+	printf("                     bus info string equal to <dev>.\n");
 	printf("  -M, --media-device-only <dev>\n");
 	printf("                     Use device <dev> as the media controller device. Only test this\n");
 	printf("                     device, don't walk over all the interfaces.\n");
 	printf("                     If <dev> starts with a digit, then /dev/media<dev> is used.\n");
-	printf("  -e, --expbuf-device <dev>\n");
-	printf("                     Use device <dev> to obtain DMABUF handles.\n");
-	printf("                     If <dev> starts with a digit, then /dev/video<dev> is used.\n");
-	printf("                     only /dev/videoX devices are supported.\n");
+	printf("                     If <dev> doesn't exist, then attempt to find a media device with a\n");
+	printf("                     bus info string equal to <dev>.\n");
 	printf("  -s, --streaming <count>\n");
 	printf("                     Enable the streaming tests. Set <count> to the number of\n");
 	printf("                     frames to stream (default 60). Requires a valid input/output\n");
 	printf("                     and frequency (when dealing with a tuner). For DMABUF testing\n");
 	printf("                     --expbuf-device needs to be set as well.\n");
-	printf("  -f, --stream-all-formats\n");
+	printf("  --stream-from [<pixelformat>=]<file>\n");
+	printf("  --stream-from-hdr [<pixelformat>=]<file>\n");
+	printf("                     Use the contents of the file to fill in output buffers.\n");
+	printf("                     If the fourcc of the pixelformat is given, then use the file\n");
+	printf("                     for output buffers using that pixelformat only.\n");
+	printf("                     The --stream-from-hdr variant uses the format written by\n");
+	printf("                     v4l2-ctl --stream-to-hdr where the payload sizes for each\n");
+	printf("                     buffer are stored in a header. Useful for compressed formats.\n");
+	printf("  -f, --stream-all-formats [<count>]\n");
 	printf("                     Test streaming all available formats.\n");
 	printf("                     This attempts to stream using MMAP mode or read/write\n");
 	printf("                     for one second for all formats, at all sizes, at all intervals\n");
-	printf("                     and with all field values.\n");
+	printf("                     and with all field values. If <count> is given, then stream\n");
+	printf("                     for that many frames instead of one second.\n");
 	printf("  -a, --stream-all-io\n");
 	printf("                     Do streaming tests for all inputs or outputs instead of just\n");
 	printf("                     the current input or output. This requires that a valid video\n");
@@ -198,7 +237,10 @@ static void usage(void)
 	printf("                     then this defaults to 90%%.\n");
 	printf("  -E, --exit-on-fail Exit on the first fail.\n");
 	printf("  -h, --help         Display this help message.\n");
+	printf("  -C, --color <when> Highlight OK/warn/fail/FAIL strings with colors\n");
+	printf("                     <when> can be set to always, never, or auto (the default)\n");
 	printf("  -n, --no-warnings  Turn off warning messages.\n");
+	printf("  -P, --no-progress  Turn off progress messages.\n");
 	printf("  -T, --trace        Trace all called ioctls.\n");
 	printf("  -v, --verbose      Turn on verbose reporting.\n");
 #ifndef NO_LIBV4L2
@@ -213,15 +255,17 @@ const char *ok(int res)
 	static char buf[100];
 
 	if (res == ENOTTY) {
-		strcpy(buf, "OK (Not Supported)");
+		strcpy(buf, show_colors ?
+		       COLOR_GREEN("OK") " (Not Supported)" :
+		       "OK (Not Supported)");
 		res = 0;
 	} else {
-		strcpy(buf, "OK");
+		strcpy(buf, show_colors ? COLOR_GREEN("OK") : "OK");
 	}
 	tests_total++;
 	if (res) {
 		app_result = res;
-		sprintf(buf, "FAIL");
+		sprintf(buf, show_colors ? COLOR_RED("FAIL") : "FAIL");
 	} else {
 		tests_ok++;
 	}
@@ -254,6 +298,20 @@ int check_0(const void *p, int len)
 	return 0;
 }
 
+static std::map<std::string, std::string> stream_from_map;
+static std::map<std::string, bool> stream_hdr_map;
+
+std::string stream_from(std::string pixelformat, bool &use_hdr)
+{
+	if (stream_from_map.find(pixelformat) == stream_from_map.end()) {
+		if (pixelformat.empty())
+			return "";
+		return stream_from("", use_hdr);
+	}
+	use_hdr = stream_hdr_map[pixelformat];
+	return stream_from_map[pixelformat];
+}
+
 static void storeStateTimings(struct node *node, __u32 caps)
 {
 	if (caps & V4L2_IN_CAP_STD)
@@ -430,6 +488,101 @@ static void signal_handler_interrupt(int signum)
 	exit(-1);
 }
 
+static void determine_codec_mask(struct node &node)
+{
+	struct v4l2_fmtdesc fmt_desc;
+	int num_cap_fmts = 0;
+	int num_compressed_cap_fmts = 0;
+	int num_out_fmts = 0;
+	int num_compressed_out_fmts = 0;
+	unsigned mask = 0;
+
+	node.codec_mask = 0;
+	if (!node.has_vid_m2m())
+		return;
+
+	if (node.enum_fmt(fmt_desc, true, 0, node.g_type()))
+		return;
+
+	do {
+		if (fmt_desc.flags & V4L2_FMT_FLAG_COMPRESSED) {
+			num_compressed_cap_fmts++;
+			switch (fmt_desc.pixelformat) {
+			case V4L2_PIX_FMT_JPEG:
+			case V4L2_PIX_FMT_MJPEG:
+				mask |= JPEG_ENCODER;
+				break;
+			case V4L2_PIX_FMT_H263:
+			case V4L2_PIX_FMT_H264:
+			case V4L2_PIX_FMT_H264_NO_SC:
+			case V4L2_PIX_FMT_H264_MVC:
+			case V4L2_PIX_FMT_MPEG1:
+			case V4L2_PIX_FMT_MPEG2:
+			case V4L2_PIX_FMT_MPEG4:
+			case V4L2_PIX_FMT_XVID:
+			case V4L2_PIX_FMT_VC1_ANNEX_G:
+			case V4L2_PIX_FMT_VC1_ANNEX_L:
+			case V4L2_PIX_FMT_VP8:
+			case V4L2_PIX_FMT_VP9:
+			case V4L2_PIX_FMT_HEVC:
+			case V4L2_PIX_FMT_FWHT:
+				mask |= STATEFUL_ENCODER;
+				break;
+			case V4L2_PIX_FMT_MPEG2_SLICE:
+				mask |= STATELESS_ENCODER;
+				break;
+			default:
+				return;
+			}
+		}
+		num_cap_fmts++;
+	} while (!node.enum_fmt(fmt_desc));
+
+
+	if (node.enum_fmt(fmt_desc, true, 0, v4l_type_invert(node.g_type())))
+		return;
+
+	do {
+		if (fmt_desc.flags & V4L2_FMT_FLAG_COMPRESSED) {
+			num_compressed_out_fmts++;
+			switch (fmt_desc.pixelformat) {
+			case V4L2_PIX_FMT_JPEG:
+			case V4L2_PIX_FMT_MJPEG:
+				mask |= JPEG_DECODER;
+				break;
+			case V4L2_PIX_FMT_H263:
+			case V4L2_PIX_FMT_H264:
+			case V4L2_PIX_FMT_H264_NO_SC:
+			case V4L2_PIX_FMT_H264_MVC:
+			case V4L2_PIX_FMT_MPEG1:
+			case V4L2_PIX_FMT_MPEG2:
+			case V4L2_PIX_FMT_MPEG4:
+			case V4L2_PIX_FMT_XVID:
+			case V4L2_PIX_FMT_VC1_ANNEX_G:
+			case V4L2_PIX_FMT_VC1_ANNEX_L:
+			case V4L2_PIX_FMT_VP8:
+			case V4L2_PIX_FMT_VP9:
+			case V4L2_PIX_FMT_HEVC:
+			case V4L2_PIX_FMT_FWHT:
+				mask |= STATEFUL_DECODER;
+				break;
+			case V4L2_PIX_FMT_MPEG2_SLICE:
+			case V4L2_PIX_FMT_FWHT_STATELESS:
+				mask |= STATELESS_DECODER;
+				break;
+			default:
+				return;
+			}
+		}
+		num_out_fmts++;
+	} while (!node.enum_fmt(fmt_desc));
+
+	if (num_compressed_out_fmts == 0 && num_compressed_cap_fmts == num_cap_fmts)
+		node.codec_mask = mask;
+	else if (num_compressed_cap_fmts == 0 && num_compressed_out_fmts == num_out_fmts)
+		node.codec_mask = mask;
+}
+
 static int testCap(struct node *node)
 {
 	struct v4l2_capability vcap;
@@ -449,7 +602,7 @@ static int testCap(struct node *node)
 	const __u32 output_caps = V4L2_CAP_VIDEO_OUTPUT | V4L2_CAP_VIDEO_OUTPUT_MPLANE |
 			V4L2_CAP_VIDEO_OUTPUT_OVERLAY | V4L2_CAP_VBI_OUTPUT |
 			V4L2_CAP_SDR_OUTPUT | V4L2_CAP_SLICED_VBI_OUTPUT |
-			V4L2_CAP_MODULATOR;
+			V4L2_CAP_MODULATOR | V4L2_CAP_META_OUTPUT;
 	const __u32 overlay_caps = V4L2_CAP_VIDEO_OVERLAY | V4L2_CAP_VIDEO_OUTPUT_OVERLAY;
 	const __u32 m2m_caps = V4L2_CAP_VIDEO_M2M | V4L2_CAP_VIDEO_M2M_MPLANE;
 	const __u32 io_caps = V4L2_CAP_STREAMING | V4L2_CAP_READWRITE;
@@ -474,6 +627,10 @@ static int testCap(struct node *node)
 	    memcmp(vcap.bus_info, "platform:", 9) &&
 	    memcmp(vcap.bus_info, "rmi4:", 5))
 		return fail("missing bus_info prefix ('%s')\n", vcap.bus_info);
+	if (!node->media_bus_info.empty() &&
+	    node->media_bus_info != std::string((const char *)vcap.bus_info))
+		warn("media bus_info '%s' differs from V4L2 bus_info '%s'\n",
+		     node->media_bus_info.c_str(), vcap.bus_info);
 	fail_on_test((vcap.version >> 16) < 3);
 	fail_on_test(check_0(vcap.reserved, sizeof(vcap.reserved)));
 	caps = vcap.capabilities;
@@ -524,6 +681,17 @@ static int testCap(struct node *node)
 	// having both mplane and splane caps is not allowed (at least for now)
 	fail_on_test((dcaps & mplane_caps) && (dcaps & splane_caps));
 
+	if (node->codec_mask) {
+		bool found_bit = false;
+
+		// Only one bit may be set in the mask
+		for (unsigned i = 0; i < 32; i++)
+			if (node->codec_mask & (1U << i)) {
+				fail_on_test(found_bit);
+				found_bit = true;
+			}
+	}
+
 	return 0;
 }
 
@@ -598,12 +766,14 @@ static int testPrio(struct node *node, struct node *node2)
 static void streamingSetup(struct node *node)
 {
 	if (node->can_capture) {
+		v4l2_fract min_period = { 1, 1000 };
 		struct v4l2_input input;
 
 		memset(&input, 0, sizeof(input));
 		doioctl(node, VIDIOC_G_INPUT, &input.index);
 		doioctl(node, VIDIOC_ENUMINPUT, &input);
 		node->cur_io_caps = input.capabilities;
+		node->set_interval(min_period);
 	} else if (node->can_output) {
 		struct v4l2_output output;
 
@@ -630,25 +800,122 @@ static int parse_subopt(char **subs, const char * const *subopts, char **value)
 	return opt;
 }
 
-static const char *make_devname(const char *device, const char *devname)
+static int open_media_bus_info(const std::string &bus_info, std::string &media_devname)
+{
+	DIR *dp;
+	struct dirent *ep;
+
+	dp = opendir("/dev");
+	if (dp == NULL)
+		return -1;
+
+	while ((ep = readdir(dp))) {
+		const char *name = ep->d_name;
+
+		if (!memcmp(name, "media", 5) && isdigit(name[5])) {
+			struct media_device_info mdi;
+			media_devname = std::string("/dev/") + name;
+
+			int fd = open(media_devname.c_str(), O_RDWR);
+			if (fd < 0)
+				continue;
+			if (!ioctl(fd, MEDIA_IOC_DEVICE_INFO, &mdi) &&
+			    bus_info == mdi.bus_info) {
+				closedir(dp);
+				return fd;
+			}
+			close(fd);
+		}
+	}
+	closedir(dp);
+	return -1;
+}
+
+static std::string make_devname(const char *device, const char *devname,
+				const std::string &media_bus_info, bool is_media = false)
 {
 	if (device[0] >= '0' && device[0] <= '9' && strlen(device) <= 3) {
-		static char newdev[32];
+		char newdev[32];
 
 		sprintf(newdev, "/dev/%s%s", devname, device);
 		return newdev;
 	}
-	return device;
+	if (media_bus_info.empty())
+		return device;
+
+	std::string media_devname;
+	int media_fd = open_media_bus_info(media_bus_info, media_devname);
+	if (media_fd < 0)
+		return device;
+	if (is_media) {
+		close(media_fd);
+		return media_devname.c_str();
+	}
+
+	media_v2_topology topology;
+	memset(&topology, 0, sizeof(topology));
+	if (ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology)) {
+		close(media_fd);
+		return device;
+	}
+
+	media_v2_entity *ents = new media_v2_entity[topology.num_entities];
+	topology.ptr_entities = (__u64)ents;
+	media_v2_link *links = new media_v2_link[topology.num_links];
+	topology.ptr_links = (__u64)links;
+	media_v2_interface *ifaces = new media_v2_interface[topology.num_interfaces];
+	topology.ptr_interfaces = (__u64)ifaces;
+
+	unsigned i, ent_id, iface_id = 0;
+
+	std::string result(device);
+
+	if (ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology))
+		goto err;
+
+	if (device[0] == '0' && device[1] == 'x')
+		iface_id = strtoul(device, NULL, 16);
+
+	if (!iface_id) {
+		for (i = 0; i < topology.num_entities; i++)
+			if (!strcmp(ents[i].name, device))
+				break;
+		if (i >= topology.num_entities)
+			goto err;
+		ent_id = ents[i].id;
+		for (i = 0; i < topology.num_links; i++)
+			if (links[i].sink_id == ent_id &&
+			    (links[i].flags & MEDIA_LNK_FL_LINK_TYPE) ==
+			    MEDIA_LNK_FL_INTERFACE_LINK)
+				break;
+		if (i >= topology.num_links)
+			goto err;
+		iface_id = links[i].source_id;
+	}
+	for (i = 0; i < topology.num_interfaces; i++)
+		if (ifaces[i].id == iface_id)
+			break;
+	if (i >= topology.num_interfaces)
+		goto err;
+
+	result = mi_media_get_device(ifaces[i].devnode.major, ifaces[i].devnode.minor);
+	
+err:
+	delete [] ents;
+	delete [] links;
+	delete [] ifaces;
+	close(media_fd);
+	return result;
 }
 
-void testNode(struct node &node, struct node &expbuf_node, media_type type,
-	      unsigned frame_count)
+void testNode(struct node &node, struct node &node_m2m_cap, struct node &expbuf_node, media_type type,
+	      unsigned frame_count, unsigned all_fmt_frame_count)
 {
 	struct node node2;
 	struct v4l2_capability vcap;		/* list_cap */
+	std::string driver;
 
-	printf("Compliance test for device %s%s:\n\n",
-			node.device, node.g_direct() ? "" : " (using libv4l2)");
+	tests_total = tests_ok = warnings = 0;
 
 	node.is_video = type == MEDIA_TYPE_VIDEO;
 	node.is_vbi = type == MEDIA_TYPE_VBI;
@@ -656,10 +923,38 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 	node.is_sdr = type == MEDIA_TYPE_SDR;
 	node.is_touch = type == MEDIA_TYPE_TOUCH;
 
-	if (node.is_v4l2())
+	if (node.is_v4l2()) {
 		doioctl(&node, VIDIOC_QUERYCAP, &vcap);
-	else
+		driver = (const char *)vcap.driver;
+		is_vivid = driver == "vivid";
+		if (is_vivid)
+			node.bus_info = (const char *)vcap.bus_info;
+		determine_codec_mask(node);
+	} else {
 		memset(&vcap, 0, sizeof(vcap));
+	}
+
+	if (!node.is_media())
+		media_fd = mi_get_media_fd(node.g_fd(), node.bus_info);
+
+	int fd = node.is_media() ? node.g_fd() : media_fd;
+	if (fd >= 0) {
+		struct media_device_info mdinfo;
+
+		if (!ioctl(fd, MEDIA_IOC_DEVICE_INFO, &mdinfo)) {
+			if (!node.is_v4l2())
+				driver = mdinfo.driver;
+			node.media_bus_info = mdinfo.bus_info;
+		}
+	}
+
+	if (driver.empty())
+		printf("Compliance test for device %s%s:\n\n",
+		       node.device, node.g_direct() ? "" : " (using libv4l2)");
+	else
+		printf("Compliance test for %s device %s%s:\n\n",
+		       driver.c_str(), node.device, node.g_direct() ? "" : " (using libv4l2)");
+
 	if (node.g_caps() & (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VBI_CAPTURE |
 			 V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_SLICED_VBI_CAPTURE))
 		node.has_inputs = true;
@@ -675,40 +970,41 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 	if (node.g_caps() & (V4L2_CAP_VIDEO_OUTPUT | V4L2_CAP_VBI_OUTPUT |
 			 V4L2_CAP_VIDEO_OUTPUT_MPLANE | V4L2_CAP_VIDEO_M2M_MPLANE |
 			 V4L2_CAP_VIDEO_M2M | V4L2_CAP_SLICED_VBI_OUTPUT |
-			 V4L2_CAP_RDS_OUTPUT | V4L2_CAP_SDR_OUTPUT))
+			 V4L2_CAP_RDS_OUTPUT | V4L2_CAP_SDR_OUTPUT |
+			 V4L2_CAP_META_OUTPUT))
 		node.can_output = true;
 	if (node.g_caps() & (V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_VIDEO_OUTPUT_MPLANE |
 			 V4L2_CAP_VIDEO_M2M_MPLANE))
 		node.is_planar = true;
-	if (node.g_caps() & V4L2_CAP_META_CAPTURE) {
+	if (node.g_caps() & (V4L2_CAP_META_CAPTURE | V4L2_CAP_META_OUTPUT)) {
 		node.is_video = false;
 		node.is_meta = true;
 	}
 
 	/* Information Opts */
 
-	if (!node.is_media())
-		media_fd = mi_get_media_fd(node.g_fd());
-
 	if (node.is_v4l2()) {
 		printf("Driver Info:\n");
 		v4l2_info_capability(vcap);
-
-		if (!strcmp((const char *)vcap.driver, "vivid")) {
-#define VIVID_CID_VIVID_BASE		(0x00f00000 | 0xf000)
-#define VIVID_CID_DISCONNECT		(VIVID_CID_VIVID_BASE + 65)
-
-			struct v4l2_queryctrl qc;
-
-			// This control is present for all devices if error
-			// injection is enabled in the vivid driver.
-			qc.id = VIVID_CID_DISCONNECT;
-			if (!doioctl(&node, VIDIOC_QUERYCTRL, &qc)) {
-				printf("\nThe vivid driver has error injection enabled which will cause\n");
-				printf("the compliance test to fail. Load the vivid module with the\n");
-				printf("no_error_inj=1 module option to disable error injection.\n");
-				exit(1);
-			}
+		switch (node.codec_mask) {
+		case JPEG_ENCODER:
+			printf("\tDetected JPEG Encoder\n");
+			break;
+		case JPEG_DECODER:
+			printf("\tDetected JPEG Decoder\n");
+			break;
+		case STATEFUL_ENCODER:
+			printf("\tDetected Stateful Encoder\n");
+			break;
+		case STATEFUL_DECODER:
+			printf("\tDetected Stateful Decoder\n");
+			break;
+		case STATELESS_ENCODER:
+			printf("\tDetected Stateless Encoder\n");
+			break;
+		case STATELESS_DECODER:
+			printf("\tDetected Stateless Decoder\n");
+			break;
 		}
 	}
 
@@ -810,9 +1106,7 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 		printf("\ttest MEDIA_IOC_ENUM_ENTITIES/LINKS: %s\n", ok(testMediaEnum(&node)));
 		printf("\ttest MEDIA_IOC_SETUP_LINK: %s\n", ok(testMediaSetupLink(&node)));
 		printf("\n");
-		if (options[OptSetMediaDevice])
-			walkTopology(node, expbuf_node, frame_count);
-		goto done;
+		goto show_total;
 	}
 
 	/* Debug ioctls */
@@ -915,6 +1209,7 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 	max_io = node.inputs > node.outputs ? node.inputs : node.outputs;
 
 	for (unsigned io = 0; io < (max_io ? max_io : 1); io++) {
+		v4l2_fract min_period = { 1, 1000 };
 		char suffix[100] = "";
 
 		node.std_controls = node.priv_controls = 0;
@@ -923,6 +1218,9 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 		node.frmsizes.clear();
 		node.frmsizes_count.clear();
 		node.has_frmintervals = false;
+		node.valid_buftypes = 0;
+		node.valid_memorytype = 0;
+		node.buf_caps = 0;
 		for (unsigned idx = 0; idx < V4L2_BUF_TYPE_LAST + 1; idx++)
 			node.buftype_pixfmts[idx].clear();
 
@@ -986,7 +1284,16 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 
 		printf("Buffer ioctls%s:\n", suffix);
 		printf("\ttest VIDIOC_REQBUFS/CREATE_BUFS/QUERYBUF: %s\n", ok(testReqBufs(&node)));
+		// Reopen after each streaming test to reset the streaming state
+		// in case of any errors in the preceeding test.
+		node.reopen();
 		printf("\ttest VIDIOC_EXPBUF: %s\n", ok(testExpBuf(&node)));
+		if (node.can_capture)
+			node.set_interval(min_period);
+		printf("\ttest Requests: %s\n", ok(testRequests(&node, options[OptStreaming])));
+		// Reopen after each streaming test to reset the streaming state
+		// in case of any errors in the preceeding test.
+		node.reopen();
 		printf("\n");
 	}
 
@@ -1024,16 +1331,39 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 			node.reopen();
 			printf("\ttest blocking wait: %s\n", ok(testBlockingWait(&node)));
 			node.reopen();
-			printf("\ttest MMAP: %s\n", ok(testMmap(&node, frame_count)));
+			if (!(node.codec_mask & (STATEFUL_ENCODER | STATEFUL_DECODER))) {
+				printf("\ttest MMAP (no poll): %s\n",
+				       ok(testMmap(&node, &node_m2m_cap, frame_count, POLL_MODE_NONE)));
+				node.reopen();
+			}
+			printf("\ttest MMAP (select): %s\n",
+			       ok(testMmap(&node, &node_m2m_cap, frame_count, POLL_MODE_SELECT)));
 			node.reopen();
-			printf("\ttest USERPTR: %s\n", ok(testUserPtr(&node, frame_count)));
+			printf("\ttest MMAP (epoll): %s\n",
+			       ok(testMmap(&node, &node_m2m_cap, frame_count, POLL_MODE_EPOLL)));
+			node.reopen();
+			if (!(node.codec_mask & (STATEFUL_ENCODER | STATEFUL_DECODER))) {
+				printf("\ttest USERPTR (no poll): %s\n",
+				       ok(testUserPtr(&node, &node_m2m_cap, frame_count, POLL_MODE_NONE)));
+				node.reopen();
+			}
+			printf("\ttest USERPTR (select): %s\n",
+			       ok(testUserPtr(&node, &node_m2m_cap, frame_count, POLL_MODE_SELECT)));
 			node.reopen();
 			if (options[OptSetExpBufDevice] ||
-					!(node.valid_memorytype & (1 << V4L2_MEMORY_DMABUF)))
-				printf("\ttest DMABUF: %s\n", ok(testDmaBuf(&expbuf_node, &node, frame_count)));
-			else if (!options[OptSetExpBufDevice])
+			    !(node.valid_memorytype & (1 << V4L2_MEMORY_DMABUF))) {
+				if (!(node.codec_mask & (STATEFUL_ENCODER | STATEFUL_DECODER))) {
+					printf("\ttest DMABUF (no poll): %s\n",
+					       ok(testDmaBuf(&expbuf_node, &node, &node_m2m_cap,
+							     frame_count, POLL_MODE_NONE)));
+					node.reopen();
+				}
+				printf("\ttest DMABUF (select): %s\n",
+				       ok(testDmaBuf(&expbuf_node, &node, &node_m2m_cap, frame_count, POLL_MODE_SELECT)));
+				node.reopen();
+			} else if (!options[OptSetExpBufDevice]) {
 				printf("\ttest DMABUF: Cannot test, specify --expbuf-device\n");
-			node.reopen();
+			}
 
 			printf("\n");
 		}
@@ -1042,10 +1372,15 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 			printf("Stream using all formats:\n");
 
 			if (node.is_m2m) {
-				printf("\tNot supported for M2M devices\n");
+				if (node.codec_mask &
+				    (JPEG_DECODER | STATEFUL_DECODER | STATELESS_DECODER)) {
+					printf("\tNot supported for decoder devices\n");
+				} else {
+					streamM2MAllFormats(&node, all_fmt_frame_count);
+				}
 			} else {
 				streamingSetup(&node);
-				streamAllFormats(&node);
+				streamAllFormats(&node, all_fmt_frame_count);
 			}
 		}
 
@@ -1069,7 +1404,29 @@ void testNode(struct node &node, struct node &expbuf_node, media_type type,
 
 	restoreState();
 
-done:
+show_total:
+	/* Final test report */
+	if (driver.empty())
+		printf("Total for device %s%s: %d, Succeeded: %d, Failed: %d, Warnings: %d\n",
+		       node.device, node.g_direct() ? "" : " (using libv4l2)",
+		       tests_total, tests_ok, tests_total - tests_ok, warnings);
+	else
+		printf("Total for %s device %s%s: %d, Succeeded: %d, Failed: %d, Warnings: %d\n",
+		       driver.c_str(), node.device, node.g_direct() ? "" : " (using libv4l2)",
+		       tests_total, tests_ok, tests_total - tests_ok, warnings);
+	grand_total += tests_total;
+	grand_ok += tests_ok;
+	grand_warnings += warnings;
+
+	if (node.is_media() && options[OptSetMediaDevice]) {
+		walkTopology(node, expbuf_node,
+			     frame_count, all_fmt_frame_count);
+		/* Final test report */
+		printf("\nGrand Total for %s device %s: %d, Succeeded: %d, Failed: %d, Warnings: %d\n",
+		       driver.c_str(), node.device,
+		       grand_total, grand_ok, grand_total - grand_ok, grand_warnings);
+	}
+
 	node.close();
 	if (node.node2)
 		node.node2->close();
@@ -1081,14 +1438,17 @@ int main(int argc, char **argv)
 	struct node node;
 	media_type type = MEDIA_TYPE_UNKNOWN;
 	struct node expbuf_node;
+	std::string media_bus_info;
+	const char *env_media_apps_color = getenv("MEDIA_APPS_COLOR");
 
 	/* command args */
 	int ch;
-	const char *device = "/dev/video0";
-	const char *expbuf_device = NULL;	/* --expbuf-device device */
+	std::string device("/dev/video0");
+	std::string expbuf_device;	/* --expbuf-device device */
 	struct utsname uts;
 	int v1, v2, v3;
 	unsigned frame_count = 60;
+	unsigned all_fmt_frame_count = 0;
 	char short_options[26 * 2 * 3 + 1];
 	char *value, *subs;
 	int idx = 0;
@@ -1109,6 +1469,17 @@ int main(int argc, char **argv)
 		printf("Running on 2.6.%d\n", kernel_version);
 	printf("\n");
 
+	if (!env_media_apps_color || !strcmp(env_media_apps_color, "auto"))
+		show_colors = isatty(STDOUT_FILENO);
+	else if (!strcmp(env_media_apps_color, "always"))
+		show_colors = true;
+	else if (!strcmp(env_media_apps_color, "never"))
+		show_colors = false;
+	else {
+		fprintf(stderr,
+			"v4l2-compliance: invalid value for MEDIA_APPS_COLOR environment variable\n");
+	}
+
 	for (i = 0; long_options[i].name; i++) {
 		if (!isalpha(long_options[i].val))
 			continue;
@@ -1147,41 +1518,59 @@ int main(int argc, char **argv)
 			usage();
 			return 0;
 		case OptSetDevice:
-			device = make_devname(optarg, "video");
+			device = make_devname(optarg, "video", media_bus_info);
 			break;
 		case OptSetVbiDevice:
-			device = make_devname(optarg, "vbi");
+			device = make_devname(optarg, "vbi", media_bus_info);
 			break;
 		case OptSetRadioDevice:
-			device = make_devname(optarg, "radio");
+			device = make_devname(optarg, "radio", media_bus_info);
 			break;
 		case OptSetSWRadioDevice:
-			device = make_devname(optarg, "swradio");
+			device = make_devname(optarg, "swradio", media_bus_info);
 			break;
 		case OptSetTouchDevice:
-			device = make_devname(optarg, "v4l-touch");
+			device = make_devname(optarg, "v4l-touch", media_bus_info);
 			break;
 		case OptSetSubDevDevice:
-			device = make_devname(optarg, "v4l-subdev");
+			device = make_devname(optarg, "v4l-subdev", media_bus_info);
 			break;
 		case OptSetMediaDevice:
 		case OptSetMediaDeviceOnly:
-			device = make_devname(optarg, "media");
+			device = make_devname(optarg, "media", optarg, true);
 			type = MEDIA_TYPE_MEDIA;
 			break;
+		case OptMediaBusInfo:
+			media_bus_info = optarg;
+			break;
 		case OptSetExpBufDevice:
-			expbuf_device = optarg;
-			if (expbuf_device[0] >= '0' && expbuf_device[0] <= '9' && strlen(expbuf_device) <= 3) {
-				static char newdev[20];
-
-				sprintf(newdev, "/dev/video%s", expbuf_device);
-				expbuf_device = newdev;
-			}
+			expbuf_device = make_devname(optarg, "video", media_bus_info);
 			break;
 		case OptStreaming:
 			if (optarg)
 				frame_count = strtoul(optarg, NULL, 0);
 			break;
+		case OptStreamFrom:
+		case OptStreamFromHdr: {
+			char *equal = strchr(optarg, '=');
+			bool has_hdr = ch == OptStreamFromHdr;
+
+			if (equal == optarg)
+				equal = NULL;
+			if (equal) {
+				*equal = '\0';
+				stream_from_map[optarg] = equal + 1;
+				stream_hdr_map[optarg] = has_hdr;
+			} else {
+				stream_from_map[""] = optarg;
+				stream_hdr_map[""] = has_hdr;
+			}
+			break;
+		}
+		case OptStreamAllFormats:
+			if (optarg)
+				all_fmt_frame_count = strtoul(optarg, NULL, 0);
+			break;
 		case OptStreamAllColorTest:
 			subs = optarg;
 			while (*subs != '\0') {
@@ -1221,6 +1610,18 @@ int main(int argc, char **argv)
 				}
 			}
 			break;
+		case OptColor:
+			if (!strcmp(optarg, "always"))
+				show_colors = true;
+			else if (!strcmp(optarg, "never"))
+				show_colors = false;
+			else if (!strcmp(optarg, "auto"))
+				show_colors = isatty(STDOUT_FILENO);
+			else {
+				usage();
+				exit(1);
+			}
+			break;
 		case OptNoWarnings:
 			show_warnings = false;
 			break;
@@ -1233,6 +1634,9 @@ int main(int argc, char **argv)
 		case OptVerbose:
 			show_info = true;
 			break;
+		case OptNoProgress:
+			no_progress = true;
+			break;
 		case ':':
 			fprintf(stderr, "Option `%s' requires a value\n",
 				argv[optind]);
@@ -1258,60 +1662,57 @@ int main(int argc, char **argv)
 	int fd;
 
 	if (type == MEDIA_TYPE_UNKNOWN)
-		type = mi_media_detect_type(device);
+		type = mi_media_detect_type(device.c_str());
 	if (type == MEDIA_TYPE_CANT_STAT) {
 		fprintf(stderr, "Cannot open device %s, exiting.\n",
-			device);
+			device.c_str());
 		exit(1);
 	}
 	if (type == MEDIA_TYPE_UNKNOWN) {
 		fprintf(stderr, "Unable to detect what device %s is, exiting.\n",
-			device);
+			device.c_str());
 		exit(1);
 	}
 
-	node.device = device;
+	node.device = device.c_str();
 	node.s_trace(options[OptTrace]);
 	switch (type) {
 	case MEDIA_TYPE_MEDIA:
 		node.s_direct(true);
-		fd = node.media_open(device, false);
+		fd = node.media_open(device.c_str(), false);
 		break;
 	case MEDIA_TYPE_SUBDEV:
 		node.s_direct(true);
-		fd = node.subdev_open(device, false);
+		fd = node.subdev_open(device.c_str(), false);
 		break;
 	default:
 		node.s_direct(direct);
-		fd = node.open(device, false);
+		fd = node.open(device.c_str(), false);
 		break;
 	}
 	if (fd < 0) {
-		fprintf(stderr, "Failed to open %s: %s\n", device,
+		fprintf(stderr, "Failed to open %s: %s\n", device.c_str(),
 			strerror(errno));
 		exit(1);
 	}
 
-	if (expbuf_device) {
+	if (!expbuf_device.empty()) {
 		expbuf_node.s_trace(options[OptTrace]);
 		expbuf_node.s_direct(true);
-		fd = expbuf_node.open(expbuf_device, false);
+		fd = expbuf_node.open(expbuf_device.c_str(), false);
 		if (fd < 0) {
-			fprintf(stderr, "Failed to open %s: %s\n", expbuf_device,
+			fprintf(stderr, "Failed to open %s: %s\n", expbuf_device.c_str(),
 				strerror(errno));
 			exit(1);
 		}
 	}
 
-	testNode(node, expbuf_node, type, frame_count);
+	testNode(node, node, expbuf_node, type, frame_count, all_fmt_frame_count);
 
-	if (expbuf_device)
+	if (!expbuf_device.empty())
 		expbuf_node.close();
 	if (media_fd >= 0)
 		close(media_fd);
 
-	/* Final test report */
-	printf("Total: %d, Succeeded: %d, Failed: %d, Warnings: %d\n",
-			tests_total, tests_ok, tests_total - tests_ok, warnings);
 	exit(app_result);
 }
