@@ -32,6 +32,8 @@
 #include <vector>
 #include "v4l2-compliance.h"
 
+#define V4L2_CTRL_CLASS_VIVID 0x00f00000
+
 static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 {
 	struct v4l2_querymenu qmenu;
@@ -390,6 +392,9 @@ int testSimpleControls(struct node *node)
 
 		if (qctrl.type >= V4L2_CTRL_COMPOUND_TYPES)
 			continue;
+		if (is_vivid && V4L2_CTRL_ID2WHICH(qctrl.id) == V4L2_CTRL_CLASS_VIVID)
+			continue;
+
 		info("checking control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
 		ctrl.id = qctrl.id;
 		if (qctrl.type == V4L2_CTRL_TYPE_INTEGER64 ||
@@ -611,6 +616,12 @@ int testExtendedControls(struct node *node)
 	for (iter = node->controls.begin(); iter != node->controls.end(); ++iter) {
 		test_query_ext_ctrl &qctrl = iter->second;
 
+		if (is_vivid && V4L2_CTRL_ID2WHICH(qctrl.id) == V4L2_CTRL_CLASS_VIVID)
+			continue;
+		// Skip V4L2_CID_MPEG_VIDEO_FWHT_PARAMS for now
+		if (qctrl.id == (V4L2_CID_MPEG_BASE + 292))
+			continue;
+
 		info("checking extended control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
 		ctrl.id = qctrl.id;
 		ctrl.size = 0;
@@ -722,6 +733,9 @@ int testExtendedControls(struct node *node)
 
 		if (qctrl.flags & (V4L2_CTRL_FLAG_READ_ONLY | V4L2_CTRL_FLAG_WRITE_ONLY))
 			continue;
+		// Skip V4L2_CID_MPEG_VIDEO_FWHT_PARAMS for now
+		if (qctrl.id == (V4L2_CID_MPEG_BASE + 292))
+			continue;
 		ctrl.id = qctrl.id;
 		ctrl.size = 0;
 		if (qctrl.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD) {
@@ -841,13 +855,16 @@ int testEvents(struct node *node)
 
 		if (node->can_capture) {
 			fail_on_test(doioctl(node, VIDIOC_G_INPUT, &id));
-			if (node->controls.find(V4L2_CID_DV_RX_POWER_PRESENT) == node->controls.end())
+			if (node->is_video &&
+			    node->controls.find(V4L2_CID_DV_RX_POWER_PRESENT) == node->controls.end())
 				warn("V4L2_CID_DV_RX_POWER_PRESENT not found for input %d\n", id);
 		} else {
 			fail_on_test(doioctl(node, VIDIOC_G_OUTPUT, &id));
-			if (node->controls.find(V4L2_CID_DV_TX_HOTPLUG) == node->controls.end())
+			if (node->is_video &&
+			    node->controls.find(V4L2_CID_DV_TX_HOTPLUG) == node->controls.end())
 				warn("V4L2_CID_DV_TX_HOTPLUG not found for output %d\n", id);
-			if (node->controls.find(V4L2_CID_DV_TX_EDID_PRESENT) == node->controls.end())
+			if (node->is_video &&
+			    node->controls.find(V4L2_CID_DV_TX_EDID_PRESENT) == node->controls.end())
 				warn("V4L2_CID_DV_TX_EDID_PRESENT not found for output %d\n", id);
 		}
 		if (node->can_capture) {
@@ -858,6 +875,39 @@ int testEvents(struct node *node)
 		}
 	}
 
+	if (node->codec_mask & STATEFUL_ENCODER)
+		fail_on_test(node->controls.find(V4L2_CID_MIN_BUFFERS_FOR_OUTPUT) == node->controls.end());
+
+	struct v4l2_event_subscription sub = { 0 };
+
+	sub.type = V4L2_EVENT_EOS;
+	bool have_eos = !doioctl(node, VIDIOC_SUBSCRIBE_EVENT, &sub);
+	if (have_eos)
+		fail_on_test(doioctl(node, VIDIOC_UNSUBSCRIBE_EVENT, &sub));
+	else
+		fail_on_test(node->codec_mask & STATEFUL_ENCODER);
+	sub.type = V4L2_EVENT_SOURCE_CHANGE;
+	bool have_source_change = !doioctl(node, VIDIOC_SUBSCRIBE_EVENT, &sub);
+	if (have_source_change)
+		fail_on_test(doioctl(node, VIDIOC_UNSUBSCRIBE_EVENT, &sub));
+
+	switch (node->codec_mask) {
+	case STATEFUL_ENCODER:
+		fail_on_test(have_source_change || !have_eos);
+		break;
+	case STATEFUL_DECODER:
+		fail_on_test(!have_source_change || !have_eos);
+		break;
+	case STATELESS_ENCODER:
+	case STATELESS_DECODER:
+		fail_on_test(have_source_change || have_eos);
+		break;
+	default:
+		break;
+	}
+	if (node->can_output && !node->is_m2m)
+		fail_on_test(have_source_change);
+
 	if (node->controls.empty())
 		return ENOTTY;
 	return 0;
