@@ -47,6 +47,7 @@ static const __u32 buftype2cap[] = {
 	V4L2_CAP_SDR_CAPTURE,
 	V4L2_CAP_SDR_OUTPUT,
 	V4L2_CAP_META_CAPTURE,
+	V4L2_CAP_META_OUTPUT,
 };
 
 static int testEnumFrameIntervals(struct node *node, __u32 pixfmt,
@@ -258,9 +259,16 @@ static int testEnumFormatsType(struct node *node, unsigned type)
 			return fail("fmtdesc.pixelformat not set\n");
 		if (node->g_direct() && (fmtdesc.flags & V4L2_FMT_FLAG_EMULATED))
 			return fail("drivers must never set the emulated flag\n");
-		if (fmtdesc.flags & ~(V4L2_FMT_FLAG_COMPRESSED | V4L2_FMT_FLAG_EMULATED))
+		if (fmtdesc.flags & ~(V4L2_FMT_FLAG_COMPRESSED | V4L2_FMT_FLAG_EMULATED |
+				      V4L2_FMT_FLAG_CONTINUOUS_BYTESTREAM |
+				      V4L2_FMT_FLAG_DYN_RESOLUTION))
 			return fail("unknown flag %08x returned\n", fmtdesc.flags);
+		if (!(fmtdesc.flags & V4L2_FMT_FLAG_COMPRESSED))
+			fail_on_test(fmtdesc.flags & (V4L2_FMT_FLAG_CONTINUOUS_BYTESTREAM |
+						      V4L2_FMT_FLAG_DYN_RESOLUTION));
 		ret = testEnumFrameSizes(node, fmtdesc.pixelformat);
+		if (ret)
+			fail_on_test(node->codec_mask & STATEFUL_ENCODER);
 		if (ret && ret != ENOTTY)
 			return ret;
 		f++;
@@ -298,6 +306,7 @@ int testEnumFormats(struct node *node)
 		case V4L2_BUF_TYPE_SDR_CAPTURE:
 		case V4L2_BUF_TYPE_SDR_OUTPUT:
 		case V4L2_BUF_TYPE_META_CAPTURE:
+		case V4L2_BUF_TYPE_META_OUTPUT:
 			if (ret && (node->g_caps() & buftype2cap[type]))
 				return fail("%s cap set, but no %s formats defined\n",
 						buftype2s(type).c_str(), buftype2s(type).c_str());
@@ -325,9 +334,11 @@ int testEnumFormats(struct node *node)
 	return supported ? 0 : ENOTTY;
 }
 
-static int testColorspace(__u32 pixelformat, __u32 colorspace, __u32 ycbcr_enc, __u32 quantization)
+static int testColorspace(bool non_zero_colorspace,
+			  __u32 pixelformat, __u32 colorspace, __u32 ycbcr_enc, __u32 quantization)
 {
-	fail_on_test(!colorspace);
+	if (non_zero_colorspace)
+		fail_on_test(!colorspace);
 	fail_on_test(colorspace == V4L2_COLORSPACE_BT878);
 	fail_on_test(pixelformat != V4L2_PIX_FMT_JPEG &&
 		     pixelformat != V4L2_PIX_FMT_MJPEG &&
@@ -379,7 +390,7 @@ int testFBuf(struct node *node)
 	}*/
 	fail_on_test(fbuf.fmt.bytesperline && fbuf.fmt.bytesperline < fbuf.fmt.width);
 	fail_on_test(fbuf.fmt.sizeimage && fbuf.fmt.sizeimage < fbuf.fmt.bytesperline * fbuf.fmt.height);
-	fail_on_test(testColorspace(fbuf.fmt.pixelformat, fbuf.fmt.colorspace, 0, 0));
+	fail_on_test(testColorspace(true, fbuf.fmt.pixelformat, fbuf.fmt.colorspace, 0, 0));
 	return 0;
 }
 
@@ -434,8 +445,9 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 		fail_on_test(pix.bytesperline && pix.bytesperline < pix.width);
 		fail_on_test(!pix.sizeimage);
 		if (!node->is_m2m)
-			fail_on_test(testColorspace(pix.pixelformat, pix.colorspace,
-					    pix.ycbcr_enc, pix.quantization));
+			fail_on_test(testColorspace(node->has_inputs || node->has_outputs,
+						    pix.pixelformat, pix.colorspace,
+						    pix.ycbcr_enc, pix.quantization));
 		fail_on_test(pix.field == V4L2_FIELD_ANY);
 		if (pix.priv && pix.priv != V4L2_PIX_FMT_PRIV_MAGIC)
 			return fail("priv is non-zero and non-magic!\n");
@@ -449,8 +461,9 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 			return fail("pixelformat %08x (%s) for buftype %d not reported by ENUM_FMT\n",
 					pix_mp.pixelformat, fcc2s(pix_mp.pixelformat).c_str(), type);
 		if (!node->is_m2m)
-			fail_on_test(testColorspace(pix_mp.pixelformat, pix_mp.colorspace, 
-                                            pix_mp.ycbcr_enc, pix_mp.quantization));
+			fail_on_test(testColorspace(node->has_inputs || node->has_outputs,
+						    pix_mp.pixelformat, pix_mp.colorspace,
+						    pix_mp.ycbcr_enc, pix_mp.quantization));
 		fail_on_test(pix_mp.field == V4L2_FIELD_ANY);
 		ret = check_0(pix_mp.reserved, sizeof(pix_mp.reserved));
 		if (ret)
@@ -545,6 +558,7 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 		fail_on_test(check_0(sdr.reserved, sizeof(sdr.reserved)));
 		break;
 	case V4L2_BUF_TYPE_META_CAPTURE:
+	case V4L2_BUF_TYPE_META_OUTPUT:
 		if (map.find(meta.dataformat) == map.end())
 			return fail("dataformat %08x (%s) for buftype %d not reported by ENUM_FMT\n",
 					meta.dataformat, fcc2s(meta.dataformat).c_str(), type);
@@ -567,14 +581,16 @@ int testGetFormats(struct node *node)
 	for (type = 0; type <= V4L2_BUF_TYPE_LAST; type++) {
 		createInvalidFmt(fmt, clip, type);
 		ret = doioctl(node, VIDIOC_G_FMT, &fmt);
+		if (!ret)
+			node->valid_buftypes |= 1 << type;
+
 		ret = testFormatsType(node, ret, type, fmt);
 
 		if (ret && ret != ENOTTY)
 			return ret;
-		if (!ret) {
+		if (!ret)
 			supported = true;
-			node->valid_buftypes |= 1 << type;
-		}
+
 		switch (type) {
 		case V4L2_BUF_TYPE_VIDEO_CAPTURE:
 		case V4L2_BUF_TYPE_VIDEO_OUTPUT:
@@ -584,6 +600,7 @@ int testGetFormats(struct node *node)
 		case V4L2_BUF_TYPE_SDR_CAPTURE:
 		case V4L2_BUF_TYPE_SDR_OUTPUT:
 		case V4L2_BUF_TYPE_META_CAPTURE:
+		case V4L2_BUF_TYPE_META_OUTPUT:
 			if (ret && (node->g_caps() & buftype2cap[type]))
 				return fail("%s cap set, but no %s formats defined\n",
 					buftype2s(type).c_str(), buftype2s(type).c_str());
@@ -609,6 +626,8 @@ static bool matchFormats(const struct v4l2_format &f1, const struct v4l2_format
 {
 	const struct v4l2_pix_format &pix1 = f1.fmt.pix;
 	const struct v4l2_pix_format &pix2 = f2.fmt.pix;
+	const struct v4l2_window &win1 = f1.fmt.win;
+	const struct v4l2_window &win2 = f2.fmt.win;
 
 	if (f1.type != f2.type)
 		return false;
@@ -617,16 +636,24 @@ static bool matchFormats(const struct v4l2_format &f1, const struct v4l2_format
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
 		if (!memcmp(&f1.fmt.pix, &f2.fmt.pix, sizeof(f1.fmt.pix)))
 			return true;
-		printf("\t\tG_FMT:     %dx%d, %x, %d, %d, %d, %d, %d, %d, %x\n",
-			pix1.width, pix1.height, pix1.pixelformat, pix1.field, pix1.bytesperline,
+		printf("\t\tG_FMT:     %dx%d, %s, %d, %d, %d, %d, %d, %d, %x\n",
+			pix1.width, pix1.height, fcc2s(pix1.pixelformat).c_str(), pix1.field, pix1.bytesperline,
 			pix1.sizeimage, pix1.colorspace, pix1.ycbcr_enc, pix1.quantization, pix1.priv);
-		printf("\t\tTRY/S_FMT: %dx%d, %x, %d, %d, %d, %d, %d, %d, %x\n",
-			pix2.width, pix2.height, pix2.pixelformat, pix2.field, pix2.bytesperline,
+		printf("\t\tTRY/S_FMT: %dx%d, %s, %d, %d, %d, %d, %d, %d, %x\n",
+			pix2.width, pix2.height, fcc2s(pix2.pixelformat).c_str(), pix2.field, pix2.bytesperline,
 			pix2.sizeimage, pix2.colorspace, pix2.ycbcr_enc, pix2.quantization, pix2.priv);
 		return false;
 	case V4L2_BUF_TYPE_VIDEO_OVERLAY:
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT_OVERLAY:
-		return !memcmp(&f1.fmt.win, &f2.fmt.win, sizeof(f1.fmt.win));
+		if (!memcmp(&f1.fmt.win, &f2.fmt.win, sizeof(f1.fmt.win)))
+			return true;
+		printf("\t\tG_FMT:     %dx%d@%dx%d, %d, %x, %p, %d, %p, %x\n",
+			win1.w.width, win1.w.height, win1.w.left, win1.w.top, win1.field,
+			win1.chromakey, win1.clips, win1.clipcount, win1.bitmap, win1.global_alpha);
+		printf("\t\tTRY/S_FMT: %dx%d@%dx%d, %d, %x, %p, %d, %p, %x\n",
+			win2.w.width, win2.w.height, win2.w.left, win2.w.top, win2.field,
+			win2.chromakey, win2.clips, win2.clipcount, win2.bitmap, win2.global_alpha);
+		return false;
 	case V4L2_BUF_TYPE_VBI_CAPTURE:
 	case V4L2_BUF_TYPE_VBI_OUTPUT:
 		return !memcmp(&f1.fmt.vbi, &f2.fmt.vbi, sizeof(f1.fmt.vbi));
@@ -640,6 +667,7 @@ static bool matchFormats(const struct v4l2_format &f1, const struct v4l2_format
 	case V4L2_BUF_TYPE_SDR_OUTPUT:
 		return !memcmp(&f1.fmt.sdr, &f2.fmt.sdr, sizeof(f1.fmt.sdr));
 	case V4L2_BUF_TYPE_META_CAPTURE:
+	case V4L2_BUF_TYPE_META_OUTPUT:
 		return !memcmp(&f1.fmt.meta, &f2.fmt.meta, sizeof(f1.fmt.meta));
 
 	}
@@ -717,6 +745,7 @@ int testTryFormats(struct node *node)
 				pixelformat = fmt.fmt.sdr.pixelformat;
 				break;
 			case V4L2_BUF_TYPE_META_CAPTURE:
+			case V4L2_BUF_TYPE_META_OUTPUT:
 				pixelformat = fmt.fmt.meta.dataformat;
 				break;
 			case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
@@ -748,8 +777,7 @@ int testTryFormats(struct node *node)
 		}
 		ret = testFormatsType(node, ret, type, fmt, true);
 		if (ret)
-			return fail("%s is valid, but TRY_FMT failed to return a format\n",
-					buftype2s(type).c_str());
+			return ret;
 	}
 
 	memset(&fmt, 0, sizeof(fmt));
@@ -763,6 +791,7 @@ int testTryFormats(struct node *node)
 static int testM2MFormats(struct node *node)
 {
 	cv4l_fmt fmt_out;
+	cv4l_fmt fmt;
 	cv4l_fmt fmt_cap;
 	__u32 cap_type = node->g_type();
 	__u32 out_type = v4l_type_invert(cap_type);
@@ -796,6 +825,67 @@ static int testM2MFormats(struct node *node)
 	fail_on_test(fmt_cap.g_xfer_func() != xfer_func);
 	fail_on_test(fmt_cap.g_ycbcr_enc() != ycbcr_enc);
 	fail_on_test(fmt_cap.g_quantization() != quant);
+
+	if (!(node->codec_mask & STATEFUL_ENCODER))
+		return 0;
+
+	fmt = fmt_out;
+	unsigned w = (fmt.g_width() & ~7) - 4;
+	unsigned h = (fmt.g_height() & ~7) - 4;
+	fmt.s_width(w);
+	fmt.s_height(h);
+	node->s_fmt(fmt);
+	fail_on_test(fmt.g_width() < w || fmt.g_width() > fmt_out.g_width());
+	fail_on_test(fmt.g_height() < h || fmt.g_height() > fmt_out.g_height());
+	node->g_fmt(fmt_cap);
+	fail_on_test(fmt_cap.g_width() < fmt.g_width());
+	fail_on_test(fmt_cap.g_height() < fmt.g_height());
+
+	v4l2_selection sel = {
+		.type = fmt.g_type(),
+		.target = V4L2_SEL_TGT_CROP,
+	};
+	if (node->g_selection(sel) == ENOTTY) {
+		fail_on_test(fmt_cap.g_width() != fmt.g_width());
+		fail_on_test(fmt_cap.g_height() != fmt.g_height());
+		return 0;
+	}
+	fail_on_test(sel.r.top || sel.r.left);
+	fail_on_test(sel.r.width != fmt.g_width());
+	fail_on_test(sel.r.height != fmt.g_height());
+	sel.r.width = w;
+	sel.r.height = h;
+	node->s_selection(sel);
+	node->g_selection(sel);
+	fail_on_test(sel.r.top || sel.r.left);
+	fail_on_test(sel.r.width != w);
+	fail_on_test(sel.r.height != h);
+	sel.target = V4L2_SEL_TGT_CROP_BOUNDS;
+	node->g_selection(sel);
+	fail_on_test(sel.r.top || sel.r.left);
+	fail_on_test(sel.r.width != fmt.g_width());
+	fail_on_test(sel.r.height != fmt.g_height());
+	sel.target = V4L2_SEL_TGT_CROP_DEFAULT;
+	node->g_selection(sel);
+	fail_on_test(sel.r.top || sel.r.left);
+	fail_on_test(sel.r.width != fmt.g_width());
+	fail_on_test(sel.r.height != fmt.g_height());
+
+	// It is unlikely that an encoder supports composition,
+	// so warn if this is detected. Should we get encoders
+	// that can actually do this, then this test needs to
+	// be refined.
+	sel.target = V4L2_SEL_TGT_COMPOSE;
+	if (!node->g_selection(sel))
+		warn("The stateful encoder unexpectedly supports composition\n");
+
+	sel.type = fmt_cap.g_type();
+	fail_on_test(!node->g_selection(sel));
+	sel.target = V4L2_SEL_TGT_CROP;
+	fail_on_test(!node->g_selection(sel));
+
+	node->s_fmt(fmt_out);
+
 	return 0;
 }
 
@@ -969,6 +1059,7 @@ int testSetFormats(struct node *node)
 
 			switch (type) {
 			case V4L2_BUF_TYPE_META_CAPTURE:
+			case V4L2_BUF_TYPE_META_OUTPUT:
 				pixelformat = fmt_set.fmt.meta.dataformat;
 				break;
 			case V4L2_BUF_TYPE_SDR_CAPTURE:
@@ -1013,8 +1104,7 @@ int testSetFormats(struct node *node)
 		}
 		ret = testFormatsType(node, ret, type, fmt_set, true);
 		if (ret)
-			return fail("%s is valid, but no S_FMT was implemented\n",
-					buftype2s(type).c_str());
+			return ret;
 
 		fmt_set = fmt;
 		ret = doioctl(node, VIDIOC_S_FMT, &fmt_set);
@@ -1346,7 +1436,6 @@ static int testBasicCrop(struct node *node, unsigned type)
 	};
 	struct v4l2_selection sel_def;
 	struct v4l2_selection sel_bounds;
-	int s_sel_ret = EINVAL;
 
 	fail_on_test(doioctl(node, VIDIOC_G_SELECTION, &sel_crop));
 	fail_on_test(!sel_crop.r.width || !sel_crop.r.height);
@@ -1371,7 +1460,8 @@ static int testBasicCrop(struct node *node, unsigned type)
 
 	// Check handling of invalid type.
 	sel_crop.type = 0xff;
-	fail_on_test(doioctl(node, VIDIOC_S_SELECTION, &sel_crop) != s_sel_ret);
+	int s_sel_ret = doioctl(node, VIDIOC_S_SELECTION, &sel_crop);
+	fail_on_test(s_sel_ret != EINVAL && s_sel_ret != ENOTTY);
 	// Check handling of invalid target.
 	sel_crop.type = type;
 	sel_crop.target = 0xffff;
@@ -1422,7 +1512,8 @@ static int testLegacyCrop(struct node *node)
 			cap.type = V4L2_BUF_TYPE_VIDEO_OUTPUT;
 			break;
 		}
-		fail_on_test(doioctl(node, VIDIOC_CROPCAP, &cap));
+		// Replace with fail_on_test once kernel is fixed for this.
+		warn_on_test(doioctl(node, VIDIOC_CROPCAP, &cap));
 		cap.type = 0xff;
 		fail_on_test(doioctl(node, VIDIOC_CROPCAP, &cap) != EINVAL);
 	} else {
@@ -1480,7 +1571,6 @@ static int testBasicCompose(struct node *node, unsigned type)
 	struct v4l2_selection sel_def;
 	struct v4l2_selection sel_bounds;
 	struct v4l2_selection sel_padded;
-	int s_sel_ret = EINVAL;
 	int ret;
 
 	fail_on_test(doioctl(node, VIDIOC_G_SELECTION, &sel_compose));
@@ -1514,7 +1604,8 @@ static int testBasicCompose(struct node *node, unsigned type)
 
 	// Check handling of invalid type.
 	sel_compose.type = 0xff;
-	fail_on_test(doioctl(node, VIDIOC_S_SELECTION, &sel_compose) != s_sel_ret);
+	int s_sel_ret = doioctl(node, VIDIOC_S_SELECTION, &sel_compose);
+	fail_on_test(s_sel_ret != EINVAL && s_sel_ret != ENOTTY);
 	// Check handling of invalid target.
 	sel_compose.type = type;
 	sel_compose.target = 0xffff;
