@@ -416,8 +416,12 @@ int testInput(struct node *node)
 	int i = 0;
 
 	if (ret == ENOTTY) {
-		if (node->has_inputs)
-			return fail("G_INPUT not supported for a capture device\n");
+		if (node->has_inputs) {
+			if (media_fd < 0)
+				return fail("G_INPUT not supported for a capture device\n");
+			node->has_inputs = false;
+			return ENOTTY;
+		}
 		descr.index = 0;
 		ret = doioctl(node, VIDIOC_ENUMINPUT, &descr);
 		if (ret != ENOTTY)
@@ -747,9 +751,10 @@ static int checkOutput(struct node *node, const struct v4l2_output &descr, unsig
 		return fail("invalid index\n");
 	if (check_ustring(descr.name, sizeof(descr.name)))
 		return fail("invalid name\n");
-	if (descr.type != V4L2_OUTPUT_TYPE_MODULATOR && descr.type != V4L2_OUTPUT_TYPE_ANALOG)
+	if (descr.type != V4L2_OUTPUT_TYPE_MODULATOR && descr.type != V4L2_OUTPUT_TYPE_ANALOG &&
+	    descr.type != V4L2_OUTPUT_TYPE_ANALOGVGAOVERLAY)
 		return fail("invalid type\n");
-	if (descr.type == V4L2_OUTPUT_TYPE_ANALOG && descr.modulator)
+	if (descr.type != V4L2_OUTPUT_TYPE_MODULATOR && descr.modulator)
 		return fail("invalid modulator\n");
 	if (descr.type == V4L2_OUTPUT_TYPE_MODULATOR && node->modulators == 0)
 		return fail("no modulators found for modulator output\n");
@@ -786,8 +791,12 @@ int testOutput(struct node *node)
 	int o = 0;
 
 	if (ret == ENOTTY) {
-		if (node->has_outputs)
-			return fail("G_OUTPUT not supported for an output device\n");
+		if (node->has_outputs) {
+			if (media_fd < 0)
+				return fail("G_OUTPUT not supported for an output device\n");
+			node->has_outputs = false;
+			return ENOTTY;
+		}
 		descr.index = 0;
 		ret = doioctl(node, VIDIOC_ENUMOUTPUT, &descr);
 		if (ret != ENOTTY)
