@@ -47,7 +47,21 @@ int testMediaDeviceInfo(struct node *node)
 	fail_on_test(check_string(mdinfo.driver, sizeof(mdinfo.driver)));
 	fail_on_test(mdinfo.model[0] && check_string(mdinfo.model, sizeof(mdinfo.model)));
 	fail_on_test(mdinfo.serial[0] && check_string(mdinfo.serial, sizeof(mdinfo.serial)));
-	fail_on_test(mdinfo.bus_info[0] && check_string(mdinfo.bus_info, sizeof(mdinfo.bus_info)));
+	if (!mdinfo.bus_info[0]) {
+		warn("empty bus_info\n");
+	} else {
+		fail_on_test(check_string(mdinfo.bus_info, sizeof(mdinfo.bus_info)));
+		// Check for valid prefixes
+		if (memcmp(mdinfo.bus_info, "usb-", 4) &&
+		    memcmp(mdinfo.bus_info, "PCI:", 4) &&
+		    memcmp(mdinfo.bus_info, "PCIe:", 5) &&
+		    memcmp(mdinfo.bus_info, "ISA:", 4) &&
+		    memcmp(mdinfo.bus_info, "I2C:", 4) &&
+		    memcmp(mdinfo.bus_info, "parport", 7) &&
+		    memcmp(mdinfo.bus_info, "platform:", 9) &&
+		    memcmp(mdinfo.bus_info, "rmi4:", 5))
+			return fail("missing bus_info prefix ('%s')\n", mdinfo.bus_info);
+	}
 	fail_on_test(mdinfo.media_version == 0);
 	if (mdinfo.media_version != MEDIA_API_VERSION)
 		fail_on_test(mdinfo.driver_version != mdinfo.media_version);
@@ -157,6 +171,23 @@ int testMediaTopology(struct node *node)
 
 	for (unsigned i = 0; i < topology.num_entities; i++) {
 		media_v2_entity &ent = v2_ents[i];
+		std::string key = ent.name;
+
+		/*
+		 * The v2_entity_names_set set is used to check if a specific
+		 * entity name is reused, and to check if the topology matches
+		 * what ENUM_ENTITIES returns.
+		 *
+		 * However, the size of the entity name array of media_v2_entity
+		 * is 64 characters, while it is 32 for media_entity_desc.
+		 *
+		 * So cut off the last 32 characters before storing the key.
+		 * This means that the first 31 characters of the entity name
+		 * must be unique, otherwise ENUM_ENTITIES would return
+		 * duplicate entity names.
+		 */
+		if (key.length() >= 32)
+			key.erase(31, key.length() - 1);
 
 		if (show_info) {
 			printf("\t\tEntity: 0x%08x (Name: '%s', Function: %s",
@@ -170,11 +201,11 @@ int testMediaTopology(struct node *node)
 		fail_on_test(!ent.id);
 		fail_on_test(checkFunction(ent.function, true));
 		fail_on_test(v2_entities_set.find(ent.id) != v2_entities_set.end());
-		fail_on_test(v2_entity_names_set.find(ent.name) != v2_entity_names_set.end());
+		fail_on_test(v2_entity_names_set.find(key) != v2_entity_names_set.end());
 		if (!MEDIA_V2_ENTITY_HAS_FLAGS(node->media_version))
 			fail_on_test(ent.flags);
 		v2_entities_set.insert(ent.id);
-		v2_entity_names_set.insert(ent.name);
+		v2_entity_names_set.insert(key);
 		v2_entity_map[ent.id] = &ent;
 	}
 	for (unsigned i = 0; i < topology.num_interfaces; i++) {
@@ -193,7 +224,7 @@ int testMediaTopology(struct node *node)
 		fail_on_test(!iface.id);
 		fail_on_test(!iface.intf_type);
 		fail_on_test(iface.intf_type < MEDIA_INTF_T_DVB_BASE);
-		fail_on_test(iface.intf_type > MEDIA_INTF_T_V4L_BASE + 0xff);
+		fail_on_test(iface.intf_type > MEDIA_INTF_T_ALSA_BASE + 0xff);
 		fail_on_test(iface.flags);
 		fail_on_test(v2_interfaces_set.find(iface.id) != v2_interfaces_set.end());
 		v2_interfaces_set.insert(iface.id);
@@ -417,7 +448,7 @@ int testMediaEnum(struct node *node)
 					link_immutable = links.links[i];
 			}
 			if (fl & MEDIA_LNK_FL_DYNAMIC)
-				fail_on_test(!(fl & MEDIA_LNK_FL_IMMUTABLE));
+				fail_on_test(fl & MEDIA_LNK_FL_IMMUTABLE);
 			if (is_sink && (fl & MEDIA_LNK_FL_ENABLED)) {
 				// only one incoming link can be enabled
 				fail_on_test(found_enabled);
@@ -494,7 +525,8 @@ int testMediaSetupLink(struct node *node)
 	return 0;
 }
 
-void walkTopology(struct node &node, struct node &expbuf_node, unsigned frame_count)
+void walkTopology(struct node &node, struct node &expbuf_node,
+		  unsigned frame_count, unsigned all_fmt_frame_count)
 {
 	media_v2_topology topology;
 
@@ -569,7 +601,8 @@ void walkTopology(struct node &node, struct node &expbuf_node, unsigned frame_co
 			continue;
 		}
 
-		testNode(test_node, expbuf_node, type, frame_count);
+		testNode(test_node, test_node, expbuf_node, type,
+			 frame_count, all_fmt_frame_count);
 		test_node.close();
 	}
 }
