@@ -10,9 +10,11 @@
 #include <ctype.h>
 #include <errno.h>
 #include <sys/ioctl.h>
+#include <sys/sysmacros.h>
 #include <sys/time.h>
 #include <dirent.h>
 #include <math.h>
+#include <linux/media.h>
 
 #include "v4l2-ctl.h"
 
@@ -64,9 +66,24 @@ void common_usage(void)
 	       "  -D, --info         show driver info [VIDIOC_QUERYCAP]\n"
 	       "  -d, --device <dev> use device <dev> instead of /dev/video0\n"
 	       "                     if <dev> starts with a digit, then /dev/video<dev> is used\n"
+	       "                     Otherwise if -z was specified earlier, then <dev> is the entity name\n"
+	       "                     or interface ID (if prefixed with 0x) as found in the topology of the\n"
+	       "                     media device with the bus info string as specified by the -z option.\n"
 	       "  -e, --out-device <dev> use device <dev> for output streams instead of the\n"
 	       "                     default device as set with --device\n"
 	       "                     if <dev> starts with a digit, then /dev/video<dev> is used\n"
+	       "                     Otherwise if -z was specified earlier, then <dev> is the entity name\n"
+	       "                     or interface ID (if prefixed with 0x) as found in the topology of the\n"
+	       "                     media device with the bus info string as specified by the -z option.\n"
+	       "  -E, --export-device <dev> use device <dev> for exporting DMA buffers\n"
+	       "                     if <dev> starts with a digit, then /dev/video<dev> is used\n"
+	       "                     Otherwise if -z was specified earlier, then <dev> is the entity name\n"
+	       "                     or interface ID (if prefixed with 0x) as found in the topology of the\n"
+	       "                     media device with the bus info string as specified by the -z option.\n"
+	       "  -z, --media-bus-info <bus-info>\n"
+	       "                     find the media device with the given bus info string. If set, then\n"
+	       "                     -d, -e and -E options can use the entity name or interface ID to refer\n"
+	       "                     to the device nodes.\n"
 	       "  -h, --help         display this help message\n"
 	       "  --help-all         all options\n"
 	       "  --help-io          input/output options\n"
@@ -93,7 +110,9 @@ void common_usage(void)
 #ifndef NO_LIBV4L2
 	       "  -w, --wrapper      use the libv4l2 wrapper library.\n"
 #endif
-	       "  --list-devices     list all v4l devices\n"
+	       "  --list-devices     list all v4l devices. If -z was given, then list just the\n"
+	       "                     devices of the media device with the bus info string as\n"
+	       "                     specified by the -z option.\n"
 	       "  --log-status       log the board status in the kernel log [VIDIOC_LOG_STATUS]\n"
 	       "  --get-priority     query the current access priority [VIDIOC_G_PRIORITY]\n"
 	       "  --set-priority <prio>\n"
@@ -111,6 +130,8 @@ static const char *prefixes[] = {
 	"vbi",
 	"swradio",
 	"v4l-subdev",
+	"v4l-touch",
+	"media",
 	NULL
 };
 
@@ -153,6 +174,70 @@ static bool sort_on_device_name(const std::string &s1, const std::string &s2)
 	return n1 < n2;
 }
 
+static void list_media_devices(const std::string &media_bus_info)
+{
+	DIR *dp;
+	struct dirent *ep;
+	int media_fd = -1;
+	std::map<dev_t, std::string> devices;
+
+	dp = opendir("/dev");
+	if (dp == NULL) {
+		perror ("Couldn't open the directory");
+		return;
+	}
+	while ((ep = readdir(dp))) {
+		std::string s("/dev/");
+
+		s += ep->d_name;
+		if (memcmp(ep->d_name, "media", 5)) {
+			if (!is_v4l_dev(ep->d_name))
+				continue;
+			struct stat st;
+			if (stat(s.c_str(), &st))
+				continue;
+			devices[st.st_rdev] = s;
+			continue;
+		}
+		int fd = open(s.c_str(), O_RDWR);
+
+		if (fd < 0)
+			continue;
+		struct media_device_info mdi;
+
+		if (!ioctl(fd, MEDIA_IOC_DEVICE_INFO, &mdi) &&
+		    media_bus_info == mdi.bus_info) {
+			media_fd = fd;
+			printf("%s\n", s.c_str());
+		} else {
+			close(fd);
+		}
+	}
+	closedir(dp);
+	if (media_fd < 0)
+		return;
+
+	media_v2_topology topology;
+	memset(&topology, 0, sizeof(topology));
+	if (ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology)) {
+		close(media_fd);
+		return;
+	}
+
+	media_v2_interface *ifaces = new media_v2_interface[topology.num_interfaces];
+	topology.ptr_interfaces = (__u64)ifaces;
+
+	if (!ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology))
+		for (unsigned i = 0; i < topology.num_interfaces; i++) {
+			dev_t dev = makedev(ifaces[i].devnode.major,
+					    ifaces[i].devnode.minor);
+
+			if (devices.find(dev) != devices.end())
+				printf("%s\n", devices[dev].c_str());
+		}
+	close(media_fd);
+}
+
 static void list_devices()
 {
 	DIR *dp;
@@ -209,16 +294,34 @@ static void list_devices()
 			iter != files.end(); ++iter) {
 		int fd = open(iter->c_str(), O_RDWR);
 		std::string bus_info;
+		std::string card;
 
 		if (fd < 0)
 			continue;
 		int err = ioctl(fd, VIDIOC_QUERYCAP, &vcap);
+		if (err) {
+			struct media_device_info mdi;
+
+			err = ioctl(fd, MEDIA_IOC_DEVICE_INFO, &mdi);
+			if (!err) {
+				if (mdi.bus_info[0])
+					bus_info = mdi.bus_info;
+				else
+					bus_info = std::string("platform:") + mdi.driver;
+				if (mdi.model[0])
+					card = mdi.model;
+				else
+					card = mdi.driver;
+			}
+		} else {
+			bus_info = (const char *)vcap.bus_info;
+			card = (const char *)vcap.card;
+		}
 		close(fd);
 		if (err)
 			continue;
-		bus_info = (const char *)vcap.bus_info;
 		if (cards[bus_info].empty())
-			cards[bus_info] += std::string((char *)vcap.card) + " (" + bus_info + "):\n";
+			cards[bus_info] += card + " (" + bus_info + "):\n";
 		cards[bus_info] += "\t" + (*iter);
 		if (!(links[*iter].empty()))
 			cards[bus_info] += " <- " + links[*iter];
@@ -686,12 +789,11 @@ static bool parse_next_subopt(char **subs, char **value)
 
 	if (opt < 0 || *value)
 		return false;
-	fprintf(stderr, "No value given to suboption <%s>\n",
-			subopts[opt]);
+	fprintf(stderr, "Missing suboption value\n");
 	return true;
 }
 
-void common_cmd(int ch, char *optarg)
+void common_cmd(const std::string &media_bus_info, int ch, char *optarg)
 {
 	char *value, *subs;
 
@@ -738,7 +840,10 @@ void common_cmd(int ch, char *optarg)
 		prio = (enum v4l2_priority)strtoul(optarg, 0L, 0);
 		break;
 	case OptListDevices:
-		list_devices();
+		if (media_bus_info.empty())
+			list_devices();
+		else
+			list_media_devices(media_bus_info);
 		break;
 	}
 }
@@ -1091,11 +1196,7 @@ void common_get(cv4l_fd &_fd)
 
 void common_list(cv4l_fd &fd)
 {
-	if (options[OptListCtrlsMenus]) {
-		list_controls(fd.g_fd(), 1);
-	}
-
-	if (options[OptListCtrls]) {
-		list_controls(fd.g_fd(), 0);
+	if (options[OptListCtrls] || options[OptListCtrlsMenus]) {
+		list_controls(fd.g_fd(), options[OptListCtrlsMenus]);
 	}
 }
