@@ -36,6 +36,9 @@ void misc_usage(void)
 	       "  --poll-for-event <event>\n"
 	       "                     poll for an event [VIDIOC_DQEVENT]\n"
 	       "                     see --wait-for-event for possible events\n"
+	       "  --epoll-for-event <event>\n"
+	       "                     epoll for an event [VIDIOC_DQEVENT]\n"
+	       "                     see --wait-for-event for possible events\n"
 	       "  -P, --get-parm     display video parameters [VIDIOC_G_PARM]\n"
 	       "  -p, --set-parm <fps>\n"
 	       "                     set video framerate in <fps> [VIDIOC_S_PARM]\n"
