@@ -22,6 +22,8 @@ static bool query_and_set_dv_timings = false;
 static bool cleared_dv_timings = false;
 static int enum_and_set_dv_timings = -1;
 static unsigned set_dv_timing_opts;
+static __u32 list_dv_timings_pad;
+static __u32 dv_timings_cap_pad;
 
 void stds_usage(void)
 {
@@ -29,7 +31,7 @@ void stds_usage(void)
 	       "  --list-standards   display supported video standards [VIDIOC_ENUMSTD]\n"
 	       "  -S, --get-standard\n"
 	       "                     query the video standard [VIDIOC_G_STD]\n"
-	       "  -s, --set-standard=<num>\n"
+	       "  -s, --set-standard <num>\n"
 	       "                     set the video standard to <num> [VIDIOC_S_STD]\n"
 	       "                     <num> a numerical v4l2_std value, or one of:\n"
 	       "                     pal or pal-X (X = B/G/H/N/Nc/I/D/K/M/60) (V4L2_STD_PAL)\n"
@@ -37,7 +39,9 @@ void stds_usage(void)
 	       "                     secam or secam-X (X = B/G/H/D/K/L/Lc) (V4L2_STD_SECAM)\n"
 	       "  --get-detected-standard\n"
 	       "                     display detected input video standard [VIDIOC_QUERYSTD]\n"
-	       "  --list-dv-timings  list supp. standard dv timings [VIDIOC_ENUM_DV_TIMINGS]\n"
+	       "  --list-dv-timings [<pad>]\n"
+	       "                     list supp. standard dv timings [VIDIOC_ENUM_DV_TIMINGS]\n"
+	       "                     for subdevs the pad can be specified (default is 0)\n"
 	       "  --set-dv-bt-timings\n"
 	       "                     query: use the output of VIDIOC_QUERY_DV_TIMINGS\n"
 	       "                     index=<index>: use the index as provided by --list-dv-timings\n"
@@ -66,8 +70,9 @@ void stds_usage(void)
 	       "                     standard [VIDIOC_S_DV_TIMINGS]\n"
 	       "  --get-dv-timings   get the digital video timings in use [VIDIOC_G_DV_TIMINGS]\n"
 	       "  --query-dv-timings query the detected dv timings [VIDIOC_QUERY_DV_TIMINGS]\n"
-	       "  --get-dv-timings-cap\n"
+	       "  --get-dv-timings-cap [<pad>]\n"
 	       "                     get the dv timings capabilities [VIDIOC_DV_TIMINGS_CAP]\n"
+	       "                     for subdevs the pad can be specified (default is 0)\n"
 	       );
 }
 
@@ -357,44 +362,6 @@ static void parse_dv_bt_timings(char *optarg, struct v4l2_dv_timings *dv_timings
 	}
 }
 
-static const flag_def dv_standards_def[] = {
-	{ V4L2_DV_BT_STD_CEA861, "CTA-861" },
-	{ V4L2_DV_BT_STD_DMT, "DMT" },
-	{ V4L2_DV_BT_STD_CVT, "CVT" },
-	{ V4L2_DV_BT_STD_GTF, "GTF" },
-	{ V4L2_DV_BT_STD_SDI, "SDI" },
-	{ 0, NULL }
-};
-
-static std::string dvflags2s(unsigned vsync, int val)
-{
-	std::string s;
-
-	if (val & V4L2_DV_FL_REDUCED_BLANKING)
-		s += vsync == 8 ?
-			"reduced blanking v2, " :
-			"reduced blanking, ";
-	if (val & V4L2_DV_FL_CAN_REDUCE_FPS)
-		s += "framerate can be reduced by 1/1.001, ";
-	if (val & V4L2_DV_FL_REDUCED_FPS)
-		s += "framerate is reduced by 1/1.001, ";
-	if (val & V4L2_DV_FL_HALF_LINE)
-		s += "half-line, ";
-	if (val & V4L2_DV_FL_IS_CE_VIDEO)
-		s += "CE-video, ";
-	if (val & V4L2_DV_FL_FIRST_FIELD_EXTRA_LINE)
-		s += "first field has extra line, ";
-	if (val & V4L2_DV_FL_HAS_PICTURE_ASPECT)
-		s += "has picture aspect, ";
-	if (val & V4L2_DV_FL_HAS_CEA861_VIC)
-		s += "has CTA-861 VIC, ";
-	if (val & V4L2_DV_FL_HAS_HDMI_VIC)
-		s += "has HDMI VIC, ";
-	if (s.length())
-		return s.erase(s.length() - 2, 2);
-	return s;
-}
-
 static void print_dv_timings(const struct v4l2_dv_timings *t)
 {
 	const struct v4l2_bt_timings *bt;
@@ -453,7 +420,7 @@ static void print_dv_timings(const struct v4l2_dv_timings *t)
 			printf("\tVertical sync: %d\n", bt->il_vsync);
 			printf("\tVertical backporch: %d\n", bt->il_vbackporch);
 		}
-		printf("\tStandards: %s\n", flags2s(bt->standards, dv_standards_def).c_str());
+		printf("\tStandards: %s\n", dv_standards2s(bt->standards).c_str());
 		if (bt->flags & V4L2_DV_FL_HAS_PICTURE_ASPECT)
 			printf("\tPicture aspect: %u:%u\n",
 			       bt->picture_aspect.numerator,
@@ -503,11 +470,21 @@ void stds_cmd(int ch, char *optarg)
 	case OptSetDvBtTimings:
 		parse_dv_bt_timings(optarg, &dv_timings);
 		break;
+	case OptListDvTimings:
+		if (optarg)
+			list_dv_timings_pad = strtoul(optarg, 0L, 0);
+		break;
+	case OptGetDvTimingsCap:
+		if (optarg)
+			dv_timings_cap_pad = strtoul(optarg, 0L, 0);
+		break;
 	}
 }
 
-void stds_set(int fd)
+void stds_set(cv4l_fd &_fd)
 {
+	int fd = _fd.g_fd();
+
 	if (options[OptSetStandard]) {
 		if (standard & (1ULL << 63)) {
 			struct v4l2_standard vs;
@@ -602,12 +579,14 @@ void stds_set(int fd)
 	}
 }
 
-void stds_get(int fd)
+void stds_get(cv4l_fd &_fd)
 {
+	int fd = _fd.g_fd();
+
 	if (options[OptGetStandard]) {
 		if (doioctl(fd, VIDIOC_G_STD, &standard) == 0) {
 			printf("Video Standard = 0x%08llx\n", (unsigned long long)standard);
-			print_v4lstd((unsigned long long)standard);
+			printf("\t%s\n", std2s(standard, "\n\t").c_str());
 		}
 	}
 
@@ -621,14 +600,8 @@ void stds_get(int fd)
 	if (options[OptGetDvTimingsCap]) {
 		struct v4l2_dv_timings_cap dv_timings_cap = {};
 
+		dv_timings_cap.pad = dv_timings_cap_pad;
 		if (doioctl(fd, VIDIOC_DV_TIMINGS_CAP, &dv_timings_cap) >= 0) {
-			static const flag_def dv_caps_def[] = {
-				{ V4L2_DV_BT_CAP_INTERLACED, "Interlaced" },
-				{ V4L2_DV_BT_CAP_PROGRESSIVE, "Progressive" },
-				{ V4L2_DV_BT_CAP_REDUCED_BLANKING, "Reduced Blanking" },
-				{ V4L2_DV_BT_CAP_CUSTOM, "Custom Formats" },
-				{ 0, NULL }
-			};
 			struct v4l2_bt_timings_cap *bt = &dv_timings_cap.bt;
 
 			printf("DV timings capabilities:\n");
@@ -642,9 +615,9 @@ void stds_get(int fd)
 				printf("\tMinimum PClock: %llu\n", bt->min_pixelclock);
 				printf("\tMaximum PClock: %llu\n", bt->max_pixelclock);
 				printf("\tStandards: %s\n",
-					flags2s(bt->standards, dv_standards_def).c_str());
+					dv_standards2s(bt->standards).c_str());
 				printf("\tCapabilities: %s\n",
-					flags2s(bt->capabilities, dv_caps_def).c_str());
+					dv_caps2s(bt->capabilities).c_str());
 			}
 		}
 	}
@@ -652,7 +625,7 @@ void stds_get(int fd)
         if (options[OptQueryStandard]) {
 		if (doioctl(fd, VIDIOC_QUERYSTD, &standard) == 0) {
 			printf("Video Standard = 0x%08llx\n", (unsigned long long)standard);
-			print_v4lstd((unsigned long long)standard);
+			printf("\t%s\n", std2s(standard, "\n\t").c_str());
 		}
 	}
 
@@ -662,8 +635,10 @@ void stds_get(int fd)
         }
 }
 
-void stds_list(int fd)
+void stds_list(cv4l_fd &_fd)
 {
+	int fd = _fd.g_fd();
+
 	if (options[OptListStandards]) {
 		struct v4l2_standard vs;
 
@@ -692,6 +667,7 @@ void stds_list(int fd)
 		struct v4l2_enum_dv_timings dv_enum_timings = {};
 
 		printf("ioctl: VIDIOC_ENUM_DV_TIMINGS\n");
+		dv_enum_timings.pad = list_dv_timings_pad;
 		while (test_ioctl(fd, VIDIOC_ENUM_DV_TIMINGS, &dv_enum_timings) >= 0) {
 			if (options[OptConcise]) {
 				printf("\t%d:", dv_enum_timings.index);
