@@ -30,17 +30,17 @@ void tuner_usage(void)
 {
 	printf("\nTuner/Modulator options:\n"
 	       "  -F, --get-freq     query the frequency [VIDIOC_G_FREQUENCY]\n"
-	       "  -f, --set-freq=<freq>\n"
+	       "  -f, --set-freq <freq>\n"
 	       "                     set the frequency to <freq> MHz [VIDIOC_S_FREQUENCY]\n"
 	       "  -T, --get-tuner    query the tuner settings [VIDIOC_G_TUNER]\n"
-	       "  -t, --set-tuner=<mode>\n"
+	       "  -t, --set-tuner <mode>\n"
 	       "                     set the audio mode of the tuner [VIDIOC_S_TUNER]\n"
 	       "                     Possible values: mono, stereo, lang2, lang1, bilingual\n"
-	       "  --tuner-index=<idx> Use idx as tuner idx for tuner/modulator commands\n"
+	       "  --tuner-index <idx> Use idx as tuner idx for tuner/modulator commands\n"
 	       "  --list-freq-bands  display all frequency bands for the tuner/modulator\n"
 	       "                     [VIDIOC_ENUM_FREQ_BANDS]\n"
 	       "  --get-modulator    query the modulator settings [VIDIOC_G_MODULATOR]\n"
-	       "  --set-modulator=<txsubchans>\n"
+	       "  --set-modulator <txsubchans>\n"
 	       "                     set the sub-carrier modulation [VIDIOC_S_MODULATOR]\n"
 	       "                     <txsubchans> is one of:\n"
 	       "                     mono:	 Modulate as mono\n"
@@ -50,7 +50,7 @@ void tuner_usage(void)
 	       "                     bilingual:	 Modulate as bilingual\n"
 	       "                     mono-sap:	 Modulate as mono with Second Audio Program\n"
 	       "                     stereo-sap: Modulate as stereo with Second Audio Program\n"
-	       "  --freq-seek=dir=<0/1>,wrap=<0/1>,spacing=<hz>,low=<freq>,high=<freq>\n"
+	       "  --freq-seek dir=<0/1>,wrap=<0/1>,spacing=<hz>,low=<freq>,high=<freq>\n"
 	       "                     perform a hardware frequency seek [VIDIOC_S_HW_FREQ_SEEK]\n"
 	       "                     dir is 0 (seek downward) or 1 (seek upward)\n"
 	       "                     wrap is 0 (do not wrap around) or 1 (wrap around)\n"
@@ -257,8 +257,9 @@ void tuner_cmd(int ch, char *optarg)
 	}
 }
 
-void tuner_set(int fd)
+void tuner_set(cv4l_fd &_fd)
 {
+	int fd = _fd.g_fd();
 	__u32 type = (capabilities & V4L2_CAP_RADIO) ?
 		V4L2_TUNER_RADIO : V4L2_TUNER_ANALOG_TV;
 	double fac = 16;
@@ -361,8 +362,10 @@ void tuner_set(int fd)
 	}
 }
 
-void tuner_get(int fd)
+void tuner_get(cv4l_fd &_fd)
 {
+	int fd = _fd.g_fd();
+
 	if (options[OptGetFreq]) {
 		double fac = 16;
 
