@@ -100,6 +100,7 @@ void vidcap_cmd(int ch, char *optarg)
 {
 	__u32 colorspace, xfer_func, ycbcr, quantization;
 	char *value, *subs;
+	bool be_pixfmt;
 
 	switch (ch) {
 	case OptSetVideoFormat:
@@ -113,11 +114,18 @@ void vidcap_cmd(int ch, char *optarg)
 		}
 		break;
 	case OptListFrameSizes:
-		if (strlen(optarg) == 4)
+		be_pixfmt = strlen(optarg) == 7 && !memcmp(optarg + 4, "-BE", 3);
+		if (be_pixfmt || strlen(optarg) == 4) {
 			frmsize.pixel_format = v4l2_fourcc(optarg[0], optarg[1],
-					optarg[2], optarg[3]);
-		else
+							   optarg[2], optarg[3]);
+			if (be_pixfmt)
+				frmsize.pixel_format |= 1U << 31;
+		} else if (isdigit(optarg[0])) {
 			frmsize.pixel_format = strtol(optarg, 0L, 0);
+		} else {
+			fprintf(stderr, "The pixelformat '%s' is invalid\n", optarg);
+			exit(1);
+		}
 		break;
 	case OptListFrameIntervals:
 		subs = optarg;
@@ -137,12 +145,19 @@ void vidcap_cmd(int ch, char *optarg)
 				frmival.height = strtol(value, 0L, 0);
 				break;
 			case 2:
-				if (strlen(value) == 4)
+				be_pixfmt = strlen(value) == 7 && !memcmp(value + 4, "-BE", 3);
+				if (be_pixfmt || strlen(value) == 4) {
 					frmival.pixel_format =
 						v4l2_fourcc(value[0], value[1],
-								value[2], value[3]);
-				else
+							    value[2], value[3]);
+					if (be_pixfmt)
+						frmival.pixel_format |= 1U << 31;
+				} else if (isdigit(optarg[0])) {
 					frmival.pixel_format = strtol(value, 0L, 0);
+				} else {
+					fprintf(stderr, "The pixelformat '%s' is invalid\n", optarg);
+					exit(1);
+				}
 				break;
 			default:
 				vidcap_usage();
@@ -153,74 +168,98 @@ void vidcap_cmd(int ch, char *optarg)
 	}
 }
 
-void vidcap_set(cv4l_fd &_fd)
+int vidcap_get_and_update_fmt(cv4l_fd &_fd, struct v4l2_format &vfmt)
 {
 	int fd = _fd.g_fd();
 	int ret;
 
+	memset(&vfmt, 0, sizeof(vfmt));
+	vfmt.fmt.pix.priv = priv_magic;
+	vfmt.type = vidcap_buftype;
+
+	ret = doioctl(fd, VIDIOC_G_FMT, &vfmt);
+	if (ret)
+		return ret;
+
+	if (is_multiplanar) {
+		if (set_fmts & FmtWidth)
+			vfmt.fmt.pix_mp.width = width;
+		if (set_fmts & FmtHeight)
+			vfmt.fmt.pix_mp.height = height;
+		if (set_fmts & FmtPixelFormat) {
+			vfmt.fmt.pix_mp.pixelformat = pixfmt;
+			if (vfmt.fmt.pix_mp.pixelformat < 256) {
+				vfmt.fmt.pix_mp.pixelformat = pixfmt =
+					find_pixel_format(fd, vfmt.fmt.pix_mp.pixelformat,
+							  false, true);
+			}
+		}
+		if (set_fmts & FmtField)
+			vfmt.fmt.pix_mp.field = field;
+		if (set_fmts & FmtFlags)
+			vfmt.fmt.pix_mp.flags = flags;
+		if (set_fmts & FmtBytesPerLine) {
+			for (unsigned i = 0; i < VIDEO_MAX_PLANES; i++)
+				vfmt.fmt.pix_mp.plane_fmt[i].bytesperline =
+					bytesperline[i];
+		} else {
+			/*
+			 * G_FMT might return bytesperline values > width,
+			 * reset them to 0 to force the driver to update them
+			 * to the closest value for the new width.
+			 */
+			for (unsigned i = 0; i < vfmt.fmt.pix_mp.num_planes; i++)
+				vfmt.fmt.pix_mp.plane_fmt[i].bytesperline = 0;
+		}
+	} else {
+		if (set_fmts & FmtWidth)
+			vfmt.fmt.pix.width = width;
+		if (set_fmts & FmtHeight)
+			vfmt.fmt.pix.height = height;
+		if (set_fmts & FmtPixelFormat) {
+			vfmt.fmt.pix.pixelformat = pixfmt;
+			if (vfmt.fmt.pix.pixelformat < 256) {
+				vfmt.fmt.pix.pixelformat = pixfmt =
+					find_pixel_format(fd, vfmt.fmt.pix.pixelformat,
+							  false, false);
+			}
+		}
+		if (set_fmts & FmtField)
+			vfmt.fmt.pix.field = field;
+		if (set_fmts & FmtFlags)
+			vfmt.fmt.pix.flags = flags;
+		if (set_fmts & FmtBytesPerLine) {
+			vfmt.fmt.pix.bytesperline = bytesperline[0];
+		} else {
+			/*
+			 * G_FMT might return a bytesperline value > width,
+			 * reset this to 0 to force the driver to update it
+			 * to the closest value for the new width.
+			 */
+			vfmt.fmt.pix.bytesperline = 0;
+		}
+	}
+
+	if ((set_fmts & FmtPixelFormat) &&
+	    !valid_pixel_format(fd, pixfmt, false, is_multiplanar)) {
+		if (pixfmt)
+			fprintf(stderr, "The pixelformat '%s' is invalid\n",
+				fcc2s(pixfmt).c_str());
+		else
+			fprintf(stderr, "The pixelformat index was invalid\n");
+		return -EINVAL;
+	}
+	return 0;
+}
+
+void vidcap_set(cv4l_fd &_fd)
+{
 	if (options[OptSetVideoFormat] || options[OptTryVideoFormat]) {
+		int fd = _fd.g_fd();
+		int ret;
 		struct v4l2_format vfmt;
 
-		memset(&vfmt, 0, sizeof(vfmt));
-		vfmt.fmt.pix.priv = priv_magic;
-		vfmt.type = vidcap_buftype;
-
-		if (doioctl(fd, VIDIOC_G_FMT, &vfmt) == 0) {
-			if (is_multiplanar) {
-				if (set_fmts & FmtWidth)
-					vfmt.fmt.pix_mp.width = width;
-				if (set_fmts & FmtHeight)
-					vfmt.fmt.pix_mp.height = height;
-				if (set_fmts & FmtPixelFormat) {
-					vfmt.fmt.pix_mp.pixelformat = pixfmt;
-					if (vfmt.fmt.pix_mp.pixelformat < 256) {
-						vfmt.fmt.pix_mp.pixelformat =
-							find_pixel_format(fd, vfmt.fmt.pix_mp.pixelformat,
-									false, true);
-					}
-				}
-				if (set_fmts & FmtField)
-					vfmt.fmt.pix_mp.field = field;
-				if (set_fmts & FmtFlags)
-					vfmt.fmt.pix_mp.flags = flags;
-				if (set_fmts & FmtBytesPerLine) {
-					for (unsigned i = 0; i < VIDEO_MAX_PLANES; i++)
-						vfmt.fmt.pix_mp.plane_fmt[i].bytesperline =
-							bytesperline[i];
-				} else {
-					/* G_FMT might return bytesperline values > width,
-					 * reset them to 0 to force the driver to update them
-					 * to the closest value for the new width. */
-					for (unsigned i = 0; i < vfmt.fmt.pix_mp.num_planes; i++)
-						vfmt.fmt.pix_mp.plane_fmt[i].bytesperline = 0;
-				}
-			} else {
-				if (set_fmts & FmtWidth)
-					vfmt.fmt.pix.width = width;
-				if (set_fmts & FmtHeight)
-					vfmt.fmt.pix.height = height;
-				if (set_fmts & FmtPixelFormat) {
-					vfmt.fmt.pix.pixelformat = pixfmt;
-					if (vfmt.fmt.pix.pixelformat < 256) {
-						vfmt.fmt.pix.pixelformat =
-							find_pixel_format(fd, vfmt.fmt.pix.pixelformat,
-									false, false);
-					}
-				}
-				if (set_fmts & FmtField)
-					vfmt.fmt.pix.field = field;
-				if (set_fmts & FmtFlags)
-					vfmt.fmt.pix.flags = flags;
-				if (set_fmts & FmtBytesPerLine) {
-					vfmt.fmt.pix.bytesperline = bytesperline[0];
-				} else {
-					/* G_FMT might return a bytesperline value > width,
-					 * reset this to 0 to force the driver to update it
-					 * to the closest value for the new width. */
-					vfmt.fmt.pix.bytesperline = 0;
-				}
-			}
-
+		if (vidcap_get_and_update_fmt(_fd, vfmt) == 0) {
 			if (options[OptSetVideoFormat])
 				ret = doioctl(fd, VIDIOC_S_FMT, &vfmt);
 			else
@@ -261,8 +300,24 @@ void vidcap_list(cv4l_fd &fd)
 	}
 
 	if (options[OptListFrameSizes]) {
-		printf("ioctl: VIDIOC_ENUM_FRAMESIZES\n");
 		frmsize.index = 0;
+		if (frmsize.pixel_format < 256) {
+			frmsize.pixel_format =
+				find_pixel_format(fd.g_fd(), frmsize.pixel_format,
+						  false, is_multiplanar);
+			if (!frmsize.pixel_format) {
+				fprintf(stderr, "The pixelformat index was invalid\n");
+				exit(1);
+			}
+		}
+		if (!valid_pixel_format(fd.g_fd(), frmsize.pixel_format, false, is_multiplanar) &&
+		    !valid_pixel_format(fd.g_fd(), frmsize.pixel_format, true, is_multiplanar)) {
+			fprintf(stderr, "The pixelformat '%s' is invalid\n",
+				fcc2s(frmsize.pixel_format).c_str());
+			exit(1);
+		}
+
+		printf("ioctl: VIDIOC_ENUM_FRAMESIZES\n");
 		while (test_ioctl(fd.g_fd(), VIDIOC_ENUM_FRAMESIZES, &frmsize) >= 0) {
 			print_frmsize(frmsize, "");
 			frmsize.index++;
@@ -270,8 +325,23 @@ void vidcap_list(cv4l_fd &fd)
 	}
 
 	if (options[OptListFrameIntervals]) {
-		printf("ioctl: VIDIOC_ENUM_FRAMEINTERVALS\n");
 		frmival.index = 0;
+		if (frmival.pixel_format < 256) {
+			frmival.pixel_format =
+				find_pixel_format(fd.g_fd(), frmival.pixel_format,
+						  false, is_multiplanar);
+			if (!frmival.pixel_format) {
+				fprintf(stderr, "The pixelformat index was invalid\n");
+				exit(1);
+			}
+		}
+		if (!valid_pixel_format(fd.g_fd(), frmival.pixel_format, false, is_multiplanar)) {
+			fprintf(stderr, "The pixelformat '%s' is invalid\n",
+				fcc2s(frmival.pixel_format).c_str());
+			exit(1);
+		}
+
+		printf("ioctl: VIDIOC_ENUM_FRAMEINTERVALS\n");
 		while (test_ioctl(fd.g_fd(), VIDIOC_ENUM_FRAMEINTERVALS, &frmival) >= 0) {
 			print_frmival(frmival, "");
 			frmival.index++;
