@@ -28,6 +28,7 @@
 #include <getopt.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/epoll.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <errno.h>
@@ -72,6 +73,8 @@ static struct option long_options[] = {
 	{"all", no_argument, 0, OptAll},
 	{"device", required_argument, 0, OptSetDevice},
 	{"out-device", required_argument, 0, OptSetOutDevice},
+	{"export-device", required_argument, 0, OptSetExportDevice},
+	{"media-bus-info", required_argument, 0, OptMediaBusInfo},
 	{"get-fmt-video", no_argument, 0, OptGetVideoFormat},
 	{"set-fmt-video", required_argument, 0, OptSetVideoFormat},
 	{"try-fmt-video", required_argument, 0, OptTryVideoFormat},
@@ -122,6 +125,7 @@ static struct option long_options[] = {
 	{"list-formats-out", no_argument, 0, OptListOutFormats},
 	{"list-formats-out-ext", no_argument, 0, OptListOutFormatsExt},
 	{"list-formats-meta", no_argument, 0, OptListMetaFormats},
+	{"list-formats-meta-out", no_argument, 0, OptListMetaOutFormats},
 	{"list-subdev-mbus-codes", optional_argument, 0, OptListSubDevMBusCodes},
 	{"list-subdev-framesizes", required_argument, 0, OptListSubDevFrameSizes},
 	{"list-subdev-frameintervals", required_argument, 0, OptListSubDevFrameIntervals},
@@ -174,6 +178,9 @@ static struct option long_options[] = {
 	{"get-fmt-meta", no_argument, 0, OptGetMetaFormat},
 	{"set-fmt-meta", required_argument, 0, OptSetMetaFormat},
 	{"try-fmt-meta", required_argument, 0, OptTryMetaFormat},
+	{"get-fmt-meta-out", no_argument, 0, OptGetMetaOutFormat},
+	{"set-fmt-meta-out", required_argument, 0, OptSetMetaOutFormat},
+	{"try-fmt-meta-out", required_argument, 0, OptTryMetaOutFormat},
 	{"get-subdev-fmt", optional_argument, 0, OptGetSubDevFormat},
 	{"set-subdev-fmt", required_argument, 0, OptSetSubDevFormat},
 	{"try-subdev-fmt", required_argument, 0, OptTrySubDevFormat},
@@ -210,6 +217,7 @@ static struct option long_options[] = {
 	{"set-priority", required_argument, 0, OptSetPriority},
 	{"wait-for-event", required_argument, 0, OptWaitForEvent},
 	{"poll-for-event", required_argument, 0, OptPollForEvent},
+	{"epoll-for-event", required_argument, 0, OptEPollForEvent},
 	{"overlay", required_argument, 0, OptOverlay},
 	{"sleep", required_argument, 0, OptSleep},
 	{"list-devices", no_argument, 0, OptListDevices},
@@ -238,6 +246,7 @@ static struct option long_options[] = {
 	{"list-buffers-sdr", no_argument, 0, OptListBuffersSdr},
 	{"list-buffers-sdr-out", no_argument, 0, OptListBuffersSdrOut},
 	{"list-buffers-meta", no_argument, 0, OptListBuffersMeta},
+	{"list-buffers-meta-out", no_argument, 0, OptListBuffersMetaOut},
 	{"stream-count", required_argument, 0, OptStreamCount},
 	{"stream-skip", required_argument, 0, OptStreamSkip},
 	{"stream-loop", no_argument, 0, OptStreamLoop},
@@ -250,6 +259,7 @@ static struct option long_options[] = {
 	{"stream-lossless", no_argument, 0, OptStreamLossless},
 	{"stream-to-host", required_argument, 0, OptStreamToHost},
 #endif
+	{"stream-buf-caps", no_argument, 0, OptStreamBufCaps},
 	{"stream-mmap", optional_argument, 0, OptStreamMmap},
 	{"stream-user", optional_argument, 0, OptStreamUser},
 	{"stream-dmabuf", no_argument, 0, OptStreamDmaBuf},
@@ -269,6 +279,7 @@ static struct option long_options[] = {
 	{"stream-out-hor-speed", required_argument, 0, OptStreamOutHorSpeed},
 	{"stream-out-vert-speed", required_argument, 0, OptStreamOutVertSpeed},
 	{"stream-out-perc-fill", required_argument, 0, OptStreamOutPercFill},
+	{"stream-out-buf-caps", no_argument, 0, OptStreamOutBufCaps},
 	{"stream-out-mmap", optional_argument, 0, OptStreamOutMmap},
 	{"stream-out-user", optional_argument, 0, OptStreamOutUser},
 	{"stream-out-dmabuf", no_argument, 0, OptStreamOutDmaBuf},
@@ -336,6 +347,10 @@ static bool is_rgb_or_hsv(__u32 pixelformat)
 	case V4L2_PIX_FMT_YUV555:
 	case V4L2_PIX_FMT_YUV565:
 	case V4L2_PIX_FMT_YUV32:
+	case V4L2_PIX_FMT_AYUV32:
+	case V4L2_PIX_FMT_XYUV32:
+	case V4L2_PIX_FMT_VUYA32:
+	case V4L2_PIX_FMT_VUYX32:
 	case V4L2_PIX_FMT_YUV410:
 	case V4L2_PIX_FMT_YUV420:
 	case V4L2_PIX_FMT_HI240:
@@ -505,6 +520,7 @@ void printfmt(int fd, const struct v4l2_format &vfmt)
 		printf("\tBuffer Size     : %u\n", vfmt.fmt.sdr.buffersize);
 		break;
 	case V4L2_BUF_TYPE_META_CAPTURE:
+	case V4L2_BUF_TYPE_META_OUTPUT:
 		printf("\tSample Format   : '%s'%s\n", fcc2s(vfmt.fmt.meta.dataformat).c_str(),
 		       printfmtname(fd, vfmt.type, vfmt.fmt.meta.dataformat).c_str());
 		printf("\tBuffer Size     : %u\n", vfmt.fmt.meta.buffersize);
@@ -547,6 +563,12 @@ void print_frmsize(const struct v4l2_frmsizeenum &frmsize, const char *prefix)
 	printf("%s\tSize: %s ", prefix, frmtype2s(frmsize.type).c_str());
 	if (frmsize.type == V4L2_FRMSIZE_TYPE_DISCRETE) {
 		printf("%dx%d", frmsize.discrete.width, frmsize.discrete.height);
+	} else if (frmsize.type == V4L2_FRMSIZE_TYPE_CONTINUOUS) {
+		printf("%dx%d - %dx%d",
+				frmsize.stepwise.min_width,
+				frmsize.stepwise.min_height,
+				frmsize.stepwise.max_width,
+				frmsize.stepwise.max_height);
 	} else if (frmsize.type == V4L2_FRMSIZE_TYPE_STEPWISE) {
 		printf("%dx%d - %dx%d with step %d/%d",
 				frmsize.stepwise.min_width,
@@ -759,16 +781,17 @@ int parse_fmt(char *optarg, __u32 &width, __u32 &height, __u32 &pixelformat,
 			break;
 		case 2:
 			be_pixfmt = strlen(value) == 7 && !memcmp(value + 4, "-BE", 3);
-			if (be_pixfmt)
-				value[4] = 0;
-			if (strlen(value) == 4) {
+			if (be_pixfmt || strlen(value) == 4) {
 				pixelformat =
 					v4l2_fourcc(value[0], value[1],
-							value[2], value[3]);
+						    value[2], value[3]);
 				if (be_pixfmt)
-					pixelformat |= 1 << 31;
-			} else {
+					pixelformat |= 1U << 31;
+			} else if (isdigit(value[0])) {
 				pixelformat = strtol(value, 0L, 0);
+			} else {
+				fprintf(stderr, "The pixelformat '%s' is invalid\n", value);
+				exit(1);
 			}
 			fmts |= FmtPixelFormat;
 			break;
@@ -829,8 +852,8 @@ int parse_selection_flags(const char *s)
 
 void print_selection(const struct v4l2_selection &sel)
 {
-	printf("Selection: %s, Left %d, Top %d, Width %d, Height %d, Flags: %s\n",
-			seltarget2s(sel.target).c_str(),
+	printf("Selection %s: %s, Left %d, Top %d, Width %d, Height %d, Flags: %s\n",
+			buftype2s(sel.type).c_str(), seltarget2s(sel.target).c_str(),
 			sel.r.left, sel.r.top, sel.r.width, sel.r.height,
 			selflags2s(sel.flags).c_str());
 }
@@ -927,42 +950,171 @@ static __u32 parse_event(const char *e, const char **name)
 	return event;
 }
 
+bool valid_pixel_format(int fd, __u32 pixelformat, bool output, bool mplane)
+{
+	struct v4l2_fmtdesc fmt;
+
+	fmt.index = 0;
+	if (output)
+		fmt.type = mplane ? V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE :
+			V4L2_BUF_TYPE_VIDEO_OUTPUT;
+	else
+		fmt.type = mplane ? V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE :
+			V4L2_BUF_TYPE_VIDEO_CAPTURE;
+
+	while (!ioctl(fd, VIDIOC_ENUM_FMT, &fmt)) {
+		if (fmt.pixelformat == pixelformat)
+			return true;
+		fmt.index++;
+	}
+	return false;
+}
+
 __u32 find_pixel_format(int fd, unsigned index, bool output, bool mplane)
 {
 	struct v4l2_fmtdesc fmt;
 
 	fmt.index = index;
 	if (output)
-		fmt.type = mplane ?  V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE :
+		fmt.type = mplane ? V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE :
 			V4L2_BUF_TYPE_VIDEO_OUTPUT;
 	else
-		fmt.type = mplane ?  V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE :
+		fmt.type = mplane ? V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE :
 			V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
-	if (doioctl(fd, VIDIOC_ENUM_FMT, &fmt))
+	if (ioctl(fd, VIDIOC_ENUM_FMT, &fmt))
 		return 0;
 	return fmt.pixelformat;
 }
 
+static int open_media_bus_info(const std::string &bus_info)
+{
+	DIR *dp;
+	struct dirent *ep;
+
+	dp = opendir("/dev");
+	if (dp == NULL)
+		return -1;
+
+	while ((ep = readdir(dp))) {
+		const char *name = ep->d_name;
+
+		if (!memcmp(name, "media", 5) && isdigit(name[5])) {
+			struct media_device_info mdi;
+			std::string devname = std::string("/dev/") + name;
+
+			int fd = open(devname.c_str(), O_RDWR);
+			if (fd < 0)
+				continue;
+			if (!ioctl(fd, MEDIA_IOC_DEVICE_INFO, &mdi) &&
+			    bus_info == mdi.bus_info) {
+				closedir(dp);
+				return fd;
+			}
+			close(fd);
+		}
+	}
+	closedir(dp);
+	return -1;
+}
+
+static const char *make_devname(const char *device, const char *devname,
+				const std::string &media_bus_info)
+{
+	if (device[0] >= '0' && device[0] <= '9' && strlen(device) <= 3) {
+		static char newdev[32];
+
+		sprintf(newdev, "/dev/%s%s", devname, device);
+		return newdev;
+	}
+	if (media_bus_info.empty())
+		return device;
+	int media_fd = open_media_bus_info(media_bus_info);
+	if (media_fd < 0)
+		return device;
+
+	media_v2_topology topology;
+	memset(&topology, 0, sizeof(topology));
+	if (ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology)) {
+		close(media_fd);
+		return device;
+	}
+
+	media_v2_entity *ents = new media_v2_entity[topology.num_entities];
+	topology.ptr_entities = (__u64)ents;
+	media_v2_link *links = new media_v2_link[topology.num_links];
+	topology.ptr_links = (__u64)links;
+	media_v2_interface *ifaces = new media_v2_interface[topology.num_interfaces];
+	topology.ptr_interfaces = (__u64)ifaces;
+
+	unsigned i, ent_id, iface_id = 0;
+
+	if (ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology))
+		goto err;
+
+	if (device[0] == '0' && device[1] == 'x')
+		iface_id = strtoul(device, NULL, 16);
+
+	if (!iface_id) {
+		for (i = 0; i < topology.num_entities; i++)
+			if (!strcmp(ents[i].name, device))
+				break;
+		if (i >= topology.num_entities)
+			goto err;
+		ent_id = ents[i].id;
+		for (i = 0; i < topology.num_links; i++)
+			if (links[i].sink_id == ent_id &&
+			    (links[i].flags & MEDIA_LNK_FL_LINK_TYPE) ==
+			    MEDIA_LNK_FL_INTERFACE_LINK)
+				break;
+		if (i >= topology.num_links)
+			goto err;
+		iface_id = links[i].source_id;
+	}
+	for (i = 0; i < topology.num_interfaces; i++)
+		if (ifaces[i].id == iface_id)
+			break;
+	if (i >= topology.num_interfaces)
+		goto err;
+
+	static char newdev[32];
+	sprintf(newdev, "/dev/char/%d:%d",
+		ifaces[i].devnode.major, ifaces[i].devnode.minor);
+	device = newdev;
+	
+err:
+	delete [] ents;
+	delete [] links;
+	delete [] ifaces;
+	close(media_fd);
+	return device;
+}
+
 int main(int argc, char **argv)
 {
 	int i;
 	cv4l_fd c_fd;
 	cv4l_fd c_out_fd;
+	cv4l_fd c_exp_fd;
 	int fd = -1;
 	int out_fd = -1;
+	int exp_fd = -1;
 	int media_fd = -1;
 	bool is_subdev = false;
+	std::string media_bus_info;
 
 	/* command args */
 	int ch;
 	const char *device = "/dev/video0";	/* -d device */
 	const char *out_device = NULL;
+	const char *export_device = NULL;
 	struct v4l2_capability vcap;	/* list_cap */
 	__u32 wait_for_event = 0;	/* wait for this event */
 	const char *wait_event_id = NULL;
 	__u32 poll_for_event = 0;	/* poll for this event */
 	const char *poll_event_id = NULL;
+	__u32 epoll_for_event = 0;	/* epoll for this event */
+	const char *epoll_event_id = NULL;
 	unsigned secs = 0;
 	char short_options[26 * 2 * 3 + 1];
 	int idx = 0;
@@ -1056,22 +1208,16 @@ int main(int argc, char **argv)
 			usage_all();
 			return 0;
 		case OptSetDevice:
-			device = optarg;
-			if (device[0] >= '0' && device[0] <= '9' && strlen(device) <= 3) {
-				static char newdev[20];
-
-				sprintf(newdev, "/dev/video%s", device);
-				device = newdev;
-			}
+			device = make_devname(optarg, "video", media_bus_info);
 			break;
 		case OptSetOutDevice:
-			out_device = optarg;
-			if (out_device[0] >= '0' && out_device[0] <= '9' && strlen(out_device) <= 3) {
-				static char newdev[20];
-
-				sprintf(newdev, "/dev/video%s", out_device);
-				out_device = newdev;
-			}
+			out_device = make_devname(optarg, "video", media_bus_info);
+			break;
+		case OptSetExportDevice:
+			export_device = make_devname(optarg, "video", media_bus_info);
+			break;
+		case OptMediaBusInfo:
+			media_bus_info = optarg;
 			break;
 		case OptWaitForEvent:
 			wait_for_event = parse_event(optarg, &wait_event_id);
@@ -1083,6 +1229,11 @@ int main(int argc, char **argv)
 			if (poll_for_event == 0)
 				return 1;
 			break;
+		case OptEPollForEvent:
+			epoll_for_event = parse_event(optarg, &epoll_event_id);
+			if (epoll_for_event == 0)
+				return 1;
+			break;
 		case OptSleep:
 			secs = strtoul(optarg, 0L, 0);
 			break;
@@ -1097,7 +1248,7 @@ int main(int argc, char **argv)
 			common_usage();
 			return 1;
 		default:
-			common_cmd(ch, optarg);
+			common_cmd(media_bus_info, ch, optarg);
 			tuner_cmd(ch, optarg);
 			io_cmd(ch, optarg);
 			stds_cmd(ch, optarg);
@@ -1155,6 +1306,7 @@ int main(int argc, char **argv)
 		options[OptUseWrapper] = 0;
 	c_fd.s_direct(!options[OptUseWrapper]);
 	c_out_fd.s_direct(!options[OptUseWrapper]);
+	c_exp_fd.s_direct(!options[OptUseWrapper]);
 
 	if (is_subdev)
 		fd = c_fd.subdev_open(device);
@@ -1168,7 +1320,6 @@ int main(int argc, char **argv)
 	}
 	verbose = options[OptVerbose];
 	c_fd.s_trace(options[OptSilent] ? 0 : (verbose ? 2 : 1));
-	c_out_fd.s_trace(options[OptSilent] ? 0 : (verbose ? 2 : 1));
 
 	if (!is_subdev && doioctl(fd, VIDIOC_QUERYCAP, &vcap)) {
 		fprintf(stderr, "%s: not a v4l2 node\n", device);
@@ -1198,6 +1349,7 @@ int main(int argc, char **argv)
 					strerror(errno));
 			exit(1);
 		}
+		c_out_fd.s_trace(options[OptSilent] ? 0 : (verbose ? 2 : 1));
 		if (doioctl(out_fd, VIDIOC_QUERYCAP, &vcap)) {
 			fprintf(stderr, "%s: not a v4l2 node\n", out_device);
 			exit(1);
@@ -1209,6 +1361,20 @@ int main(int argc, char **argv)
 				V4L2_PIX_FMT_PRIV_MAGIC : 0;
 	}
 
+	if (export_device) {
+		exp_fd = c_exp_fd.open(export_device);
+		if (exp_fd < 0) {
+			fprintf(stderr, "Failed to open %s: %s\n", export_device,
+					strerror(errno));
+			exit(1);
+		}
+		c_exp_fd.s_trace(options[OptSilent] ? 0 : (verbose ? 2 : 1));
+		if (doioctl(exp_fd, VIDIOC_QUERYCAP, &vcap)) {
+			fprintf(stderr, "%s: not a v4l2 node\n", export_device);
+			exit(1);
+		}
+	}
+
 	common_process_controls(c_fd);
 
 	if (wait_for_event == V4L2_EVENT_CTRL && wait_event_id)
@@ -1221,10 +1387,14 @@ int main(int argc, char **argv)
 			fprintf(stderr, "unknown control '%s'\n", poll_event_id);
 			exit(1);
 		}
+	if (epoll_for_event == V4L2_EVENT_CTRL && epoll_event_id)
+		if (!common_find_ctrl_id(epoll_event_id)) {
+			fprintf(stderr, "unknown control '%s'\n", epoll_event_id);
+			exit(1);
+		}
 
 	if (options[OptAll]) {
 		options[OptGetVideoFormat] = 1;
-		options[OptGetCrop] = 1;
 		options[OptGetVideoOutFormat] = 1;
 		options[OptGetDriverInfo] = 1;
 		options[OptGetInput] = 1;
@@ -1244,6 +1414,8 @@ int main(int argc, char **argv)
 		options[OptGetSlicedVbiOutFormat] = 1;
 		options[OptGetSdrFormat] = 1;
 		options[OptGetSdrOutFormat] = 1;
+		options[OptGetMetaFormat] = 1;
+		options[OptGetMetaOutFormat] = 1;
 		options[OptGetFBuf] = 1;
 		options[OptGetCropCap] = 1;
 		options[OptGetOutputCropCap] = 1;
@@ -1253,7 +1425,7 @@ int main(int argc, char **argv)
 		options[OptGetPriority] = 1;
 		options[OptGetSelection] = 1;
 		options[OptGetOutputSelection] = 1;
-		options[OptListCtrls] = 1;
+		options[OptListCtrlsMenus] = 1;
 		options[OptSilent] = 1;
 	}
 
@@ -1317,7 +1489,7 @@ int main(int argc, char **argv)
 
 	/* Special case: handled last */
 
-	streaming_set(c_fd, c_out_fd);
+	streaming_set(c_fd, c_out_fd, c_exp_fd);
 
 	if (options[OptWaitForEvent]) {
 		struct v4l2_event_subscription sub;
@@ -1369,6 +1541,46 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (options[OptEPollForEvent]) {
+		struct epoll_event epoll_ev;
+		int epollfd = -1;
+		struct v4l2_event_subscription sub;
+		struct v4l2_event ev;
+
+		epollfd = epoll_create1(0);
+		epoll_ev.events = EPOLLPRI;
+		epoll_ev.data.fd = fd;
+
+		memset(&sub, 0, sizeof(sub));
+		sub.flags = V4L2_EVENT_SUB_FL_SEND_INITIAL;
+		sub.type = epoll_for_event;
+		if (epoll_for_event == V4L2_EVENT_CTRL)
+			sub.id = common_find_ctrl_id(epoll_event_id);
+		else if (epoll_for_event == V4L2_EVENT_SOURCE_CHANGE)
+			sub.id = strtoul(epoll_event_id, 0L, 0);
+		if (!doioctl(fd, VIDIOC_SUBSCRIBE_EVENT, &sub)) {
+			__u32 seq = 0;
+
+			fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_NONBLOCK);
+			epoll_ctl(epollfd, EPOLL_CTL_ADD, fd, &epoll_ev);
+			while (1) {
+				int res;
+
+				res = epoll_wait(epollfd, &epoll_ev, 1, -1);
+				if (res <= 0)
+					break;
+				if (!doioctl(fd, VIDIOC_DQEVENT, &ev)) {
+					print_event(&ev);
+					if (ev.sequence > seq)
+						printf("\tMissed %d events\n",
+							ev.sequence - seq);
+					seq = ev.sequence + 1;
+				}
+			}
+		}
+		close(epollfd);
+	}
+
 	if (options[OptSleep]) {
 		sleep(secs);
 		printf("Test VIDIOC_QUERYCAP:\n");
@@ -1381,6 +1593,8 @@ int main(int argc, char **argv)
 	c_fd.close();
 	if (out_device)
 		c_out_fd.close();
+	if (export_device)
+		c_exp_fd.close();
 	if (media_fd >= 0)
 		close(media_fd);
 
