@@ -57,6 +57,7 @@ enum Option {
 	OptSetDevice = 'd',
 	OptGetDriverInfo = 'D',
 	OptSetOutDevice = 'e',
+	OptSetExportDevice = 'E',
 	OptGetFreq = 'F',
 	OptSetFreq = 'f',
 	OptHelp = 'h',
@@ -81,6 +82,7 @@ enum Option {
 	OptUseWrapper = 'w',
 	OptGetVideoOutFormat = 'X',
 	OptSetVideoOutFormat = 'x',
+	OptMediaBusInfo = 'z',
 
 	OptGetSlicedVbiOutFormat = 128,
 	OptGetOverlayFormat,
@@ -89,6 +91,7 @@ enum Option {
 	OptGetSdrFormat,
 	OptGetSdrOutFormat,
 	OptGetMetaFormat,
+	OptGetMetaOutFormat,
 	OptGetSubDevFormat,
 	OptSetSlicedVbiOutFormat,
 	OptSetOverlayFormat,
@@ -97,6 +100,7 @@ enum Option {
 	OptSetSdrFormat,
 	OptSetSdrOutFormat,
 	OptSetMetaFormat,
+	OptSetMetaOutFormat,
 	OptSetSubDevFormat,
 	OptTryVideoOutFormat,
 	OptTrySlicedVbiOutFormat,
@@ -108,6 +112,7 @@ enum Option {
 	OptTrySdrFormat,
 	OptTrySdrOutFormat,
 	OptTryMetaFormat,
+	OptTryMetaOutFormat,
 	OptTrySubDevFormat,
 	OptAll,
 	OptListStandards,
@@ -122,6 +127,7 @@ enum Option {
 	OptListOutFormats,
 	OptListOutFormatsExt,
 	OptListMetaFormats,
+	OptListMetaOutFormats,
 	OptListSubDevMBusCodes,
 	OptListSubDevFrameSizes,
 	OptListSubDevFrameIntervals,
@@ -177,6 +183,7 @@ enum Option {
 	OptSetOutputParm,
 	OptQueryStandard,
 	OptPollForEvent,
+	OptEPollForEvent,
 	OptWaitForEvent,
 	OptGetPriority,
 	OptSetPriority,
@@ -205,6 +212,7 @@ enum Option {
 	OptListBuffersSdr,
 	OptListBuffersSdrOut,
 	OptListBuffersMeta,
+	OptListBuffersMetaOut,
 	OptStreamCount,
 	OptStreamSkip,
 	OptStreamLoop,
@@ -215,6 +223,7 @@ enum Option {
 	OptStreamToHdr,
 	OptStreamToHost,
 	OptStreamLossless,
+	OptStreamBufCaps,
 	OptStreamMmap,
 	OptStreamUser,
 	OptStreamDmaBuf,
@@ -234,6 +243,7 @@ enum Option {
 	OptStreamOutRGBLimitedRange,
 	OptStreamOutPixelAspect,
 	OptStreamOutVideoAspect,
+	OptStreamOutBufCaps,
 	OptStreamOutMmap,
 	OptStreamOutUser,
 	OptStreamOutDmaBuf,
@@ -304,17 +314,29 @@ int parse_selection_target(const char *s, unsigned int &target);
 int parse_selection_flags(const char *s);
 void print_selection(const struct v4l2_selection &sel);
 __u32 find_pixel_format(int fd, unsigned index, bool output, bool mplane);
+bool valid_pixel_format(int fd, __u32 pixelformat, bool output, bool mplane);
 void print_frmsize(const struct v4l2_frmsizeenum &frmsize, const char *prefix);
 void print_frmival(const struct v4l2_frmivalenum &frmival, const char *prefix);
 void printfmt(int fd, const struct v4l2_format &vfmt);
 void print_video_formats(cv4l_fd &fd, __u32 type);
 void print_video_formats_ext(cv4l_fd &fd, __u32 type);
 
+static inline bool subscribe_event(cv4l_fd &fd, __u32 type)
+{
+	struct v4l2_event_subscription sub;
+
+	memset(&sub, 0, sizeof(sub));
+	sub.type = type;
+
+	cv4l_disable_trace dt(fd);
+	return !fd.subscribe_event(sub);
+}
+
 #define doioctl(n, r, p) doioctl_name(n, r, p, #r)
 
 // v4l2-ctl-common.cpp
 void common_usage(void);
-void common_cmd(int ch, char *optarg);
+void common_cmd(const std::string &media_bus_info, int ch, char *optarg);
 void common_set(cv4l_fd &fd);
 void common_get(cv4l_fd &fd);
 void common_list(cv4l_fd &fd);
@@ -345,6 +367,7 @@ void stds_list(cv4l_fd &fd);
 // v4l2-ctl-vidcap.cpp
 void vidcap_usage(void);
 void vidcap_cmd(int ch, char *optarg);
+int vidcap_get_and_update_fmt(cv4l_fd &_fd, struct v4l2_format &vfmt);
 void vidcap_set(cv4l_fd &fd);
 void vidcap_get(cv4l_fd &fd);
 void vidcap_list(cv4l_fd &fd);
@@ -408,7 +431,7 @@ void misc_get(cv4l_fd &fd);
 // v4l2-ctl-streaming.cpp
 void streaming_usage(void);
 void streaming_cmd(int ch, char *optarg);
-void streaming_set(cv4l_fd &fd, cv4l_fd &out_fd);
+void streaming_set(cv4l_fd &fd, cv4l_fd &out_fd, cv4l_fd &exp_fd);
 void streaming_list(cv4l_fd &fd, cv4l_fd &out_fd);
 
 // v4l2-ctl-edid.cpp
